@@ -6,6 +6,7 @@
 #include "cache.h"
 #include "colormanager.h"
 #include "configcontainer.h"
+#include "configparser.h"
 #include "feedcontainer.h"
 #include "filtercontainer.h"
 #include "fslock.h"
@@ -23,8 +24,6 @@ class CliArgsParser;
 class ConfigPaths;
 class View;
 
-class CurlHandle;
-
 class Controller {
 public:
 	Controller(ConfigPaths& configpaths);
@@ -49,11 +48,13 @@ public:
 	{
 		rsscache->mark_all_read(feed);
 	}
+	void mark_all_read(const std::vector<std::string>& item_guids);
 	bool get_refresh_on_start() const
 	{
 		return refresh_on_start;
 	}
-	void enqueue_url(std::shared_ptr<RssItem> item, std::shared_ptr<RssFeed> feed);
+	EnqueueResult enqueue_url(std::shared_ptr<RssItem> item,
+		std::shared_ptr<RssFeed> feed);
 
 	void reload_urls_file();
 	void edit_urls_file();
@@ -72,7 +73,7 @@ public:
 
 	void load_configfile(const std::string& filename);
 
-	void dump_config(const std::string& filename);
+	void dump_config(const std::string& filename) const;
 
 	void update_flags(std::shared_ptr<RssItem> item);
 
@@ -86,6 +87,11 @@ public:
 		unsigned int pos,
 		bool unattended);
 
+	ConfigContainer* get_config()
+	{
+		return &cfg;
+	}
+
 	RssIgnores* get_ignores()
 	{
 		return &ign;
@@ -112,8 +118,8 @@ public:
 	}
 
 private:
-	void import_opml(const std::string& opmlFile, const std::string& urlFile);
-	void export_opml();
+	int import_opml(const std::string& opmlFile, const std::string& urlFile);
+	void export_opml(bool version2);
 	void rec_find_rss_outlines(xmlNode* node, std::string tag);
 	int execute_commands(const std::vector<std::string>& cmds);
 
@@ -134,7 +140,7 @@ private:
 	RegexManager rxman;
 	RemoteApi* api;
 
-	std::unique_ptr<FsLock> fslock;
+	FsLock fslock;
 
 	ConfigPaths& configpaths;
 
