@@ -5,9 +5,10 @@
 #include <grp.h>
 
 #include "configcontainer.h"
+#include "file_system.h"
+#include "formaction.h"
 #include "listformatter.h"
 #include "listwidget.h"
-#include "formaction.h"
 
 namespace newsboat {
 
@@ -17,12 +18,7 @@ public:
 	~DirBrowserFormAction() override;
 	void prepare() override;
 	void init() override;
-	KeyMapHintEntry* get_keymap_hint() override;
-
-	void set_dir(const std::string& d)
-	{
-		dir = d;
-	}
+	const std::vector<KeyMapHintEntry>& get_keymap_hint() const override;
 
 	std::string id() const override
 	{
@@ -30,6 +26,12 @@ public:
 	}
 	std::string title() override;
 
+protected:
+	std::string main_widget() const override
+	{
+		return "files";
+	}
+
 private:
 	bool process_operation(Operation op,
 		bool automatic = false,
@@ -37,17 +39,13 @@ private:
 	void update_title(const std::string& working_directory);
 
 	void add_directory(ListFormatter& listfmt,
-		std::vector<std::string>& id_at_position,
+		std::vector<file_system::FileSystemEntry>& id_at_position,
 		std::string dirname);
-	std::string get_rwx(unsigned short val);
-	std::vector<std::string> id_at_position;
+	std::vector<file_system::FileSystemEntry> id_at_position;
 
-	std::string get_owner(uid_t uid);
-	std::string get_group(gid_t gid);
-	std::string get_formatted_dirname(std::string dirname, char ftype, mode_t mode);
+	std::string get_formatted_dirname(std::string dirname, mode_t mode);
 
 	std::string cwd;
-	std::string dir;
 
 	ListWidget files_list;
 };
