@@ -27,7 +27,7 @@ public:
 	void init() override;
 	void set_feedlist(std::vector<std::shared_ptr<RssFeed>>& feeds);
 	void update_visible_feeds(std::vector<std::shared_ptr<RssFeed>>& feeds);
-	KeyMapHintEntry* get_keymap_hint() override;
+	const std::vector<KeyMapHintEntry>& get_keymap_hint() const override;
 	std::shared_ptr<RssFeed> get_feed();
 
 	std::string id() const override
@@ -48,18 +48,27 @@ public:
 
 	void mark_pos_if_visible(unsigned int pos);
 
+protected:
+	std::string main_widget() const override
+	{
+		return "feeds";
+	}
+
 private:
 	void register_format_styles();
 
 	void update_form_title(unsigned int width);
 
 	unsigned int count_unread_feeds();
+	unsigned int count_unread_articles();
 
 	int get_pos(unsigned int realidx);
 	bool process_operation(Operation op,
 		bool automatic = false,
 		std::vector<std::string>* args = nullptr) override;
 
+	bool open_position_in_browser(unsigned int pos, bool interactive) const;
+
 	void goto_feed(const std::string& str);
 
 	void save_filterpos();
@@ -68,7 +77,8 @@ private:
 	void op_start_search();
 
 	void handle_cmdline_num(unsigned int idx);
-
+	void handle_tag(const std::string& params);
+	void handle_goto(const std::string& param);
 	void set_pos();
 
 	std::string get_title(std::shared_ptr<RssFeed> feed);
@@ -83,7 +93,8 @@ private:
 	std::string tag;
 
 	Matcher matcher;
-	bool apply_filter;
+	bool filter_active;
+	void apply_filter(const std::string& filter_query);
 
 	History filterhistory;
 
@@ -92,7 +103,7 @@ private:
 
 	RegexManager& rxman;
 
-	FilterContainer& filters;
+	FilterContainer& filter_container;
 
 	nonstd::optional<FeedSortStrategy> old_sort_strategy;
 
