@@ -5,9 +5,10 @@
 #include <grp.h>
 
 #include "configcontainer.h"
+#include "file_system.h"
+#include "formaction.h"
 #include "listformatter.h"
 #include "listwidget.h"
-#include "formaction.h"
 
 namespace newsboat {
 
@@ -17,12 +18,8 @@ public:
 	~FileBrowserFormAction() override;
 	void prepare() override;
 	void init() override;
-	KeyMapHintEntry* get_keymap_hint() override;
+	const std::vector<KeyMapHintEntry>& get_keymap_hint() const override;
 
-	void set_dir(const std::string& d)
-	{
-		dir = d;
-	}
 	void set_default_filename(const std::string& fn)
 	{
 		default_filename = fn;
@@ -34,6 +31,12 @@ public:
 	}
 	std::string title() override;
 
+protected:
+	std::string main_widget() const override
+	{
+		return "filename";
+	}
+
 private:
 	bool process_operation(Operation op,
 		bool automatic = false,
@@ -41,21 +44,15 @@ private:
 	void update_title(const std::string& working_directory);
 
 	void add_file(ListFormatter& listfmt,
-		std::vector<std::string>& id_at_position,
+		std::vector<file_system::FileSystemEntry>& id_at_position,
 		std::string filename);
 	std::string get_filename_suggestion(const std::string& s);
-	std::string get_rwx(unsigned short val);
-	std::vector<std::string> id_at_position;
+	std::vector<file_system::FileSystemEntry> id_at_position;
 
-	char get_filetype(mode_t mode);
-	std::string get_owner(uid_t uid);
-	std::string get_group(gid_t gid);
-	std::string get_formatted_filename(std::string filename, char ftype,
-		mode_t mode);
+	std::string get_formatted_filename(std::string filename, mode_t mode);
 
 	bool quit;
 	std::string cwd;
-	std::string dir;
 	std::string default_filename;
 
 	ListWidget files_list;
