@@ -7,6 +7,7 @@
 
 #include "history.h"
 #include "keymap.h"
+#include "listwidget.h"
 #include "stflpp.h"
 
 namespace newsboat {
@@ -67,14 +68,13 @@ public:
 
 	void start_cmdline(std::string default_value = "");
 
-	std::string get_qna_response(unsigned int i)
-	{
-		return (qna_responses.size() >= (i + 1)) ? qna_responses[i]
-			: "";
-	}
 	void start_qna(const std::vector<QnaPair>& prompts,
 		Operation finish_op,
 		History* h = nullptr);
+	void finish_qna_question();
+	void cancel_qna();
+	void qna_next_history();
+	void qna_previous_history();
 
 	void set_parent_formaction(std::shared_ptr<FormAction> fa)
 	{
@@ -107,6 +107,10 @@ protected:
 		std::vector<std::string>* args = nullptr) = 0;
 	virtual void set_keymap_hints();
 
+	/// The name of the "main" STFL widget, i.e. the one that should be focused
+	/// by default.
+	virtual std::string main_widget() const = 0;
+
 	void start_bookmark_qna(const std::string& default_title,
 		const std::string& default_url,
 		const std::string& default_feed_title);
@@ -116,6 +120,8 @@ protected:
 
 	void handle_parsed_command(const Command& command);
 
+	bool handle_list_operations(ListWidget& list, Operation op);
+
 	View* v;
 	ConfigContainer* cfg;
 	Stfl::Form f;
