@@ -5,6 +5,8 @@
 
 #include "3rd-party/optional.hpp"
 
+#include "configcontainer.h"
+#include "fmtstrformatter.h"
 #include "history.h"
 #include "listformaction.h"
 #include "listformatter.h"
@@ -13,6 +15,8 @@
 
 namespace newsboat {
 
+class RssItem;
+
 typedef std::pair<std::shared_ptr<RssItem>, unsigned int> ItemPtrPosPair;
 
 enum class InvalidationMode { NONE, PARTIAL, COMPLETE };
@@ -58,15 +62,6 @@ public:
 
 	void finished_qna(Operation op) override;
 
-	void set_show_searchresult(bool b)
-	{
-		show_searchresult = b;
-	}
-	void set_searchphrase(const std::string& s)
-	{
-		search_phrase = s;
-	}
-
 	void invalidate_list()
 	{
 		invalidation_mode = InvalidationMode::COMPLETE;
@@ -74,23 +69,52 @@ public:
 
 	void restore_selected_position();
 
+protected:
+	bool process_operation(Operation op,
+		bool automatic = false,
+		std::vector<std::string>* args = nullptr) override;
+	std::string main_widget() const override
+	{
+		return "items";
+	}
+
+	void invalidate(const unsigned int invalidated_pos)
+	{
+		if (invalidation_mode == InvalidationMode::COMPLETE) {
+			return;
+		}
+
+		invalidation_mode = InvalidationMode::PARTIAL;
+		invalidated_itempos.push_back(invalidated_pos);
+	}
+
+	virtual FmtStrFormatter setup_head_formatter(const std::string& s,
+		unsigned int unread,
+		unsigned int total,
+		const std::string& url);
+
+	std::vector<ItemPtrPosPair> visible_items;
+	int old_itempos;
+	std::shared_ptr<RssFeed> feed;
+
+	Matcher matcher;
+	bool filter_active;
+	void apply_filter(const std::string& filter_query);
+
 private:
 	void register_format_styles();
 
 	void do_update_visible_items();
 	void draw_items();
 
-	bool process_operation(Operation op,
-		bool automatic = false,
-		std::vector<std::string>* args = nullptr) override;
-
 	bool open_position_in_browser(unsigned int pos,
 		bool interactive) const;
 
-	void set_head(const std::string& s,
+	virtual void set_head(const std::string& s,
 		unsigned int unread,
 		unsigned int total,
 		const std::string& url);
+
 	int get_pos(unsigned int idx);
 
 	void save_article(const std::string& filename,
@@ -110,16 +134,6 @@ private:
 
 	void prepare_set_filterpos();
 
-	void invalidate(const unsigned int invalidated_pos)
-	{
-		if (invalidation_mode == InvalidationMode::COMPLETE) {
-			return;
-		}
-
-		invalidation_mode = InvalidationMode::PARTIAL;
-		invalidated_itempos.push_back(invalidated_pos);
-	}
-
 	std::string item2formatted_line(const ItemPtrPosPair& item,
 		const unsigned int width,
 		const std::string& itemlist_format,
@@ -127,13 +141,9 @@ private:
 
 	void goto_item(const std::string& title);
 
+	void handle_op_saveall();
+
 	unsigned int pos;
-	std::shared_ptr<RssFeed> feed;
-	bool apply_filter;
-	Matcher matcher;
-	std::vector<ItemPtrPosPair> visible_items;
-	bool show_searchresult;
-	std::string search_phrase;
 
 	History filterhistory;
 
@@ -145,7 +155,6 @@ private:
 	RegexManager& rxman;
 
 	unsigned int old_width;
-	int old_itempos;
 	nonstd::optional<ArticleSortStrategy> old_sort_strategy;
 
 	InvalidationMode invalidation_mode;
@@ -153,9 +162,7 @@ private:
 
 	ListFormatter listfmt;
 	Cache* rsscache;
-	FilterContainer& filters;
-
-	void handle_op_saveall();
+	FilterContainer& filter_container;
 };
 
 } // namespace newsboat
