@@ -17,6 +17,7 @@ public:
 	std::vector<TaggedFeedUrl> get_subscribed_urls() override;
 	bool mark_all_read(const std::string& feedurl) override;
 	bool mark_article_read(const std::string& guid, bool read) override;
+	bool mark_articles_read(const std::vector<std::string>& guids) override;
 	bool update_article_flags(const std::string& oldflags,
 		const std::string& newflags,
 		const std::string& guid) override;
@@ -29,7 +30,6 @@ private:
 	bool query(const std::string& query,
 		json_object** result = nullptr,
 		const std::string& post = "");
-	std::string md5(const std::string& str);
 	std::string auth;
 	std::string server;
 	FeedMap known_feeds;
