@@ -10,13 +10,18 @@
 #include <utility>
 #include <vector>
 
+#include "3rd-party/expected.hpp"
 #include "3rd-party/optional.hpp"
 
 #include "configcontainer.h"
 #include "logger.h"
 
+#include "libnewsboat-ffi/src/utils.rs.h"
+
 namespace newsboat {
 
+class CurlHandle;
+
 // Forward declaration for a type from htmlrenderer.h
 enum class LinkType;
 
@@ -45,27 +50,29 @@ std::string consolidate_whitespace(const std::string& str);
 
 std::string translit(const std::string& tocode,
 	const std::string& fromcode);
-std::string convert_text(const std::string& text,
-	const std::string& tocode,
-	const std::string& fromcode);
 
 /// Converts input string from UTF-8 to the locale's encoding (as detected by
 /// nl_langinfo(CODESET)).
 std::string utf8_to_locale(const std::string& text);
+/// Converts input string from the locale's encoding (as detected by
+/// nl_langinfo(CODESET)) to UTF-8.
+std::string locale_to_utf8(const std::string& text);
 
 std::string get_command_output(const std::string& cmd);
-void extract_filter(const std::string& line,
-	std::string& filter,
-	std::string& url);
 std::string http_method_str(const HTTPMethod method);
+std::string link_type_str(LinkType type);
+
 std::string retrieve_url(const std::string& url,
 	ConfigContainer* cfgcont = nullptr,
 	const std::string& authinfo = "",
 	const std::string* body = nullptr,
-	const HTTPMethod method = HTTPMethod::GET,
-	CURL* easyhandle = nullptr);
-void run_command(const std::string& cmd,
-	const std::string& param); // used for notifications only
+	const HTTPMethod method = HTTPMethod::GET);
+std::string retrieve_url(const std::string& url,
+	CurlHandle& easyhandle,
+	ConfigContainer* cfgcont = nullptr,
+	const std::string& authinfo = "",
+	const std::string* body = nullptr,
+	const HTTPMethod method = HTTPMethod::GET);
 std::string run_program(const char* argv[], const std::string& input);
 
 std::string resolve_tilde(const std::string&);
@@ -73,6 +80,8 @@ std::string resolve_relative(const std::string&, const std::string&);
 std::string replace_all(std::string str,
 	const std::string& from,
 	const std::string& to);
+std::string replace_all(const std::string& str,
+	const std::vector<std::pair<std::string, std::string>> from_to_pairs);
 
 std::wstring str2wstr(const std::string& str);
 std::string wstr2str(const std::wstring& wstr);
@@ -84,9 +93,6 @@ std::string absolute_url(const std::string& url,
 
 std::string get_useragent(ConfigContainer* cfgcont);
 
-size_t strwidth(const std::string& s);
-size_t strwidth_stfl(const std::string& str);
-
 std::string substr_with_width(const std::string& str,
 	const size_t max_width);
 
@@ -96,9 +102,6 @@ std::string substr_with_width_stfl(const std::string& str,
 unsigned int to_u(const std::string& str,
 	const unsigned int default_value = 0);
 
-bool is_valid_color(const std::string& color);
-bool is_valid_attribute(const std::string& attrib);
-
 std::vector<std::pair<unsigned int, unsigned int>> partition_indexes(
 		unsigned int start,
 		unsigned int end,
@@ -117,20 +120,11 @@ void trim(std::string& str);
 
 std::string quote(const std::string& str);
 
-unsigned int get_random_value(unsigned int max);
-
 std::string quote_if_necessary(const std::string& str);
 
-void set_common_curl_options(CURL* handle, ConfigContainer* cfg);
+void set_common_curl_options(CurlHandle& handle, ConfigContainer* cfg);
 
 curl_proxytype get_proxy_type(const std::string& type);
-unsigned long get_auth_method(const std::string& type);
-
-bool is_special_url(const std::string& url);
-bool is_http_url(const std::string& url);
-bool is_query_url(const std::string& url);
-bool is_filter_url(const std::string& url);
-bool is_exec_url(const std::string& url);
 
 std::string get_content(xmlNode* node);
 std::string get_basename(const std::string& url);
@@ -138,8 +132,6 @@ std::string get_basename(const std::string& url);
 std::string unescape_url(const std::string& url);
 void initialize_ssl_implementation(void);
 
-unsigned int gentabs(const std::string& str);
-
 int mkdir_parents(const std::string& pathname,
 	mode_t mode = 0755);
 
@@ -148,9 +140,21 @@ std::string make_title(const std::string& url);
 nonstd::optional<std::uint8_t> run_interactively(const std::string& command,
 	const std::string& caller);
 
+nonstd::optional<std::uint8_t> run_non_interactively(const std::string& command,
+	const std::string& caller);
+
 std::string getcwd();
 
-int strnaturalcmp(const std::string& a, const std::string& b);
+enum class ReadTextFileErrorKind {
+	CantOpen,
+	LineError
+};
+struct ReadTextFileError {
+	ReadTextFileErrorKind kind;
+	std::string message;
+};
+nonstd::expected<std::vector<std::string>, ReadTextFileError> read_text_file(
+	const std::string& filename);
 
 void remove_soft_hyphens(std::string& text);
 
@@ -161,15 +165,19 @@ nonstd::optional<LinkType> podcast_mime_to_link_type(const std::string&
 
 std::string get_default_browser();
 
+std::string md5hash(const std::string& input);
+
 /// The tag and Git commit ID the program was built from, or a pre-defined
 /// value from config.h if there is no Git directory.
 std::string program_version();
 
-/// Newsboat's major version number.
-unsigned int newsboat_version_major();
-
 /// Threadsafe combination of strftime() and localtime()
 std::string mt_strf_localtime(const std::string& format, time_t t);
+
+/// Preserves single quotes by enclosing each word in
+/// single quotes and replaces "'" with "\'"
+/// (e.g. "it's" -> "'it'\''s'")
+std::string preserve_quotes(const std::string& s);
 }
 
 } // namespace newsboat
