@@ -2,31 +2,33 @@
 #define NEWSBOAT_VIEW_H_
 
 #include <cstdint>
-#include <list>
 #include <mutex>
 #include <string>
 #include <vector>
 
 #include "3rd-party/optional.hpp"
 
-#include "colormanager.h"
-#include "configcontainer.h"
-#include "controller.h"
-#include "dirbrowserformaction.h"
-#include "feedlistformaction.h"
-#include "filebrowserformaction.h"
 #include "htmlrenderer.h"
-#include "keymap.h"
-#include "regexmanager.h"
-#include "stflpp.h"
+#include "statusline.h"
 
 namespace newsboat {
 
+class Cache;
+class ColorManager;
+class Controller;
+class ConfigContainer;
+class FeedListFormAction;
+class FilterContainer;
+struct FilterNameExprPair;
+class FormAction;
 class ItemListFormAction;
 class ItemViewFormAction;
-class FeedListFormAction;
+class KeyMap;
+struct MacroCmd;
+class RegexManager;
+class RssFeed;
 
-class View {
+class View : public IStatus {
 public:
 	explicit View(Controller*);
 	~View();
@@ -38,8 +40,7 @@ public:
 	void update_visible_feeds(std::vector<std::shared_ptr<RssFeed>> feeds);
 	void set_keymap(KeyMap* k);
 	void set_config_container(ConfigContainer* cfgcontainer);
-	void show_error(const std::string& msg);
-	void set_status(const std::string& msg);
+	StatusLine& get_statusline();
 	Controller* get_ctrl()
 	{
 		return ctrl;
@@ -52,8 +53,12 @@ public:
 	{
 		return keys;
 	}
+	const KeyMap* get_keymap() const
+	{
+		return keys;
+	}
 	void set_tags(const std::vector<std::string>& t);
-	void push_empty_formaction();
+	void drop_queued_input();
 	void pop_current_formaction();
 	void remove_formaction(unsigned int pos);
 	void set_current_formaction(unsigned int pos);
@@ -69,6 +74,7 @@ public:
 	void push_itemview(std::shared_ptr<RssFeed> f,
 		const std::string& guid,
 		const std::string& searchphrase = "");
+	void push_empty_formaction();
 	void push_help();
 	void push_urlview(const std::vector<LinkPair>& links,
 		std::shared_ptr<RssFeed>& feed);
@@ -76,14 +82,15 @@ public:
 		const std::string& phrase = "");
 	void view_dialogs();
 
-	std::string run_filebrowser(const std::string& default_filename = "",
-		const std::string& dir = "");
-	std::string run_dirbrowser(const std::string& dir = "");
-	std::string select_tag();
+	std::string run_filebrowser(const std::string& default_filename = "");
+	std::string run_dirbrowser();
+	std::string select_tag(const std::string& current_tag);
 	std::string select_filter(
 		const std::vector<FilterNameExprPair>& filters);
 
-	nonstd::optional<std::uint8_t> open_in_browser(const std::string& url);
+	nonstd::optional<std::uint8_t> open_in_browser(const std::string& url,
+		const std::string& feedurl, const std::string& type, const std::string& title,
+		bool interactive);
 	void open_in_pager(const std::string& filename);
 
 	std::string get_filename_suggestion(const std::string& s);
@@ -131,8 +138,6 @@ public:
 	void inside_qna(bool f);
 	void inside_cmdline(bool f);
 
-	void dump_current_form();
-
 	static void ctrl_c_action(int sig);
 
 protected:
@@ -143,8 +148,9 @@ protected:
 	void handle_cmdline_completion(std::shared_ptr<FormAction> fa);
 	void clear_line(std::shared_ptr<FormAction> fa);
 	void clear_eol(std::shared_ptr<FormAction> fa);
-	void cancel_input(std::shared_ptr<FormAction> fa);
 	void delete_word(std::shared_ptr<FormAction> fa);
+	bool handle_qna_event(const std::string& event, std::shared_ptr<FormAction> fa);
+	void handle_resize();
 
 	Controller* ctrl;
 
@@ -156,12 +162,14 @@ protected:
 	unsigned int current_formaction;
 	std::shared_ptr<FeedListFormAction> feedlist_form;
 
+	void set_status(const std::string& msg) override;
+	void show_error(const std::string& msg) override;
+	StatusLine status_line;
+
 	std::vector<std::string> tags;
 
 	RegexManager& rxman;
 
-	std::map<std::string, TextStyle> text_styles;
-
 	bool is_inside_qna;
 	bool is_inside_cmdline;
 
@@ -171,6 +179,9 @@ protected:
 	FilterContainer& filters;
 	const ColorManager& colorman;
 	std::vector<std::string> suggestions;
+
+private:
+	bool try_prepare_query_feed(std::shared_ptr<RssFeed> feed);
 };
 
 } // namespace newsboat
