@@ -14,6 +14,7 @@
 #include "exception.h"
 #include "matcherexception.h"
 #include "rss/parser.h"
+#include "stflpp.h"
 #include "utils.h"
 #include "view.h"
 #include "xlicense.h"
@@ -44,6 +45,7 @@ void print_usage(const std::string& argv0, const std::string& config_path,
 
 	static const std::vector<arg> args = {
 		{'e', "export-to-opml", "", _s("export OPML feed to stdout")},
+		{'-', "export-to-opml2", "", _s("export OPML 2.0 feed including tags to stdout")},
 		{'r', "refresh-on-start", "", _s("refresh feeds on start")},
 		{'i', "import-from-opml", _s("<file>"), _s("import OPML file")},
 		{
@@ -77,9 +79,8 @@ void print_usage(const std::string& argv0, const std::string& config_path,
 			'l',
 			"log-level",
 			_s("<loglevel>"),
-			_s("write a log with a certain loglevel (valid values: "
-				"1 to "
-				"6)")
+			_s("write a log with a certain log level (valid values: 1 to 6,"
+				" for user error, critical, error, warning, info, and debug respectively)")
 		},
 		{
 			'd',
@@ -126,11 +127,11 @@ void print_usage(const std::string& argv0, const std::string& config_path,
 	std::cout << '\n';
 
 	std::cout << _("Files:") << '\n';
-	/// This is printed out by --help before the path to the config file
+	// i18n: This is printed out by --help before the path to the config file
 	const std::string tr_config = _("configuration");
-	/// This is printed out by --help before the path to the urls file
+	// i18n This is printed out by --help before the path to the urls file
 	const std::string tr_urls = _("feed URLs");
-	/// This is printed out by --help before the path to the cache file
+	// i18n This is printed out by --help before the path to the cache file
 	const std::string tr_cache = _("cache");
 	const auto widest = std::max({tr_config.length(), tr_urls.length(), tr_cache.length()});
 
@@ -160,11 +161,11 @@ void print_version(const std::string& argv0, unsigned int level)
 			<< PROGRAM_URL << std::endl;
 		ss << "Copyright (C) 2006-2015 Andreas Krennmair"
 			<< std::endl;
-		ss << "Copyright (C) 2015-2022 Alexander Batischev"
+		ss << "Copyright (C) 2015-2023 Alexander Batischev"
 			<< std::endl;
 		ss << "Copyright (C) 2006-2017 Newsbeuter contributors"
 			<< std::endl;
-		ss << "Copyright (C) 2017-2022 Newsboat contributors"
+		ss << "Copyright (C) 2017-2023 Newsboat contributors"
 			<< std::endl;
 		ss << std::endl;
 
