@@ -22,19 +22,22 @@ int main(int argc, char* argv[])
 	setlocale(LC_CTYPE, "");
 	setlocale(LC_MESSAGES, "");
 
-	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
+	bindtextdomain(PACKAGE, LOCALEDIR);
+	// Internally, Newsboat stores all strings in UTF-8, so we require gettext
+	// to return messages in that encoding.
+	bind_textdomain_codeset(PACKAGE, "UTF-8");
 
 	PbController c;
 
 	try {
 		c.initialize(argc, argv);
 
-		podboat::PbView v(&c);
-		c.set_view(&v);
+		podboat::PbView v(c);
 
-		return c.run();
+		return c.run(v);
 	} catch (const newsboat::Exception& e) {
+		Stfl::reset();
 		std::cerr << strprintf::fmt(_("Caught newsboat::Exception with "
 					"message: %s"),
 				e.what())
