@@ -6,10 +6,10 @@
 #include "exception.h"
 #include "feed.h"
 #include "item.h"
+#include "medianamespace.h"
 #include "rsspp_uris.h"
 #include "utils.h"
-
-#define MEDIA_RSS_URI "http://search.yahoo.com/mrss/"
+#include "xmlutilities.h"
 
 using namespace newsboat;
 
@@ -69,13 +69,16 @@ Item Rss09xParser::parse_item(xmlNode* itemNode)
 			it.title = get_content(node);
 			it.title_type = "text";
 		} else if (node_is(node, "link", ns)) {
-			it.link = utils::absolute_url(base, get_content(node));
+			if (it.link.empty() || !utils::is_http_url(it.link)) {
+				it.link = utils::absolute_url(base, get_content(node));
+			}
 		} else if (node_is(node, "description", ns)) {
 			it.base = get_prop(node, "base", XML_URI);
 			if (it.base.empty()) {
 				it.base = base;
 			}
 			it.description = get_content(node);
+			it.description_mime_type = "";
 		} else if (node_is(node, "encoded", CONTENT_URI)) {
 			it.content_encoded = get_content(node);
 		} else if (node_is(node, "summary", ITUNES_URI)) {
@@ -112,32 +115,14 @@ Item Rss09xParser::parse_item(xmlNode* itemNode)
 		} else if (node_is(node, "creator", DC_URI)) {
 			author = get_content(node);
 		} else if (node_is(node, "enclosure", ns)) {
-			const std::string type = get_prop(node, "type");
-			if (utils::is_valid_podcast_type(type)) {
-				it.enclosure_url = get_prop(node, "url");
-				it.enclosure_type = std::move(type);
-			}
-		} else if (node_is(node, "content", MEDIA_RSS_URI)) {
-			const std::string type = get_prop(node, "type");
-			if (utils::is_valid_podcast_type(type)) {
-				it.enclosure_url = get_prop(node, "url");
-				it.enclosure_type = std::move(type);
-			}
-		} else if (node_is(node, "group", MEDIA_RSS_URI)) {
-			for (xmlNode* mnode = node->children; mnode != nullptr;
-				mnode = mnode->next) {
-				if (node_is(mnode, "content", MEDIA_RSS_URI)) {
-					const std::string type =
-						get_prop(mnode, "type");
-					if (utils::is_valid_podcast_type(
-							type)) {
-						it.enclosure_url =
-							get_prop(mnode, "url");
-						it.enclosure_type =
-							std::move(type);
-					}
-				}
+			it.enclosures.push_back(
+			Enclosure {
+				get_prop(node, "url"),
+				get_prop(node, "type"),
 			}
+			);
+		} else if (is_media_node(node)) {
+			parse_media_node(node, it);
 		}
 	}
 
