@@ -16,6 +16,7 @@ pub mod fmtstrformatter;
 pub mod fslock;
 pub mod history;
 pub mod human_panic;
+pub mod keymap;
 pub mod logger;
 pub mod matchererror;
 pub mod scopemeasure;
@@ -36,6 +37,6 @@ pub unsafe extern "C" fn rs_cstring_free(string: *mut c_char) {
         if string.is_null() {
             return;
         }
-        CString::from_raw(string);
+        drop(CString::from_raw(string));
     })
 }
