@@ -21,7 +21,7 @@ use std::collections::BTreeMap;
 /// Since Newsboat contains a number of different format strings with different specifiers, we need
 /// an object to encapsulate the common part: the act of formatting. That way, the above example
 /// with titles and authors can be implemented very easily:
-/// ```
+/// ```no_run
 /// use libnewsboat::fmtstrformatter::*;
 ///
 /// let format = "%t (%a)";
@@ -91,7 +91,7 @@ impl FmtStrFormatter {
                 }
             };
 
-            let padding_value = &format!("{}", c);
+            let padding_value = &format!("{c}");
             let padding = padding_value.repeat(padding_width);
             result.push_str(&padding);
         };
@@ -100,7 +100,7 @@ impl FmtStrFormatter {
 
     fn format_format(&self, c: char, padding: &Padding, width: u32, result: &mut LimitedString) {
         let empty_string = String::new();
-        let value = self.fmts.get(&c).unwrap_or_else(|| &empty_string);
+        let value = self.fmts.get(&c).unwrap_or(&empty_string);
         match *padding {
             Padding::None => result.push_str(value),
 
@@ -157,7 +157,7 @@ impl FmtStrFormatter {
             }
             _ => {
                 if let Some(ref els) = *els {
-                    result.push_str(&self.formatting_helper(&els, width))
+                    result.push_str(&self.formatting_helper(els, width))
                 }
             }
         }
@@ -180,7 +180,7 @@ impl FmtStrFormatter {
                 }
 
                 Specifier::Format(c, ref padding) => {
-                    self.format_format(c, &padding, width, &mut result);
+                    self.format_format(c, padding, width, &mut result);
                 }
 
                 Specifier::Text(s) => {
@@ -188,9 +188,9 @@ impl FmtStrFormatter {
                         result.push_str(s);
                     } else {
                         let remaining = width as usize - result.length();
-                        let count = utils::strwidth(&s);
+                        let count = utils::strwidth(s);
                         if remaining >= count {
-                            result.push_str(&s);
+                            result.push_str(s);
                         } else {
                             result.push_str(&utils::substr_with_width(s, remaining));
                         }
@@ -198,7 +198,7 @@ impl FmtStrFormatter {
                 }
 
                 Specifier::Conditional(cond, ref then, ref els) => {
-                    self.format_conditional(cond, &then, &els, width, &mut result)
+                    self.format_conditional(cond, then, els, width, &mut result)
                 }
             }
         }
@@ -621,27 +621,27 @@ mod tests {
         #[test]
         fn does_not_crash_when_formatting_with_no_formats_registered(ref input in "\\PC*") {
             let fmt = FmtStrFormatter::new();
-            fmt.do_format(&input, 0);
+            fmt.do_format(input, 0);
         }
 
         #[test]
         fn does_not_crash_on_any_spacing_character(ref c in "\\PC") {
             let fmt = FmtStrFormatter::new();
-            let format = format!("%>{}", c);
+            let format = format!("%>{c}");
             fmt.do_format(&format, 0);
         }
 
         #[test]
         fn does_not_crash_on_any_character_in_conditional(ref c in "\\PC") {
             let fmt = FmtStrFormatter::new();
-            let format = format!("%?{}?hello?%", c);
+            let format = format!("%?{c}?hello?%");
             fmt.do_format(&format, 0);
         }
 
         #[test]
         fn result_is_never_longer_than_specified_width(length in 1u32..10000, ref input in "\\PC*") {
             let fmt = FmtStrFormatter::new();
-            let result = fmt.do_format(&input, length);
+            let result = fmt.do_format(input, length);
             assert!(utils::strwidth(&result) <= length as usize);
         }
     }
