@@ -207,11 +207,11 @@ static int fill_content_callback(void* myfeed,
 	char** /* azColName */)
 {
 	RssFeed* feed = static_cast<RssFeed*>(myfeed);
-	assert(argc == 2);
+	assert(argc == 3);
 	if (argv[0]) {
 		std::shared_ptr<RssItem> item =
 			feed->get_item_by_guid_unlocked(argv[0]);
-		item->set_description(argv[1] ? argv[1] : "");
+		item->set_description(argv[1] ? argv[1] : "", argv[2] ? argv[2] : "");
 	}
 	return 0;
 }
@@ -287,6 +287,7 @@ Cache::~Cache()
 
 void Cache::set_pragmas()
 {
+	std::lock_guard<std::recursive_mutex> lock(mtx);
 	// first, we need to swithc off synchronous writing as it's slow as hell
 	run_sql("PRAGMA synchronous = OFF;");
 
@@ -376,21 +377,32 @@ static const schema_patches schemaPatches{
 		{
 			"CREATE TABLE metadata ( "
 			" db_schema_version_major INTEGER NOT NULL, "
-			" db_schema_version_minor INTEGER NOT NULL );"
+			" db_schema_version_minor INTEGER NOT NULL );",
 
 			"INSERT INTO metadata VALUES ( 2, 11 );"
 		}
-	}};
+	},
+	{	{2, 22},
+		{
+			"UPDATE metadata SET db_schema_version_major = 2, db_schema_version_minor = 22;",
+
+			"ALTER TABLE rss_item ADD COLUMN content_mime_type VARCHAR(255) NOT NULL DEFAULT \"\";"
+		}
+	}
+
+	// Note: schema changes should use the version number of the release that introduced them.
+};
 
 void Cache::populate_tables()
 {
+	std::lock_guard<std::recursive_mutex> lock(mtx);
 	const SchemaVersion version = get_schema_version();
 	LOG(Level::INFO,
 		"Cache::populate_tables: DB schema version %u.%u",
 		version.major,
 		version.minor);
 
-	if (version.major > utils::newsboat_version_major()) {
+	if (version.major > utils::newsboat_major_version()) {
 		const std::string msg =
 			"Database schema isn't supported because it's too new";
 		LOG(Level::ERROR, msg);
@@ -422,7 +434,7 @@ void Cache::fetch_lastmodified(const std::string& feedurl,
 	time_t& t,
 	std::string& etag)
 {
-	std::lock_guard<std::mutex> lock(mtx);
+	std::lock_guard<std::recursive_mutex> lock(mtx);
 	std::string query = prepare_query(
 			"SELECT lastmodified, etag FROM rss_feed WHERE rssurl = '%q';",
 			feedurl);
@@ -449,7 +461,7 @@ void Cache::update_lastmodified(const std::string& feedurl,
 			"empty, not updating anything");
 		return;
 	}
-	std::lock_guard<std::mutex> lock(mtx);
+	std::lock_guard<std::recursive_mutex> lock(mtx);
 	std::string query = "UPDATE rss_feed SET ";
 	if (t > 0) {
 		query.append(prepare_query("lastmodified = '%d'", t));
@@ -466,7 +478,7 @@ void Cache::update_lastmodified(const std::string& feedurl,
 
 void Cache::mark_item_deleted(const std::string& guid, bool b)
 {
-	std::lock_guard<std::mutex> lock(mtx);
+	std::lock_guard<std::recursive_mutex> lock(mtx);
 	std::string query = prepare_query(
 			"UPDATE rss_item SET deleted = %u WHERE guid = '%q'",
 			b ? 1 : 0,
@@ -474,15 +486,6 @@ void Cache::mark_item_deleted(const std::string& guid, bool b)
 	run_sql_nothrow(query);
 }
 
-void Cache::mark_feed_items_deleted(const std::string& feedurl)
-{
-	std::lock_guard<std::mutex> lock(mtx);
-	std::string query = prepare_query(
-			"UPDATE rss_item SET deleted = 1 WHERE feedurl = '%s';",
-			feedurl);
-	run_sql_nothrow(query);
-}
-
 // this function writes an RssFeed including all RssItems to the database
 void Cache::externalize_rssfeed(std::shared_ptr<RssFeed> feed,
 	bool reset_unread)
@@ -492,7 +495,7 @@ void Cache::externalize_rssfeed(std::shared_ptr<RssFeed> feed,
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(mtx);
+	std::lock_guard<std::recursive_mutex> lock(mtx);
 	std::lock_guard<std::mutex> feedlock(feed->item_mutex);
 	// scope_transaction dbtrans(db);
 
@@ -564,14 +567,13 @@ std::shared_ptr<RssFeed> Cache::internalize_rssfeed(std::string rssurl,
 {
 	ScopeMeasure m1("Cache::internalize_rssfeed");
 
-	std::shared_ptr<RssFeed> feed(new RssFeed(this));
-	feed->set_rssurl(rssurl);
+	std::shared_ptr<RssFeed> feed(new RssFeed(this, rssurl));
 
 	if (utils::is_query_url(rssurl)) {
 		return feed;
 	}
 
-	std::lock_guard<std::mutex> lock(mtx);
+	std::lock_guard<std::recursive_mutex> lock(mtx);
 	std::lock_guard<std::mutex> feedlock(feed->item_mutex);
 
 	/* first, we check whether the feed is there at all */
@@ -637,7 +639,7 @@ std::shared_ptr<RssFeed> Cache::internalize_rssfeed(std::string rssurl,
 		for (unsigned int j = max_items; j < feed->total_item_count();
 			++j) {
 			if (feed->items()[j]->flags().length() == 0) {
-				delete_item(feed->items()[j]);
+				delete_item_unlocked(feed->items()[j]);
 			} else {
 				flagged_items.push_back(feed->items()[j]);
 			}
@@ -655,13 +657,14 @@ std::shared_ptr<RssFeed> Cache::internalize_rssfeed(std::string rssurl,
 }
 
 std::vector<std::shared_ptr<RssItem>> Cache::search_for_items(
-		const std::string& querystr, const std::string& feedurl)
+		const std::string& querystr, const std::string& feedurl,
+		RssIgnores& ign)
 {
 	assert(!utils::is_query_url(feedurl));
 	std::string query;
 	std::vector<std::shared_ptr<RssItem>> items;
 
-	std::lock_guard<std::mutex> lock(mtx);
+	std::lock_guard<std::recursive_mutex> lock(mtx);
 	if (feedurl.length() > 0) {
 		query = prepare_query(
 				"SELECT guid, title, author, url, pubDate, "
@@ -694,6 +697,23 @@ std::vector<std::shared_ptr<RssItem>> Cache::search_for_items(
 	for (const auto& item : items) {
 		item->set_cache(this);
 	}
+	items.erase(
+		std::remove_if(
+			items.begin(),
+			items.end(),
+	[&](std::shared_ptr<RssItem> item) -> bool {
+		try
+		{
+			return ign.matches(item.get());
+		} catch (const MatcherException& ex)
+		{
+			LOG(Level::DEBUG,
+				"Cache::search_for_items: oops, Matcher exception: %s",
+				ex.what());
+			return false;
+		}
+	}),
+	items.end());
 
 	return items;
 }
@@ -702,8 +722,9 @@ std::unordered_set<std::string> Cache::search_in_items(
 	const std::string& querystr,
 	const std::unordered_set<std::string>& guids)
 {
-	std::string list = "(";
+	std::lock_guard<std::recursive_mutex> lock(mtx);
 
+	std::string list = "(";
 	for (const auto& guid : guids) {
 		list.append(prepare_query("%Q, ", guid));
 	}
@@ -719,12 +740,11 @@ std::unordered_set<std::string> Cache::search_in_items(
 			list);
 
 	std::unordered_set<std::string> items;
-	std::lock_guard<std::mutex> lock(mtx);
 	run_sql(query, guid_callback, &items);
 	return items;
 }
 
-void Cache::delete_item(const std::shared_ptr<RssItem>& item)
+void Cache::delete_item_unlocked(const std::shared_ptr<RssItem>& item)
 {
 	const std::string query = prepare_query(
 			"DELETE FROM rss_item WHERE guid = '%q';", item->guid());
@@ -733,15 +753,26 @@ void Cache::delete_item(const std::shared_ptr<RssItem>& item)
 
 void Cache::do_vacuum()
 {
-	std::lock_guard<std::mutex> lock(mtx);
+	std::lock_guard<std::recursive_mutex> lock(mtx);
 	run_sql("VACUUM;");
 }
 
-void Cache::cleanup_cache(std::vector<std::shared_ptr<RssFeed>> feeds)
+std::vector<std::string> Cache::cleanup_cache(std::vector<std::shared_ptr<RssFeed>> feeds,
+	bool always_clean)
 {
 	// we don't use the std::lock_guard<> here... see comments below
 	mtx.lock();
 
+	std::vector<std::string> unreachable_feeds{};
+	std::string list = "(";
+
+	for (const auto& feed : feeds) {
+		const auto name = prepare_query("'%q'", feed->rssurl());
+		list.append(name);
+		list.append(", ");
+	}
+	list.append("'')");
+
 	/*
 	 * cache cleanup means that all entries in both the RssFeed and
 	 * RssItem tables that are associated with an RSS feed URL that is not
@@ -754,17 +785,8 @@ void Cache::cleanup_cache(std::vector<std::shared_ptr<RssFeed>> feeds)
 	 * The behaviour whether the cleanup is done or not is configurable via
 	 * the configuration file.
 	 */
-	if (cfg->get_configvalue_as_bool("cleanup-on-quit")) {
+	if (always_clean || cfg->get_configvalue_as_bool("cleanup-on-quit")) {
 		LOG(Level::DEBUG, "Cache::cleanup_cache: cleaning up cache...");
-		std::string list = "(";
-
-		for (const auto& feed : feeds) {
-			std::string name =
-				prepare_query("'%q'", feed->rssurl());
-			list.append(name);
-			list.append(", ");
-		}
-		list.append("'')");
 
 		std::string cleanup_rss_feeds_statement(
 			"DELETE FROM rss_feed WHERE rssurl NOT IN ");
@@ -785,14 +807,29 @@ void Cache::cleanup_cache(std::vector<std::shared_ptr<RssFeed>> feeds)
 				"delete-read-articles-on-quit")) {
 			run_sql(cleanup_read_items_statement);
 		}
-
-		// WARNING: THE MISSING UNLOCK OPERATION IS MISSING FOR A
-		// PURPOSE! It's missing so that no database operation can occur
-		// after the cache cleanup! mtx->unlock();
 	} else {
 		LOG(Level::DEBUG,
 			"Cache::cleanup_cache: NOT cleaning up cache...");
+
+		std::string query = (
+				"SELECT DISTINCT rss "
+				"FROM ("
+				"SELECT feedurl AS rss FROM rss_item "
+				"UNION ALL "
+				"SELECT rssurl FROM rss_feed"
+				") "
+				"WHERE rss NOT IN "
+			);
+		query.append(list);
+		query.push_back(';');
+
+		run_sql(query, vectorofstring_callback, &unreachable_feeds);
 	}
+
+	// WARNING: THE MISSING UNLOCK OPERATION IS MISSING FOR A
+	// PURPOSE! It's missing so that no database operation can occur
+	// after the cache cleanup! mtx->unlock();
+	return unreachable_feeds;
 }
 
 void Cache::update_rssitem_unlocked(std::shared_ptr<RssItem> item,
@@ -804,6 +841,7 @@ void Cache::update_rssitem_unlocked(std::shared_ptr<RssItem> item,
 			item->guid());
 	CbHandler count_cbh;
 	run_sql(query, count_callback, &count_cbh);
+	const auto description = item->description();
 	if (count_cbh.count() > 0) {
 		if (reset_unread) {
 			std::string content;
@@ -812,13 +850,13 @@ void Cache::update_rssitem_unlocked(std::shared_ptr<RssItem> item,
 					"'%q';",
 					item->guid());
 			run_sql(query, single_string_callback, &content);
-			if (content != item->description()) {
+			if (content != description.text) {
 				LOG(Level::DEBUG,
 					"Cache::update_rssitem_unlocked: '%s' "
 					"is "
 					"different from '%s'",
 					content,
-					item->description());
+					description.text);
 				query = prepare_query(
 						"UPDATE rss_item SET unread = 1 WHERE "
 						"guid = '%q';",
@@ -832,7 +870,7 @@ void Cache::update_rssitem_unlocked(std::shared_ptr<RssItem> item,
 					"UPDATE rss_item "
 					"SET title = '%q', author = '%q', url = '%q', "
 					"feedurl = '%q', "
-					"content = '%q', enclosure_url = '%q', "
+					"content = '%q', content_mime_type = '%q', enclosure_url = '%q', "
 					"enclosure_type = '%q', base = '%q', unread = "
 					"'%d' "
 					"WHERE guid = '%q'",
@@ -840,7 +878,8 @@ void Cache::update_rssitem_unlocked(std::shared_ptr<RssItem> item,
 					item->author(),
 					item->link(),
 					feedurl,
-					item->description(),
+					description.text,
+					description.mime,
 					item->enclosure_url(),
 					item->enclosure_type(),
 					item->get_base(),
@@ -851,14 +890,15 @@ void Cache::update_rssitem_unlocked(std::shared_ptr<RssItem> item,
 					"UPDATE rss_item "
 					"SET title = '%q', author = '%q', url = '%q', "
 					"feedurl = '%q', "
-					"content = '%q', enclosure_url = '%q', "
+					"content = '%q', content_mime_type = '%q', enclosure_url = '%q', "
 					"enclosure_type = '%q', base = '%q' "
 					"WHERE guid = '%q'",
 					item->title(),
 					item->author(),
 					item->link(),
 					feedurl,
-					item->description(),
+					description.text,
+					description.mime,
 					item->enclosure_url(),
 					item->enclosure_type(),
 					item->get_base(),
@@ -869,10 +909,10 @@ void Cache::update_rssitem_unlocked(std::shared_ptr<RssItem> item,
 		std::string insert = prepare_query(
 				"INSERT INTO rss_item (guid, title, author, url, "
 				"feedurl, "
-				"pubDate, content, unread, enclosure_url, "
+				"pubDate, content, content_mime_type, unread, enclosure_url, "
 				"enclosure_type, enqueued, base) "
 				"VALUES "
-				"('%q','%q','%q','%q','%q','%u','%q','%d','%q','%q',%d,"
+				"('%q','%q','%q','%q','%q','%u','%q','%q','%d','%q','%q',%d,"
 				" "
 				"'%q')",
 				item->guid(),
@@ -881,7 +921,8 @@ void Cache::update_rssitem_unlocked(std::shared_ptr<RssItem> item,
 				item->link(),
 				feedurl,
 				item->pubDate_timestamp(),
-				item->description(),
+				description.text,
+				description.mime,
 				(item->unread() ? 1 : 0),
 				item->enclosure_url(),
 				item->enclosure_type(),
@@ -893,7 +934,7 @@ void Cache::update_rssitem_unlocked(std::shared_ptr<RssItem> item,
 
 void Cache::mark_all_read(std::shared_ptr<RssFeed> feed)
 {
-	std::lock_guard<std::mutex> lock(mtx);
+	std::lock_guard<std::recursive_mutex> lock(mtx);
 	std::lock_guard<std::mutex> itemlock(feed->item_mutex);
 	std::string query =
 		"UPDATE rss_item SET unread = '0' WHERE unread != '0' AND guid "
@@ -911,7 +952,7 @@ void Cache::mark_all_read(std::shared_ptr<RssFeed> feed)
  */
 void Cache::mark_all_read(const std::string& feedurl)
 {
-	std::lock_guard<std::mutex> lock(mtx);
+	std::lock_guard<std::recursive_mutex> lock(mtx);
 
 	std::string query;
 	if (feedurl.length() > 0) {
@@ -933,7 +974,7 @@ void Cache::mark_all_read(const std::string& feedurl)
 void Cache::update_rssitem_unread_and_enqueued(RssItem* item,
 	const std::string& /* feedurl */)
 {
-	std::lock_guard<std::mutex> lock(mtx);
+	std::lock_guard<std::recursive_mutex> lock(mtx);
 
 	const auto query = prepare_query(
 			"UPDATE rss_item "
@@ -986,7 +1027,7 @@ std::string Cache::prepare_query(const std::string& format, const T& argument,
 
 void Cache::update_rssitem_flags(RssItem* item)
 {
-	std::lock_guard<std::mutex> lock(mtx);
+	std::lock_guard<std::recursive_mutex> lock(mtx);
 
 	const std::string update = prepare_query(
 			"UPDATE rss_item SET flags = '%q' WHERE guid = '%q';",
@@ -1000,7 +1041,7 @@ void Cache::remove_old_deleted_items(RssFeed* feed)
 {
 	ScopeMeasure m1("Cache::remove_old_deleted_items");
 
-	std::lock_guard<std::mutex> cache_lock(mtx);
+	std::lock_guard<std::recursive_mutex> cache_lock(mtx);
 	std::lock_guard<std::mutex> feed_lock(feed->item_mutex);
 
 	std::vector<std::string> guids;
@@ -1033,6 +1074,7 @@ void Cache::remove_old_deleted_items(RssFeed* feed)
 void Cache::mark_items_read_by_guid(const std::vector<std::string>& guids)
 {
 	ScopeMeasure m1("Cache::mark_items_read_by_guid");
+	std::lock_guard<std::recursive_mutex> lock(mtx);
 	std::string guidset("(");
 	for (const auto& guid : guids) {
 		guidset.append(prepare_query("'%q', ", guid));
@@ -1044,7 +1086,6 @@ void Cache::mark_items_read_by_guid(const std::vector<std::string>& guids)
 			"%s;",
 			guidset);
 
-	std::lock_guard<std::mutex> lock(mtx);
 	run_sql(updatequery);
 }
 
@@ -1053,7 +1094,7 @@ std::vector<std::string> Cache::get_read_item_guids()
 	std::vector<std::string> guids;
 	const std::string query = "SELECT guid FROM rss_item WHERE unread = 0;";
 
-	std::lock_guard<std::mutex> lock(mtx);
+	std::lock_guard<std::recursive_mutex> lock(mtx);
 	run_sql(query, vectorofstring_callback, &guids);
 
 	return guids;
@@ -1061,7 +1102,7 @@ std::vector<std::string> Cache::get_read_item_guids()
 
 void Cache::clean_old_articles()
 {
-	std::lock_guard<std::mutex> lock(mtx);
+	std::lock_guard<std::recursive_mutex> lock(mtx);
 
 	const unsigned int days = cfg->get_configvalue_as_int("keep-articles-days");
 	if (days > 0) {
@@ -1087,6 +1128,7 @@ void Cache::clean_old_articles()
 
 void Cache::fetch_descriptions(RssFeed* feed)
 {
+	std::lock_guard<std::recursive_mutex> lock(mtx);
 	std::vector<std::string> guids;
 	for (const auto& item : feed->items()) {
 		guids.push_back(prepare_query("'%q'", item->guid()));
@@ -1094,15 +1136,16 @@ void Cache::fetch_descriptions(RssFeed* feed)
 	const std::string in_clause = utils::join(guids, ", ");
 
 	const std::string query = prepare_query(
-			"SELECT guid, content FROM rss_item WHERE guid IN (%s);",
+			"SELECT guid, content, content_mime_type FROM rss_item WHERE guid IN (%s);",
 			in_clause);
 
 	run_sql(query, fill_content_callback, feed);
 }
 
-std::string Cache::fetch_description(RssItem* item)
+std::string Cache::fetch_description(const RssItem& item)
 {
-	const std::string in_clause = prepare_query("'%q'", item->guid());
+	std::lock_guard<std::recursive_mutex> lock(mtx);
+	const std::string in_clause = prepare_query("'%q'", item.guid());
 
 	const std::string query = prepare_query(
 			"SELECT content FROM rss_item WHERE guid = %s;",
@@ -1124,6 +1167,7 @@ SchemaVersion Cache::get_schema_version()
 	sqlite3_stmt* stmt{};
 	SchemaVersion result;
 
+	std::lock_guard<std::recursive_mutex> lock(mtx);
 	int rc = sqlite3_prepare_v2(db,
 			"SELECT db_schema_version_major, db_schema_version_minor "
 			"FROM metadata",
