@@ -186,7 +186,7 @@ ConfigContainer::ConfigContainer()
 		ConfigData("no", ConfigDataType::BOOL)},
 	{
 		"podlist-format",
-		ConfigData( "%4i [%6dMB/%6tMB] [%5p %%] [%12K] %-20S %u -> %F", ConfigDataType::STR)},
+		ConfigData( _("%4i [%6dMB/%6tMB] [%5p %%] [%12K] %-20S %u -> %F"), ConfigDataType::STR)},
 	{
 		"prepopulate-query-feeds",
 		ConfigData("false", ConfigDataType::BOOL)},
@@ -266,6 +266,9 @@ ConfigContainer::ConfigContainer()
 	{"miniflux-password", ConfigData("", ConfigDataType::STR)},
 	{"miniflux-passwordfile", ConfigData("", ConfigDataType::PATH)},
 	{"miniflux-passwordeval", ConfigData("", ConfigDataType::STR)},
+	{"miniflux-token", ConfigData("", ConfigDataType::STR)},
+	{"miniflux-tokenfile", ConfigData("", ConfigDataType::PATH)},
+	{"miniflux-tokeneval", ConfigData("", ConfigDataType::STR)},
 	{"miniflux-url", ConfigData("", ConfigDataType::STR)},
 	{
 		"urls-source",
@@ -312,7 +315,7 @@ ConfigContainer::ConfigContainer()
 			ConfigDataType::STR)},
 	{
 		"searchresult-title-format",
-		ConfigData(_("%N %V - Search results (%u unread, %t total)%?F? matching filter '%F'&?"),
+		ConfigData(_("%N %V - Search results for '%s' (%u unread, %t total)%?F? matching filter '%F'&?"),
 			ConfigDataType::STR)},
 	{
 		"selectfilter-title-format",
