@@ -79,6 +79,8 @@ const std::string Download::status_text() const
 		return _s("ready");
 	case DlStatus::PLAYED:
 		return _s("played");
+	case DlStatus::RENAME_FAILED:
+		return _s("rename failed");
 	default:
 		return _s("unknown (bug).");
 	}
@@ -98,11 +100,12 @@ void Download::set_progress(double downloaded, double total)
 	totalsize = total;
 }
 
-void Download::set_status(DlStatus dls)
+void Download::set_status(DlStatus dls, const std::string& msg_)
 {
 	if (download_status != dls) {
 		cb_require_view_update();
 	}
+	msg = msg_;
 	download_status = dls;
 }
 
