@@ -6,6 +6,7 @@
 #include <vector>
 
 #include "config.h"
+#include "curlhandle.h"
 #include "strprintf.h"
 #include "utils.h"
 
@@ -26,12 +27,6 @@ namespace newsboat {
 FeedHqApi::FeedHqApi(ConfigContainer* c)
 	: RemoteApi(c)
 {
-	// TODO
-}
-
-FeedHqApi::~FeedHqApi()
-{
-	// TODO
 }
 
 bool FeedHqApi::authenticate()
@@ -51,17 +46,18 @@ static size_t my_write_data(void* buffer, size_t size, size_t nmemb,
 
 std::string FeedHqApi::retrieve_auth()
 {
-	CURL* handle = curl_easy_init();
+	CurlHandle handle;
+
 	Credentials cred = get_credentials("feedhq", "FeedHQ");
 	if (cred.user.empty() || cred.pass.empty()) {
 		return "";
 	}
 
-	char* username = curl_easy_escape(handle, cred.user.c_str(), 0);
-	char* password = curl_easy_escape(handle, cred.pass.c_str(), 0);
+	char* username = curl_easy_escape(handle.ptr(), cred.user.c_str(), 0);
+	char* password = curl_easy_escape(handle.ptr(), cred.pass.c_str(), 0);
 
 	std::string postcontent = strprintf::fmt(
-			"service=reader&Email=%s&Passwd=%s&source=%s%2F%s&accountType="
+			"service=reader&Email=%s&Passwd=%s&source=%s%%2F%s&accountType="
 			"HOSTED_OR_GOOGLE&continue=http://www.google.com/",
 			username,
 			password,
@@ -74,14 +70,13 @@ std::string FeedHqApi::retrieve_auth()
 	std::string result;
 
 	utils::set_common_curl_options(handle, cfg);
-	curl_easy_setopt(handle, CURLOPT_WRITEFUNCTION, my_write_data);
-	curl_easy_setopt(handle, CURLOPT_WRITEDATA, &result);
-	curl_easy_setopt(handle, CURLOPT_POSTFIELDS, postcontent.c_str());
-	curl_easy_setopt(handle,
+	curl_easy_setopt(handle.ptr(), CURLOPT_WRITEFUNCTION, my_write_data);
+	curl_easy_setopt(handle.ptr(), CURLOPT_WRITEDATA, &result);
+	curl_easy_setopt(handle.ptr(), CURLOPT_POSTFIELDS, postcontent.c_str());
+	curl_easy_setopt(handle.ptr(),
 		CURLOPT_URL,
 		(cfg->get_configvalue("feedhq-url") + FEEDHQ_LOGIN).c_str());
-	curl_easy_perform(handle);
-	curl_easy_cleanup(handle);
+	curl_easy_perform(handle.ptr());
 
 	for (const auto& line : utils::tokenize(result)) {
 		LOG(Level::DEBUG, "FeedHqApi::retrieve_auth: line = %s", line);
@@ -98,29 +93,26 @@ std::vector<TaggedFeedUrl> FeedHqApi::get_subscribed_urls()
 {
 	std::vector<TaggedFeedUrl> urls;
 
-	CURL* handle = curl_easy_init();
+	CurlHandle handle;
 	std::string result;
 	curl_slist* custom_headers{};
 	add_custom_headers(&custom_headers);
-	curl_easy_setopt(handle, CURLOPT_HTTPHEADER, custom_headers);
+	curl_easy_setopt(handle.ptr(), CURLOPT_HTTPHEADER, custom_headers);
 
 	utils::set_common_curl_options(handle, cfg);
-	curl_easy_setopt(handle, CURLOPT_WRITEFUNCTION, my_write_data);
-	curl_easy_setopt(handle, CURLOPT_WRITEDATA, &result);
-	curl_easy_setopt(handle,
+	curl_easy_setopt(handle.ptr(), CURLOPT_WRITEFUNCTION, my_write_data);
+	curl_easy_setopt(handle.ptr(), CURLOPT_WRITEDATA, &result);
+	curl_easy_setopt(handle.ptr(),
 		CURLOPT_URL,
 		(cfg->get_configvalue("feedhq-url") + FEEDHQ_SUBSCRIPTION_LIST)
 		.c_str());
-	curl_easy_perform(handle);
-	curl_easy_cleanup(handle);
+	curl_easy_perform(handle.ptr());
 	curl_slist_free_all(custom_headers);
 
 	LOG(Level::DEBUG,
 		"FeedHqApi::get_subscribed_urls: document = %s",
 		result);
 
-	// TODO: parse result
-
 	json_object* reply = json_tokener_parse(result.c_str());
 	if (reply == nullptr) {
 		LOG(Level::ERROR,
@@ -138,20 +130,27 @@ std::vector<TaggedFeedUrl> FeedHqApi::get_subscribed_urls()
 
 	for (int i = 0; i < len; i++) {
 		std::vector<std::string> tags;
-		json_object* sub =
-			json_object_array_get_idx(subscription_obj, i);
+		json_object* sub = json_object_array_get_idx(subscription_obj, i);
 
 		json_object* id_str{};
 		json_object_object_get_ex(sub, "id", &id_str);
 		const char* id = json_object_get_string(id_str);
+		if (id == nullptr) {
+			LOG(Level::WARN, "Skipping a subscription without an id");
+			continue;
+		}
 
 		json_object* title_str{};
 		json_object_object_get_ex(sub, "title", &title_str);
 		const char* title = json_object_get_string(title_str);
+		if (title != nullptr) {
+			tags.push_back(std::string("~") + title);
+		} else {
+			LOG(Level::WARN, "Subscription has no title, so let's call it \"%i\"", i);
+			tags.push_back(std::string("~") + std::to_string(i));
+		}
 
-		tags.push_back(std::string("~") + title);
-
-		char* escaped_id = curl_easy_escape(handle, id, 0);
+		char* escaped_id = curl_easy_escape(handle.ptr(), id, 0);
 
 		auto url = strprintf::fmt("%s%s%s?n=%u",
 				cfg->get_configvalue("feedhq-url"),
@@ -253,21 +252,20 @@ bool FeedHqApi::mark_article_read_with_token(const std::string& guid,
 
 std::string FeedHqApi::get_new_token()
 {
-	CURL* handle = curl_easy_init();
+	CurlHandle handle;
 	std::string result;
 	curl_slist* custom_headers{};
 
 	utils::set_common_curl_options(handle, cfg);
 	add_custom_headers(&custom_headers);
-	curl_easy_setopt(handle, CURLOPT_HTTPHEADER, custom_headers);
-	curl_easy_setopt(handle, CURLOPT_WRITEFUNCTION, my_write_data);
-	curl_easy_setopt(handle, CURLOPT_WRITEDATA, &result);
-	curl_easy_setopt(handle,
+	curl_easy_setopt(handle.ptr(), CURLOPT_HTTPHEADER, custom_headers);
+	curl_easy_setopt(handle.ptr(), CURLOPT_WRITEFUNCTION, my_write_data);
+	curl_easy_setopt(handle.ptr(), CURLOPT_WRITEDATA, &result);
+	curl_easy_setopt(handle.ptr(),
 		CURLOPT_URL,
 		(cfg->get_configvalue("feedhq-url") + FEEDHQ_API_TOKEN_URL)
 		.c_str());
-	curl_easy_perform(handle);
-	curl_easy_cleanup(handle);
+	curl_easy_perform(handle.ptr());
 	curl_slist_free_all(custom_headers);
 
 	LOG(Level::DEBUG, "FeedHqApi::get_new_token: token = %s", result);
@@ -284,23 +282,15 @@ bool FeedHqApi::update_article_flags(const std::string& oldflags,
 	bool success = true;
 
 	if (star_flag.length() > 0) {
-		if (strchr(oldflags.c_str(), star_flag[0]) == nullptr &&
-			strchr(newflags.c_str(), star_flag[0]) != nullptr) {
-			success = star_article(guid, true);
-		} else if (strchr(oldflags.c_str(), star_flag[0]) != nullptr &&
-			strchr(newflags.c_str(), star_flag[0]) == nullptr) {
-			success = star_article(guid, false);
-		}
+		update_flag(oldflags, newflags, star_flag[0], [&](bool added) {
+			success = star_article(guid, added);
+		});
 	}
 
 	if (share_flag.length() > 0) {
-		if (strchr(oldflags.c_str(), share_flag[0]) == nullptr &&
-			strchr(newflags.c_str(), share_flag[0]) != nullptr) {
-			success = share_article(guid, true);
-		} else if (strchr(oldflags.c_str(), share_flag[0]) != nullptr &&
-			strchr(newflags.c_str(), share_flag[0]) == nullptr) {
-			success = share_article(guid, false);
-		}
+		update_flag(oldflags, newflags, share_flag[0], [&](bool added) {
+			success = share_article(guid, added);
+		});
 	}
 
 	return success;
@@ -360,16 +350,15 @@ std::string FeedHqApi::post_content(const std::string& url,
 	std::string result;
 	curl_slist* custom_headers{};
 
-	CURL* handle = curl_easy_init();
+	CurlHandle handle;
 	utils::set_common_curl_options(handle, cfg);
 	add_custom_headers(&custom_headers);
-	curl_easy_setopt(handle, CURLOPT_HTTPHEADER, custom_headers);
-	curl_easy_setopt(handle, CURLOPT_WRITEFUNCTION, my_write_data);
-	curl_easy_setopt(handle, CURLOPT_WRITEDATA, &result);
-	curl_easy_setopt(handle, CURLOPT_POSTFIELDS, postdata.c_str());
-	curl_easy_setopt(handle, CURLOPT_URL, url.c_str());
-	curl_easy_perform(handle);
-	curl_easy_cleanup(handle);
+	curl_easy_setopt(handle.ptr(), CURLOPT_HTTPHEADER, custom_headers);
+	curl_easy_setopt(handle.ptr(), CURLOPT_WRITEFUNCTION, my_write_data);
+	curl_easy_setopt(handle.ptr(), CURLOPT_WRITEDATA, &result);
+	curl_easy_setopt(handle.ptr(), CURLOPT_POSTFIELDS, postdata.c_str());
+	curl_easy_setopt(handle.ptr(), CURLOPT_URL, url.c_str());
+	curl_easy_perform(handle.ptr());
 	curl_slist_free_all(custom_headers);
 
 	LOG(Level::DEBUG,
