@@ -35,7 +35,7 @@ FeedHqUrlReader::~FeedHqUrlReader() {}
 		tags[(url)] = tmptags;        \
 	} while (0)
 
-void FeedHqUrlReader::reload()
+nonstd::optional<utils::ReadTextFileError> FeedHqUrlReader::reload()
 {
 	urls.clear();
 	tags.clear();
@@ -51,7 +51,11 @@ void FeedHqUrlReader::reload()
 	}
 
 	FileUrlReader ur(file);
-	ur.reload();
+	const auto error_message = ur.reload();
+	if (error_message.has_value()) {
+		LOG(Level::DEBUG, "Reloading failed: %s", error_message.value().message);
+		// Ignore errors for now: https://github.com/newsboat/newsboat/issues/1273
+	}
 
 	for (const auto& url : ur.get_urls()) {
 		if (utils::is_query_url(url)) {
@@ -78,6 +82,8 @@ void FeedHqUrlReader::reload()
 			alltags.insert(tag);
 		}
 	}
+
+	return {};
 }
 
 std::string FeedHqUrlReader::get_source()
