@@ -4,10 +4,12 @@
 #include <cassert>
 #include <cstring>
 #include <langinfo.h>
+#include <numeric>
 #include <sstream>
 #include <string>
 
 #include "config.h"
+#include "controller.h"
 #include "dbexception.h"
 #include "feedcontainer.h"
 #include "fmtstrformatter.h"
@@ -30,11 +32,11 @@ FeedListFormAction::FeedListFormAction(View* vv,
 	RegexManager& r)
 	: ListFormAction(vv, formstr, "feeds", cfg)
 	, zero_feedpos(false)
-	, apply_filter(false)
+	, filter_active(false)
 	, filterpos(0)
 	, set_filterpos(false)
 	, rxman(r)
-	, filters(f)
+	, filter_container(f)
 	, cache(cc)
 {
 	valid_cmds.push_back("tag");
@@ -47,7 +49,7 @@ void FeedListFormAction::init()
 {
 	set_keymap_hints();
 
-	f.run(-3); // compute all widget dimensions
+	recalculate_widget_dimensions();
 
 	if (v->get_ctrl()->get_refresh_on_start()) {
 		v->get_ctrl()->get_reloader()->start_reload_all_thread();
@@ -108,7 +110,7 @@ REDO:
 				v->push_itemlist(pos);
 			} else {
 				// should not happen
-				v->show_error(_("No feed selected!"));
+				v->get_statusline().show_error(_("No feed selected!"));
 			}
 		}
 	}
@@ -120,21 +122,18 @@ REDO:
 		if (visible_feeds.size() > 0 && feedpos.length() > 0) {
 			v->get_ctrl()->get_reloader()->reload(pos);
 		} else {
-			v->show_error(
+			v->get_statusline().show_error(
 				_("No feed selected!")); // should not happen
 		}
 	}
 	break;
-	case OP_INT_RESIZE:
-		do_redraw = true;
-		break;
 	case OP_RELOADURLS:
 		v->get_ctrl()->reload_urls_file();
 		break;
 	case OP_SORT: {
-		/// This string is related to the letters in parentheses in the
-		/// "Sort by (f)irsttag/..." and "Reverse Sort by
-		/// (f)irsttag/..." messages
+		// i18n: This string is related to the letters in parentheses in the
+		// "Sort by (f)irsttag/..." and "Reverse Sort by
+		// (f)irsttag/..." messages
 		std::string input_options = _("ftauln");
 		char c = v->confirm(
 				_("Sort by "
@@ -213,53 +212,14 @@ REDO:
 		}
 	}
 	break;
-	case OP_OPENINBROWSER:
-		if (visible_feeds.size() > 0 && feedpos.length() > 0) {
-			std::shared_ptr<RssFeed> feed =
-				v->get_ctrl()->get_feedcontainer()->get_feed(pos);
-			if (feed) {
-				if (!feed->is_query_feed()) {
-					LOG(Level::INFO,
-						"FeedListFormAction: opening "
-						"feed "
-						"at position `%s': %s",
-						feedpos,
-						feed->link());
-
-					std::string url;
-					if (!feed->link().empty()) {
-						url = feed->link();
-					} else if (!feed->rssurl().empty()) {
-						url = feed->rssurl();
-					} else {
-						// rssurl can't be empty, so if we got to this branch,
-						// something is clearly wrong with Newsboat internals.
-						// That's why we write a message to the log, and not
-						// just display it to the user.
-						LOG(Level::INFO,
-							"FeedListFormAction: cannot open feed in browser "
-							"because both `link' and `rssurl' fields are "
-							"empty");
-					}
-
-					if (!url.empty()) {
-						const auto exit_code = v->open_in_browser(url);
-						if (!exit_code.has_value()) {
-							v->show_error(_("Failed to spawn browser"));
-							return false;
-						} else if (*exit_code != 0) {
-							v->show_error(strprintf::fmt(_("Browser returned error code %i"), *exit_code));
-							return false;
-						}
-					}
-				} else {
-					v->show_error(_("Cannot open query feeds in the browser!"));
-				}
-			}
-		} else {
-			v->show_error(_("No feed selected!"));
-		}
-		break;
+	case OP_OPENINBROWSER: {
+		const bool interactive = true;
+		return open_position_in_browser(pos, interactive);
+	}
+	case OP_OPENINBROWSER_NONINTERACTIVE: {
+		const bool interactive = false;
+		return open_position_in_browser(pos, interactive);
+	}
 	case OP_OPENALLUNREADINBROWSER:
 		if (visible_feeds.size() > 0 && feedpos.length() > 0) {
 			std::shared_ptr<RssFeed> feed =
@@ -271,17 +231,22 @@ REDO:
 					"items in feed at position `%s'",
 					feedpos.c_str());
 
-				const auto exit_code = open_unread_items_in_browser(feed, false);
+				// We can't just `const auto exit_code = ...` here because this
+				// triggers -Wmaybe-initialized in GCC 9 with -O2.
+				nonstd::optional<std::uint8_t> exit_code;
+				exit_code = open_unread_items_in_browser(feed, false);
+
 				if (!exit_code.has_value()) {
-					v->show_error(_("Failed to spawn browser"));
+					v->get_statusline().show_error(_("Failed to spawn browser"));
 					return false;
 				} else if (*exit_code != 0) {
-					v->show_error(strprintf::fmt(_("Browser returned error code %i"), *exit_code));
+					v->get_statusline().show_error(strprintf::fmt(_("Browser returned error code %i"),
+							*exit_code));
 					return false;
 				}
 			}
 		} else {
-			v->show_error(_("No feed selected!"));
+			v->get_statusline().show_error(_("No feed selected!"));
 		}
 		break;
 	case OP_OPENALLUNREADINBROWSER_AND_MARK:
@@ -296,12 +261,17 @@ REDO:
 					"marking read",
 					feedpos.c_str());
 
-				const auto exit_code = open_unread_items_in_browser(feed, true);
+				// We can't just `const auto exit_code = ...` here because this
+				// triggers -Wmaybe-initialized in GCC 9 with -O2.
+				nonstd::optional<std::uint8_t> exit_code;
+				exit_code = open_unread_items_in_browser(feed, true);
+
 				if (!exit_code.has_value()) {
-					v->show_error(_("Failed to spawn browser"));
+					v->get_statusline().show_error(_("Failed to spawn browser"));
 					return false;
 				} else if (*exit_code != 0) {
-					v->show_error(strprintf::fmt(_("Browser returned error code %i"), *exit_code));
+					v->get_statusline().show_error(strprintf::fmt(_("Browser returned error code %i"),
+							*exit_code));
 					return false;
 				}
 
@@ -317,6 +287,11 @@ REDO:
 					"reload-only-visible-feeds");
 			std::vector<int> idxs;
 			if (reload_only_visible_feeds) {
+				if (visible_feeds.empty()) {
+					// Do not reload *all* feeds, when
+					// there is no visible feeds at all.
+					break;
+				}
 				for (const auto& feed : visible_feeds) {
 					idxs.push_back(feed.second);
 				}
@@ -325,28 +300,35 @@ REDO:
 		}
 		break;
 	case OP_MARKFEEDREAD: {
-		LOG(Level::INFO,
-			"FeedListFormAction: marking feed read at position "
-			"`%s'",
-			feedpos);
-		if (visible_feeds.size() > 0 && feedpos.length() > 0) {
-			v->set_status(_("Marking feed read..."));
-			try {
-				v->get_ctrl()->mark_all_read(pos);
-				do_redraw = true;
-				v->set_status("");
-				bool show_read = cfg->get_configvalue_as_bool("show-read-feeds");
-				if (visible_feeds.size() > (pos + 1) && show_read) {
-					list.set_position(pos + 1);
+		if (!cfg->get_configvalue_as_bool(
+				"confirm-mark-feed-read") ||
+			v->confirm(_("Do you really want to mark this feed as read (y:Yes n:No)? "),
+				_("yn")) == *_("y")) {
+			LOG(Level::INFO,
+				"FeedListFormAction: marking feed read at position "
+				"`%s'",
+				feedpos);
+			if (visible_feeds.size() > 0 && feedpos.length() > 0) {
+				try {
+					{
+						const auto message_lifetime = v->get_statusline().show_message_until_finished(
+								_("Marking feed read..."));
+						v->get_ctrl()->mark_all_read(pos);
+						do_redraw = true;
+					}
+					bool show_read = cfg->get_configvalue_as_bool("show-read-feeds");
+					if (visible_feeds.size() > (list.get_position() + 1) && show_read) {
+						list.set_position(list.get_position() + 1);
+					}
+				} catch (const DbException& e) {
+					v->get_statusline().show_error(strprintf::fmt(
+							_("Error: couldn't mark feed read: %s"),
+							e.what()));
 				}
-			} catch (const DbException& e) {
-				v->show_error(strprintf::fmt(
-						_("Error: couldn't mark feed read: %s"),
-						e.what()));
+			} else {
+				v->get_statusline().show_error(
+					_("No feed selected!")); // should not happen
 			}
-		} else {
-			v->show_error(
-				_("No feed selected!")); // should not happen
 		}
 	}
 	break;
@@ -366,7 +348,7 @@ REDO:
 		LOG(Level::INFO,
 			"FeedListFormAction: jumping to next unread feed");
 		if (!jump_to_next_unread_feed(local_tmp)) {
-			v->show_error(_("No feeds with unread items."));
+			v->get_statusline().show_error(_("No feeds with unread items."));
 		}
 	}
 	break;
@@ -375,7 +357,7 @@ REDO:
 		LOG(Level::INFO,
 			"FeedListFormAction: jumping to previous unread feed");
 		if (!jump_to_previous_unread_feed(local_tmp)) {
-			v->show_error(_("No feeds with unread items."));
+			v->get_statusline().show_error(_("No feeds with unread items."));
 		}
 	}
 	break;
@@ -383,7 +365,7 @@ REDO:
 		unsigned int local_tmp;
 		LOG(Level::INFO, "FeedListFormAction: jumping to next feed");
 		if (!jump_to_next_feed(local_tmp)) {
-			v->show_error(_("Already on last feed."));
+			v->get_statusline().show_error(_("Already on last feed."));
 		}
 	}
 	break;
@@ -392,7 +374,7 @@ REDO:
 		LOG(Level::INFO,
 			"FeedListFormAction: jumping to previous feed");
 		if (!jump_to_previous_feed(local_tmp)) {
-			v->show_error(_("Already on first feed."));
+			v->get_statusline().show_error(_("Already on first feed."));
 		}
 	}
 	break;
@@ -401,15 +383,18 @@ REDO:
 		LOG(Level::INFO,
 			"FeedListFormAction: jumping to random unread feed");
 		if (!jump_to_random_unread_feed(local_tmp)) {
-			v->show_error(_("No feeds with unread items."));
+			v->get_statusline().show_error(_("No feeds with unread items."));
 		}
 	}
 	break;
 	case OP_MARKALLFEEDSREAD:
-		if (v->confirm(_("Do you really want to mark all feeds as read (y:Yes n:No)? "),
+		if (!cfg->get_configvalue_as_bool(
+				"confirm-mark-all-feeds-read") ||
+			v->confirm(_("Do you really want to mark all feeds as read (y:Yes n:No)? "),
 				_("yn")) == *_("y")) {
 			LOG(Level::INFO, "FeedListFormAction: marking all feeds read");
-			v->set_status(_("Marking all feeds read..."));
+			const auto message_lifetime = v->get_statusline().show_message_until_finished(
+					_("Marking all feeds read..."));
 			if (tag == "") {
 				v->get_ctrl()->mark_all_read("");
 			} else {
@@ -421,7 +406,6 @@ REDO:
 					v->get_ctrl()->mark_all_read(feedurl);
 				}
 			}
-			v->set_status("");
 			do_redraw = true;
 		}
 		break;
@@ -435,7 +419,7 @@ REDO:
 		if (automatic && args->size() > 0) {
 			newtag = (*args)[0];
 		} else {
-			newtag = v->select_tag();
+			newtag = v->select_tag(tag);
 		}
 		if (newtag != "") {
 			tag = newtag;
@@ -445,33 +429,23 @@ REDO:
 	}
 	break;
 	case OP_SELECTFILTER:
-		if (filters.size() > 0) {
-			std::string newfilter;
+		if (filter_container.size() > 0) {
 			if (automatic && args->size() > 0) {
-				newfilter = (*args)[0];
-			} else {
-				newfilter = v->select_filter(
-						filters.get_filters());
-			}
-			if (newfilter != "") {
-				filterhistory.add_line(newfilter);
-				if (newfilter.length() > 0) {
-					if (!matcher.parse(newfilter)) {
-						v->show_error(strprintf::fmt(
-								_("Error: couldn't "
-									"parse filter "
-									"command `%s': %s"),
-								newfilter,
-								matcher.get_parse_error()));
-					} else {
-						save_filterpos();
-						apply_filter = true;
-						do_redraw = true;
-					}
+				const std::string filter_name = (*args)[0];
+				const auto filter = filter_container.get_filter(filter_name);
+
+				if (filter.has_value()) {
+					apply_filter(filter.value());
+				} else {
+					v->get_statusline().show_error(strprintf::fmt(_("No filter found with name `%s'."),
+							filter_name));
 				}
+			} else {
+				const std::string filter_text = v->select_filter(filter_container.get_filters());
+				apply_filter(filter_text);
 			}
 		} else {
-			v->show_error(_("No filters defined."));
+			v->get_statusline().show_error(_("No filters defined."));
 		}
 		break;
 	case OP_SEARCH:
@@ -490,8 +464,20 @@ REDO:
 				qna, OP_INT_START_SEARCH, &searchhistory);
 		}
 		break;
+	case OP_GOTO_TITLE:
+		if (automatic) {
+			if (args->size() >= 1) {
+				qna_responses = {args[0]};
+				finished_qna(OP_INT_GOTO_TITLE);
+			}
+		} else {
+			std::vector<QnaPair> qna;
+			qna.push_back(QnaPair(_("Title: "), ""));
+			this->start_qna(qna, OP_INT_GOTO_TITLE);
+		}
+		break;
 	case OP_CLEARFILTER:
-		apply_filter = false;
+		filter_active = false;
 		do_redraw = true;
 		save_filterpos();
 		break;
@@ -543,6 +529,64 @@ REDO:
 	return true;
 }
 
+bool FeedListFormAction::open_position_in_browser(unsigned int pos,
+	bool interactive) const
+{
+	if (visible_feeds.empty()) {
+		v->get_statusline().show_error(_("No feed selected!"));
+		return false;
+	}
+
+	std::shared_ptr<RssFeed> feed = v->get_ctrl()->get_feedcontainer()->get_feed(
+			pos);
+	if (feed == nullptr) {
+		v->get_statusline().show_error(_("No feed selected!"));
+		return false;
+	}
+
+	if (feed->is_query_feed()) {
+		v->get_statusline().show_error(_("Cannot open query feeds in the browser!"));
+		return false;
+	}
+
+	LOG(Level::INFO, "FeedListFormAction: opening feed %s, interactive: %s",
+		feed->link(),
+		interactive ? "true" : "false");
+
+	std::string url;
+	std::string type;
+	if (!feed->link().empty()) {
+		url = feed->link();
+		type = "feed";
+	} else if (!feed->rssurl().empty()) {
+		url = feed->rssurl();
+		type = "rssfeed";
+	} else {
+		// rssurl can't be empty, so if we got to this branch,
+		// something is clearly wrong with Newsboat internals.
+		// That's why we write a message to the log, and not
+		// just display it to the user.
+		LOG(Level::INFO,
+			"FeedListFormAction: cannot open feed in browser "
+			"because both `link' and `rssurl' fields are "
+			"empty");
+	}
+
+	if (!url.empty()) {
+		const std::string feedurl = feed->rssurl();
+		const auto exit_code = v->open_in_browser(url, feedurl, type, feed->title(), interactive);
+		if (!exit_code.has_value()) {
+			v->get_statusline().show_error(_("Failed to spawn browser"));
+			return false;
+		} else if (*exit_code != 0) {
+			v->get_statusline().show_error(strprintf::fmt(_("Browser returned error code %i"),
+					*exit_code));
+			return false;
+		}
+	}
+	return true;
+}
+
 void FeedListFormAction::update_visible_feeds(
 	std::vector<std::shared_ptr<RssFeed>>& feeds)
 {
@@ -557,7 +601,7 @@ void FeedListFormAction::update_visible_feeds(
 		feed->set_index(i + 1);
 		if ((tag == "" || feed->matches_tag(tag)) &&
 			(show_read || feed->unread_item_count() > 0) &&
-			(!apply_filter || matcher.matches(feed.get())) &&
+			(!filter_active || matcher.matches(feed.get())) &&
 			!feed->hidden()) {
 			visible_feeds.push_back(FeedPtrPosPair(feed, i));
 		}
@@ -590,9 +634,9 @@ void FeedListFormAction::set_feedlist(
 	update_form_title(width);
 }
 
-KeyMapHintEntry* FeedListFormAction::get_keymap_hint()
+const std::vector<KeyMapHintEntry>& FeedListFormAction::get_keymap_hint() const
 {
-	static KeyMapHintEntry hints[] = {{OP_QUIT, _("Quit")},
+	static const std::vector<KeyMapHintEntry> hints = {{OP_QUIT, _("Quit")},
 		{OP_OPEN, _("Open")},
 		{OP_NEXTUNREAD, _("Next Unread")},
 		{OP_RELOAD, _("Reload")},
@@ -600,8 +644,7 @@ KeyMapHintEntry* FeedListFormAction::get_keymap_hint()
 		{OP_MARKFEEDREAD, _("Mark Read")},
 		{OP_MARKALLFEEDSREAD, _("Mark All Read")},
 		{OP_SEARCH, _("Search")},
-		{OP_HELP, _("Help")},
-		{OP_NIL, nullptr}
+		{OP_HELP, _("Help")}
 	};
 	return hints;
 }
@@ -610,38 +653,29 @@ bool FeedListFormAction::jump_to_previous_unread_feed(unsigned int& feedpos)
 {
 	const unsigned int curpos = list.get_position();
 	LOG(Level::DEBUG,
-		"FeedListFormAction::jump_to_previous_unread_feed: searching "
-		"for "
-		"unread feed");
+		"FeedListFormAction::jump_to_previous_unread_feed: searching for unread feed");
 
 	for (int i = curpos - 1; i >= 0; --i) {
+		const auto unread = visible_feeds[i].first->unread_item_count();
 		LOG(Level::DEBUG,
-			"FeedListFormAction::jump_to_previous_unread_feed: "
-			"visible_feeds[%u] unread items: %u",
+			"FeedListFormAction::jump_to_previous_unread_feed: visible_feeds[%u] unread items: %u",
 			i,
-			visible_feeds[i].first->unread_item_count());
-		if (visible_feeds[i].first->unread_item_count() > 0) {
-			LOG(Level::DEBUG,
-				"FeedListFormAction::jump_to_previous_unread_"
-				"feed:"
-				" hit");
+			unread);
+		if (unread > 0) {
+			LOG(Level::DEBUG, "FeedListFormAction::jump_to_previous_unread_feed: hit");
 			list.set_position(i);
 			feedpos = visible_feeds[i].second;
 			return true;
 		}
 	}
-	for (int i = visible_feeds.size() - 1; i >= static_cast<int>(curpos);
-		--i) {
+	for (int i = visible_feeds.size() - 1; i >= static_cast<int>(curpos); --i) {
+		const auto unread = visible_feeds[i].first->unread_item_count();
 		LOG(Level::DEBUG,
-			"FeedListFormAction::jump_to_previous_unread_feed: "
-			"visible_feeds[%u] unread items: %u",
+			"FeedListFormAction::jump_to_previous_unread_feed: visible_feeds[%u] unread items: %u",
 			i,
-			visible_feeds[i].first->unread_item_count());
-		if (visible_feeds[i].first->unread_item_count() > 0) {
-			LOG(Level::DEBUG,
-				"FeedListFormAction::jump_to_previous_unread_"
-				"feed:"
-				" hit");
+			unread);
+		if (unread > 0) {
+			LOG(Level::DEBUG, "FeedListFormAction::jump_to_previous_unread_feed: hit");
 			list.set_position(i);
 			feedpos = visible_feeds[i].second;
 			return true;
@@ -652,20 +686,24 @@ bool FeedListFormAction::jump_to_previous_unread_feed(unsigned int& feedpos)
 
 void FeedListFormAction::goto_feed(const std::string& str)
 {
+	if (visible_feeds.empty()) {
+		return;
+	}
+
 	const unsigned int curpos = list.get_position();
 	LOG(Level::DEBUG,
 		"FeedListFormAction::goto_feed: curpos = %u str = `%s'",
 		curpos,
 		str);
 	for (unsigned int i = curpos + 1; i < visible_feeds.size(); ++i) {
-		if (strcasestr(visible_feeds[i].first->title().c_str(),
+		if (strcasestr(get_title(visible_feeds[i].first).c_str(),
 				str.c_str()) != nullptr) {
 			list.set_position(i);
 			return;
 		}
 	}
 	for (unsigned int i = 0; i <= curpos; ++i) {
-		if (strcasestr(visible_feeds[i].first->title().c_str(),
+		if (strcasestr(get_title(visible_feeds[i].first).c_str(),
 				str.c_str()) != nullptr) {
 			list.set_position(i);
 			return;
@@ -695,36 +733,29 @@ bool FeedListFormAction::jump_to_next_unread_feed(unsigned int& feedpos)
 {
 	const unsigned int curpos = list.get_position();
 	LOG(Level::DEBUG,
-		"FeedListFormAction::jump_to_next_unread_feed: searching for "
-		"unread feed");
+		"FeedListFormAction::jump_to_next_unread_feed: searching for unread feed");
 
 	for (unsigned int i = curpos + 1; i < visible_feeds.size(); ++i) {
+		const auto unread = visible_feeds[i].first->unread_item_count();
 		LOG(Level::DEBUG,
-			"FeedListFormAction::jump_to_next_unread_feed: "
-			"visible_feeds[%u] unread items: %u",
+			"FeedListFormAction::jump_to_next_unread_feed: visible_feeds[%u] unread items: %u",
 			i,
-			visible_feeds[i].first->unread_item_count());
-		if (visible_feeds[i].first->unread_item_count() > 0) {
-			LOG(Level::DEBUG,
-				"FeedListFormAction::jump_to_next_unread_feed:"
-				" "
-				"hit");
+			unread);
+		if (unread > 0) {
+			LOG(Level::DEBUG, "FeedListFormAction::jump_to_next_unread_feed: hit");
 			list.set_position(i);
 			feedpos = visible_feeds[i].second;
 			return true;
 		}
 	}
-	for (unsigned int i = 0; i <= curpos; ++i) {
+	for (unsigned int i = 0; i <= curpos && i < visible_feeds.size(); ++i) {
+		const auto unread = visible_feeds[i].first->unread_item_count();
 		LOG(Level::DEBUG,
-			"FeedListFormAction::jump_to_next_unread_feed: "
-			"visible_feeds[%u] unread items: %u",
+			"FeedListFormAction::jump_to_next_unread_feed: visible_feeds[%u] unread items: %u",
 			i,
-			visible_feeds[i].first->unread_item_count());
-		if (visible_feeds[i].first->unread_item_count() > 0) {
-			LOG(Level::DEBUG,
-				"FeedListFormAction::jump_to_next_unread_feed:"
-				" "
-				"hit");
+			unread);
+		if (unread > 0) {
+			LOG(Level::DEBUG, "FeedListFormAction::jump_to_next_unread_feed: hit");
 			list.set_position(i);
 			feedpos = visible_feeds[i].second;
 			return true;
@@ -797,26 +828,41 @@ void FeedListFormAction::handle_cmdline(const std::string& cmd)
 		handle_cmdline_num(idx);
 	} else {
 		// hand over all other commands to formaction
-		std::vector<std::string> tokens =
-			utils::tokenize_quoted(cmd, " \t");
-		if (!tokens.empty()) {
-			if (tokens[0] == "tag") {
-				if (tokens.size() >= 2 && tokens[1] != "") {
-					tag = tokens[1];
-					do_redraw = true;
-					zero_feedpos = true;
-				}
-			} else if (tokens[0] == "goto") {
-				if (tokens.size() >= 2 && tokens[1] != "") {
-					goto_feed(tokens[1]);
-				}
-			} else {
-				FormAction::handle_cmdline(cmd);
+		constexpr auto delimiters = " \t";
+		const auto command = FormAction::parse_command(cmd, delimiters);
+		switch (command.type) {
+		case CommandType::TAG:
+			if (!command.args.empty()) {
+				handle_tag(command.args.front());
 			}
+			break;
+		case CommandType::GOTO:
+			if (!command.args.empty()) {
+				handle_goto(command.args.front());
+			}
+			break;
+		default:
+			FormAction::handle_parsed_command(command);
 		}
 	}
 }
 
+void FeedListFormAction::handle_tag(const std::string& tag_param)
+{
+	if (tag_param != "") {
+		tag = tag_param;
+		do_redraw = true;
+		zero_feedpos = true;
+	}
+}
+
+void FeedListFormAction::handle_goto(const std::string& param)
+{
+	if (param != "") {
+		goto_feed(param);
+	}
+}
+
 void FeedListFormAction::finished_qna(Operation op)
 {
 	FormAction::finished_qna(op); // important!
@@ -828,6 +874,9 @@ void FeedListFormAction::finished_qna(Operation op)
 	case OP_INT_START_SEARCH:
 		op_start_search();
 		break;
+	case OP_INT_GOTO_TITLE:
+		goto_feed(qna_responses[0]);
+		break;
 	default:
 		break;
 	}
@@ -896,10 +945,11 @@ void FeedListFormAction::update_form_title(unsigned int width)
 	fmt.register_fmt('N', PROGRAM_NAME);
 	fmt.register_fmt('V', utils::program_version());
 	fmt.register_fmt('u', std::to_string(count_unread_feeds()));
+	fmt.register_fmt('U', std::to_string(count_unread_articles()));
 	fmt.register_fmt('t', std::to_string(visible_feeds.size()));
-	fmt.register_fmt('F', apply_filter ? matcher.get_expression() : "");
+	fmt.register_fmt('F', filter_active ? matcher.get_expression() : "");
 
-	f.set("head", fmt.do_format(title_format, width));
+	set_value("head", fmt.do_format(title_format, width));
 }
 
 unsigned int FeedListFormAction::count_unread_feeds()
@@ -912,20 +962,19 @@ unsigned int FeedListFormAction::count_unread_feeds()
 	});
 }
 
+unsigned int FeedListFormAction::count_unread_articles()
+{
+	unsigned int total = 0;
+	for (const auto& feed : visible_feeds) {
+		total += feed.first->unread_item_count();
+	}
+	return total;
+}
+
 void FeedListFormAction::op_end_setfilter()
 {
 	std::string filtertext = qna_responses[0];
-	filterhistory.add_line(filtertext);
-	if (filtertext.length() > 0) {
-		if (!matcher.parse(filtertext)) {
-			v->show_error(
-				_("Error: couldn't parse filter command!"));
-		} else {
-			save_filterpos();
-			apply_filter = true;
-			do_redraw = true;
-		}
-	}
+	apply_filter(filtertext);
 }
 
 void FeedListFormAction::op_start_search()
@@ -936,28 +985,28 @@ void FeedListFormAction::op_start_search()
 		"`%s'",
 		searchphrase);
 	if (searchphrase.length() > 0) {
-		v->set_status(_("Searching..."));
+		const auto message_lifetime = v->get_statusline().show_message_until_finished(
+				_("Searching..."));
 		searchhistory.add_line(searchphrase);
 		std::vector<std::shared_ptr<RssItem>> items;
 		try {
-			std::string utf8searchphrase = utils::convert_text(
-					searchphrase, "utf-8", nl_langinfo(CODESET));
+			const auto utf8searchphrase = utils::locale_to_utf8(searchphrase);
 			items = v->get_ctrl()->search_for_items(
 					utf8searchphrase, nullptr);
 		} catch (const DbException& e) {
-			v->show_error(strprintf::fmt(
+			v->get_statusline().show_error(strprintf::fmt(
 					_("Error while searching for `%s': %s"),
 					searchphrase,
 					e.what()));
 			return;
 		}
 		if (!items.empty()) {
-			std::shared_ptr<RssFeed> search_dummy_feed(new RssFeed(cache));
+			std::shared_ptr<RssFeed> search_dummy_feed(new RssFeed(cache, ""));
 			search_dummy_feed->set_search_feed(true);
 			search_dummy_feed->add_items(items);
 			v->push_searchresult(search_dummy_feed, searchphrase);
 		} else {
-			v->show_error(_("No results."));
+			v->get_statusline().show_error(_("No results."));
 		}
 	}
 }
@@ -968,12 +1017,12 @@ void FeedListFormAction::handle_cmdline_num(unsigned int idx)
 		idx <= (visible_feeds[visible_feeds.size() - 1].second + 1)) {
 		int i = get_pos(idx - 1);
 		if (i == -1) {
-			v->show_error(_("Position not visible!"));
+			v->get_statusline().show_error(_("Position not visible!"));
 		} else {
 			list.set_position(i);
 		}
 	} else {
-		v->show_error(_("Invalid position!"));
+		v->get_statusline().show_error(_("Invalid position!"));
 	}
 }
 
@@ -1034,6 +1083,12 @@ std::string FeedListFormAction::format_line(const std::string& feedlist_format,
 
 	auto formattedLine = fmt.do_format(feedlist_format, width);
 	formattedLine = utils::quote_for_stfl(formattedLine);
+
+	const int id = rxman.feed_matches(feed.get());
+	if (id != -1) {
+		formattedLine = strprintf::fmt("<%d>%s</>", id, formattedLine);
+	}
+
 	if (unread_count > 0) {
 		formattedLine = strprintf::fmt("<unread>%s</>", formattedLine);
 	}
@@ -1048,4 +1103,23 @@ std::string FeedListFormAction::title()
 			static_cast<unsigned int>(visible_feeds.size()));
 }
 
+void FeedListFormAction::apply_filter(const std::string& filtertext)
+{
+	if (filtertext.empty()) {
+		return;
+	}
+
+	filterhistory.add_line(filtertext);
+	if (!matcher.parse(filtertext)) {
+		v->get_statusline().show_error(strprintf::fmt(
+				_("Error: couldn't parse filter expression `%s': %s"),
+				filtertext,
+				matcher.get_parse_error()));
+	} else {
+		save_filterpos();
+		filter_active = true;
+		do_redraw = true;
+	}
+}
+
 } // namespace newsboat
