@@ -3,7 +3,7 @@
 #include <algorithm>
 #include <cinttypes>
 #include <cstring>
-#include <curses.h>
+#include <ncurses.h>
 #include <dirent.h>
 #include <grp.h>
 #include <iomanip>
@@ -31,9 +31,11 @@ FileBrowserFormAction::FileBrowserFormAction(View* vv,
 	, quit(false)
 	, files_list("files", FormAction::f, cfg->get_configvalue_as_int("scrolloff"))
 {
-	// In filebrowser, keyboard focus is at the input field, so user can't
-	// possibly use 'q' key to exit the dialog
+	// In filebrowser, keyboard focus is at the input field, so user will be
+	// unable to use alphanumeric keys to confirm or quit the dialog (e.g. they
+	// can't quit with the default `q` bind).
 	KeyMap* keys = vv->get_keymap();
+	keys->set_key(OP_OPEN, "ENTER", id());
 	keys->set_key(OP_QUIT, "ESC", id());
 	vv->set_keymap(keys);
 }
@@ -55,20 +57,17 @@ bool FileBrowserFormAction::process_operation(Operation op,
 		 * needs to be returned.
 		 */
 		LOG(Level::DEBUG, "FileBrowserFormAction: 'opening' item");
-		std::string focus = f.get_focus();
+		const std::string focus = f.get_focus();
 		if (focus.length() > 0) {
 			if (focus == "files") {
 				const auto selected_position = files_list.get_position();
-				std::string selection = id_at_position[selected_position];
-				const char filetype = selection[0];
-				selection.erase(0, 1);
-				const std::string filename(selection);
-				switch (filetype) {
-				case 'd': {
-					int status = ::chdir(filename.c_str());
+				const auto selection = id_at_position[selected_position];
+				switch (selection.filetype) {
+				case file_system::FileType::Directory: {
+					const int status = ::chdir(selection.name.c_str());
 					LOG(Level::DEBUG,
 						"FileBrowserFormAction:OP_OPEN: chdir(%s) = %i",
-						filename,
+						selection.name,
 						status);
 					files_list.set_position(0);
 					std::string fn = utils::getcwd();
@@ -78,9 +77,9 @@ bool FileBrowserFormAction::process_operation(Operation op,
 						fn.push_back(NEWSBEUTER_PATH_SEP);
 					}
 
-					std::string fnstr =
+					const std::string fnstr =
 						f.get("filenametext");
-					std::string::size_type base =
+					const std::string::size_type base =
 						fnstr.find_last_of(NEWSBEUTER_PATH_SEP);
 					if (base == std::string::npos) {
 						fn.append(fnstr);
@@ -89,17 +88,17 @@ bool FileBrowserFormAction::process_operation(Operation op,
 							base + 1,
 							std::string::npos);
 					}
-					f.set("filenametext", fn);
+					set_value("filenametext", fn);
 					do_redraw = true;
 				}
 				break;
-				case '-': {
+				case file_system::FileType::RegularFile: {
 					std::string fn = utils::getcwd();
 					if (fn.back() != NEWSBEUTER_PATH_SEP) {
 						fn.push_back(NEWSBEUTER_PATH_SEP);
 					}
-					fn.append(filename);
-					f.set("filenametext", fn);
+					fn.append(selection.name);
+					set_value("filenametext", fn);
 					f.set_focus("filename");
 				}
 				break;
@@ -157,7 +156,8 @@ bool FileBrowserFormAction::process_operation(Operation op,
 	case OP_NEXT:
 	case OP_SK_DOWN:
 		if (f.get_focus() == "files") {
-			if (!files_list.move_down(cfg->get_configvalue_as_bool("wrap-scroll"))) {
+			if (!files_list.move_down(
+					cfg->get_configvalue_as_bool("wrap-scroll"))) {
 				f.set_focus("filename");
 			}
 		}
@@ -166,7 +166,7 @@ bool FileBrowserFormAction::process_operation(Operation op,
 		if (f.get_focus() == "files") {
 			files_list.move_to_first();
 		} else {
-			f.set("filenametext_pos", "0");
+			set_value("filenametext_pos", "0");
 		}
 		break;
 	case OP_SK_END:
@@ -174,31 +174,43 @@ bool FileBrowserFormAction::process_operation(Operation op,
 			files_list.move_to_last();
 		} else {
 			const std::size_t text_length = f.get("filenametext").length();
-			f.set("filenametext_pos", std::to_string(text_length));
+			set_value("filenametext_pos", std::to_string(text_length));
 		}
 		break;
 	case OP_SK_PGUP:
 		if (f.get_focus() == "files") {
-			files_list.move_page_up(cfg->get_configvalue_as_bool("wrap-scroll"));
+			files_list.move_page_up(
+				cfg->get_configvalue_as_bool("wrap-scroll"));
 		}
 		break;
 	case OP_SK_PGDOWN:
 		if (f.get_focus() == "files") {
-			files_list.move_page_down(cfg->get_configvalue_as_bool("wrap-scroll"));
+			files_list.move_page_down(
+				cfg->get_configvalue_as_bool("wrap-scroll"));
+		}
+		break;
+	case OP_SK_HALF_PAGE_UP:
+		if (f.get_focus() == "files") {
+			files_list.scroll_halfpage_up(cfg->get_configvalue_as_bool("wrap-scroll"));
+		}
+		break;
+	case OP_SK_HALF_PAGE_DOWN:
+		if (f.get_focus() == "files") {
+			files_list.scroll_halfpage_down(cfg->get_configvalue_as_bool("wrap-scroll"));
 		}
 		break;
 	case OP_QUIT:
 		LOG(Level::DEBUG, "view::filebrowser: quitting");
 		curs_set(0);
 		v->pop_current_formaction();
-		f.set("filenametext", "");
+		set_value("filenametext", "");
 		break;
 	case OP_HARDQUIT:
 		LOG(Level::DEBUG, "view::filebrowser: hard quitting");
 		while (v->formaction_stack_size() > 0) {
 			v->pop_current_formaction();
 		}
-		f.set("filenametext", "");
+		set_value("filenametext", "");
 		break;
 	default:
 		break;
@@ -218,7 +230,7 @@ void FileBrowserFormAction::update_title(const std::string& working_directory)
 	const std::string title = fmt.do_format(
 			cfg->get_configvalue("filebrowser-title-format"), width);
 
-	f.set("head", title);
+	set_value("head", title);
 }
 
 std::vector<std::string> get_sorted_filelist()
@@ -257,6 +269,9 @@ void FileBrowserFormAction::prepare()
 	 * in the current directory.
 	 */
 	if (do_redraw) {
+		const std::string cwdtmp = utils::getcwd();
+		update_title(cwdtmp);
+
 		std::vector<std::string> files = get_sorted_filelist();
 
 		ListFormatter listfmt;
@@ -282,55 +297,45 @@ void FileBrowserFormAction::init()
 {
 	set_keymap_hints();
 
-	f.set("fileprompt", _("File: "));
+	set_value("fileprompt", _("File: "));
 
-	if (dir == "") {
-		std::string save_path = cfg->get_configvalue("save-path");
+	const std::string save_path = cfg->get_configvalue("save-path");
 
-		LOG(Level::DEBUG,
-			"view::filebrowser: save-path is '%s'",
-			save_path);
+	LOG(Level::DEBUG,
+		"view::filebrowser: save-path is '%s'",
+		save_path);
 
-		dir = save_path;
-	}
+	const int status = ::chdir(save_path.c_str());
+	LOG(Level::DEBUG, "view::filebrowser: chdir(%s) = %i", save_path, status);
 
-	int status = ::chdir(dir.c_str());
-	LOG(Level::DEBUG, "view::filebrowser: chdir(%s) = %i", dir, status);
-
-	const std::string cwdtmp = utils::getcwd();
-
-	f.set("filenametext", default_filename);
+	set_value("filenametext", default_filename);
 
 	// Set position to 0 and back to ensure that the text is visible
-	f.run(-1);
-	f.set("filenametext_pos", std::to_string(default_filename.length()));
-
-	update_title(cwdtmp);
+	draw_form();
+	set_value("filenametext_pos", std::to_string(default_filename.length()));
 }
 
-KeyMapHintEntry* FileBrowserFormAction::get_keymap_hint()
+const std::vector<KeyMapHintEntry>& FileBrowserFormAction::get_keymap_hint() const
 {
-	static KeyMapHintEntry hints[] = {{OP_QUIT, _("Cancel")},
-		{OP_OPEN, _("Save")},
-		{OP_NIL, nullptr}
+	static const std::vector<KeyMapHintEntry> hints = {{OP_QUIT, _("Cancel")},
+		{OP_OPEN, _("Save")}
 	};
 	return hints;
 }
 
 void FileBrowserFormAction::add_file(
 	ListFormatter& listfmt,
-	std::vector<std::string>& id_at_position,
+	std::vector<file_system::FileSystemEntry>& id_at_position,
 	std::string filename)
 {
 	struct stat sb;
 	if (::lstat(filename.c_str(), &sb) == 0) {
-		char ftype = get_filetype(sb.st_mode);
+		const auto ftype = file_system::mode_to_filetype(sb.st_mode);
 
-		std::string rwxbits = get_rwx(sb.st_mode & 0777);
-		std::string owner = get_owner(sb.st_uid);
-		std::string group = get_group(sb.st_gid);
-		std::string formattedfilename =
-			get_formatted_filename(filename, ftype, sb.st_mode);
+		const auto rwxbits = file_system::permissions_string(sb.st_mode);
+		const auto owner = file_system::get_user_padded(sb.st_uid);
+		const auto group = file_system::get_group_padded(sb.st_gid);
+		std::string formattedfilename = get_formatted_filename(filename, sb.st_mode);
 
 		std::string sizestr = strprintf::fmt(
 				"%12" PRIi64,
@@ -339,106 +344,26 @@ void FileBrowserFormAction::add_file(
 				// bits.
 				static_cast<int64_t>(sb.st_size));
 		std::string line = strprintf::fmt("%c%s %s %s %s %s",
-				ftype,
+				file_system::filetype_to_char(ftype),
 				rwxbits,
 				owner,
 				group,
 				sizestr,
 				formattedfilename);
 		listfmt.add_line(utils::quote_for_stfl(line));
-		const std::string id = strprintf::fmt("%c%s", ftype, filename);
-		id_at_position.push_back(id);
+		id_at_position.push_back(file_system::FileSystemEntry{ftype, filename});
 	}
 }
 
 std::string FileBrowserFormAction::get_formatted_filename(std::string filename,
-	char ftype,
 	mode_t mode)
 {
-	char suffix = 0;
-
-	switch (ftype) {
-	case 'd':
-		suffix = '/';
-		break;
-	case 'l':
-		suffix = '@';
-		break;
-	case 's':
-		suffix = '=';
-		break;
-	case 'p':
-		suffix = '|';
-		break;
-	default:
-		if (mode & S_IXUSR) {
-			suffix = '*';
-		}
+	const auto suffix = file_system::mode_suffix(mode);
+	if (suffix.has_value()) {
+		return strprintf::fmt("%s%c", filename, suffix.value());
+	} else {
+		return filename;
 	}
-
-	return strprintf::fmt("%s%c", filename, suffix);
-}
-
-std::string FileBrowserFormAction::get_rwx(unsigned short val)
-{
-	std::string str;
-	const char* bitstrs[] = {
-		"---", "--x", "-w-", "-wx", "r--", "r-x", "rw-", "rwx"
-	};
-	for (int i = 0; i < 3; ++i) {
-		unsigned char bits = val % 8;
-		val /= 8;
-		str.insert(0, bitstrs[bits]);
-	}
-	return str;
-}
-
-char FileBrowserFormAction::get_filetype(mode_t mode)
-{
-	static struct FlagChar {
-		mode_t flag;
-		char ftype;
-	} flags[] = {{S_IFREG, '-'},
-		{S_IFDIR, 'd'},
-		{S_IFBLK, 'b'},
-		{S_IFCHR, 'c'},
-		{S_IFIFO, 'p'},
-		{S_IFLNK, 'l'},
-		{S_IFSOCK, 's'},
-		{0, 0}
-	};
-	for (unsigned int i = 0; flags[i].flag != 0; i++) {
-		if ((mode & S_IFMT) == flags[i].flag) {
-			return flags[i].ftype;
-		}
-	}
-	return '?';
-}
-
-std::string FileBrowserFormAction::get_owner(uid_t uid)
-{
-	struct passwd* spw = getpwuid(uid);
-	if (spw) {
-		std::string owner = spw->pw_name;
-		for (int i = owner.length(); i < 8; ++i) {
-			owner.append(" ");
-		}
-		return owner;
-	}
-	return "????????";
-}
-
-std::string FileBrowserFormAction::get_group(gid_t gid)
-{
-	struct group* sgr = getgrgid(gid);
-	if (sgr) {
-		std::string group = sgr->gr_name;
-		for (int i = group.length(); i < 8; ++i) {
-			group.append(" ");
-		}
-		return group;
-	}
-	return "????????";
 }
 
 std::string FileBrowserFormAction::title()
