@@ -1,7 +1,9 @@
 #include "fileurlreader.h"
 
+#include <cstring>
 #include <fstream>
 
+#include "strprintf.h"
 #include "utils.h"
 
 namespace newsboat {
@@ -16,19 +18,19 @@ std::string FileUrlReader::get_source()
 	return filename;
 }
 
-void FileUrlReader::reload()
+nonstd::optional<utils::ReadTextFileError> FileUrlReader::reload()
 {
 	urls.clear();
 	tags.clear();
 	alltags.clear();
 
-	std::fstream f;
-	f.open(filename, std::fstream::in);
-	if (!f.is_open()) {
-		return;
+	auto result = utils::read_text_file(filename);
+	if (!result) {
+		return result.error();
 	}
+	std::vector<std::string> lines = result.value();
 
-	for (std::string line; std::getline(f, line); /* nothing */) {
+	for (const std::string& line : lines) {
 		// skip empty lines and comments
 		if (line.empty() || line[0] == '#') {
 			continue;
@@ -50,23 +52,32 @@ void FileUrlReader::reload()
 			}
 		}
 	};
+
+	return {};
 }
 
-void FileUrlReader::write_config()
+nonstd::optional<std::string> FileUrlReader::write_config()
 {
 	std::fstream f;
 	f.open(filename, std::fstream::out);
-	if (f.is_open()) {
-		for (const auto& url : urls) {
-			f << url;
-			if (tags[url].size() > 0) {
-				for (const auto& tag : tags[url]) {
-					f << " \"" << tag << "\"";
-				}
-			}
-			f << std::endl;
-		}
+	if (!f.is_open()) {
+		const auto error_message = strerror(errno);
+		return strprintf::fmt(_("Error: failed to open file \"%s\": %s"),
+				filename,
+				error_message);
 	}
+
+	for (const auto& url : urls) {
+		f << url;
+		if (tags[url].size() > 0) {
+			for (const auto& tag : tags[url]) {
+				f << " \"" << tag << "\"";
+			}
+		}
+		f << std::endl;
+	}
+
+	return {};
 }
 
 }
