@@ -1,7 +1,10 @@
 #include "filtercontainer.h"
 
+#include <algorithm>
+
 #include "config.h"
 #include "confighandlerexception.h"
+#include "configparser.h"
 #include "matcher.h"
 #include "strprintf.h"
 #include "utils.h"
@@ -21,8 +24,7 @@ void FilterContainer::handle_action(const std::string& action,
 	 */
 	if (action == "define-filter") {
 		if (params.size() < 2) {
-			throw ConfigHandlerException(
-				ActionHandlerStatus::TOO_FEW_PARAMS);
+			throw ConfigHandlerException(ActionHandlerStatus::TOO_FEW_PARAMS);
 		}
 
 		FilterNameExprPair filter;
@@ -39,12 +41,11 @@ void FilterContainer::handle_action(const std::string& action,
 
 		filters.emplace_back(std::move(filter));
 	} else {
-		throw ConfigHandlerException(
-			ActionHandlerStatus::INVALID_COMMAND);
+		throw ConfigHandlerException(ActionHandlerStatus::INVALID_COMMAND);
 	}
 }
 
-void FilterContainer::dump_config(std::vector<std::string>& config_output)
+void FilterContainer::dump_config(std::vector<std::string>& config_output) const
 {
 	for (const auto& filter : filters) {
 		config_output.push_back(strprintf::fmt("define-filter %s %s",
@@ -53,4 +54,18 @@ void FilterContainer::dump_config(std::vector<std::string>& config_output)
 	}
 }
 
+nonstd::optional<std::string> FilterContainer::get_filter(const std::string& name)
+{
+	const auto filter = std::find_if(filters.begin(),
+	filters.end(), [&](const FilterNameExprPair& pair) {
+		return pair.name == name;
+	});
+
+	if (filter != filters.end()) {
+		return filter->expr;
+	} else {
+		return {};
+	}
+}
+
 } // namespace newsboat
