@@ -2,10 +2,14 @@
 
 #include <cassert>
 #include <cinttypes>
+#include <exception>
 #include <ncurses.h>
 
 #include "config.h"
 #include "configexception.h"
+#include "controller.h"
+#include "listmovementcontrol.h"
+#include "listwidgetbackend.h"
 #include "logger.h"
 #include "matcherexception.h"
 #include "strprintf.h"
@@ -27,10 +31,10 @@ FormAction::FormAction(View* vv, std::string formstr, ConfigContainer* cfg)
 {
 	if (v) {
 		if (cfg->get_configvalue_as_bool("show-keymap-hint") == false) {
-			f.set("showhint", "0");
+			set_value("showhint", "0");
 		}
 		if (cfg->get_configvalue_as_bool("show-title-bar") == false) {
-			f.set("showtitle", "0");
+			set_value("showtitle", "0");
 		}
 		if (cfg->get_configvalue_as_bool("swap-title-and-hints") ==
 			true) {
@@ -46,53 +50,44 @@ FormAction::FormAction(View* vv, std::string formstr, ConfigContainer* cfg)
 	valid_cmds.push_back("quit");
 	valid_cmds.push_back("source");
 	valid_cmds.push_back("dumpconfig");
-	valid_cmds.push_back("dumpform");
 	valid_cmds.push_back("exec");
 }
 
 void FormAction::set_keymap_hints()
 {
-	f.set("help", prepare_keymap_hint(this->get_keymap_hint()));
-}
-
-void FormAction::recalculate_form()
-{
-	f.run(-3);
+	set_value("help", v->get_keymap()->prepare_keymap_hint(this->get_keymap_hint(),
+			this->id()));
 }
 
 FormAction::~FormAction() {}
 
-Stfl::Form& FormAction::get_form()
+std::string FormAction::get_value(const std::string& name)
 {
-	return f;
+	return f.get(name);
 }
 
-std::string FormAction::prepare_keymap_hint(KeyMapHintEntry* hints)
+void FormAction::set_value(const std::string& name, const std::string& value)
 {
-	/*
-	 * This function generates the "keymap hint" line by putting
-	 * together the elements of a structure, and looking up the
-	 * currently set keybinding so that the "keymap hint" line always
-	 * reflects the current configuration.
-	 */
-	std::string keymap_hint;
-	for (int i = 0; hints[i].op != OP_NIL; ++i) {
-		std::string bound_keys = utils::join(v->get_keymap()->get_keys(hints[i].op,
-					this->id()), ",");
-		if (bound_keys.empty()) {
-			bound_keys = "<none>";
-		}
-		keymap_hint.append(bound_keys);
-		keymap_hint.append(":");
-		keymap_hint.append(hints[i].text);
-		keymap_hint.append(" ");
+	f.set(name, value);
+}
+
+void FormAction::draw_form()
+{
+	f.run(-1);
+}
+
+std::string FormAction::draw_form_wait_for_event(unsigned int timeout)
+{
+	const char* event = f.run(timeout);
+	if (event == nullptr) {
+		return "";
 	}
-	return keymap_hint;
+	return std::string(event);
 }
 
-std::string FormAction::get_value(const std::string& value)
+void FormAction::recalculate_widget_dimensions()
 {
-	return f.get(value);
+	f.run(-3);
 }
 
 void FormAction::start_cmdline(std::string default_value)
@@ -117,55 +112,26 @@ bool FormAction::process_op(Operation op,
 		break;
 	case OP_INT_SET:
 		if (automatic) {
-			std::string cmdline = "set ";
-			if (args) {
-				for (const auto& arg : *args) {
-					cmdline.append(strprintf::fmt(
-							"%s ", Stfl::quote(arg)));
+			if (args && args->size() == 2) {
+				const std::string key = args->at(0);
+				const std::string value = args->at(1);
+				cfg->set_configvalue(key, value);
+				set_redraw(true);
+				return true;
+			}
+			if (args && args->size() == 1) {
+				if (handle_single_argument_set(args->at(0))) {
+					return true;
 				}
 			}
-			LOG(Level::DEBUG,
-				"FormAction::process_op: running commandline "
-				"`%s'",
-				cmdline);
-			this->handle_cmdline(cmdline);
+			v->get_statusline().show_error(_("usage: set <config-option> <value>"));
+			return false;
 		} else {
 			LOG(Level::WARN,
 				"FormAction::process_op: got OP_INT_SET, but "
-				"not "
-				"automatic");
+				"not automatic");
 		}
 		break;
-	case OP_INT_CANCEL_QNA:
-		f.modify("lastline",
-			"replace",
-			"{hbox[lastline] .expand:0 {label[msglabel] .expand:h "
-			"text[msg]:\"\"}}");
-		v->inside_qna(false);
-		v->inside_cmdline(false);
-		break;
-	case OP_INT_QNA_NEXTHIST:
-		if (qna_history) {
-			std::string entry = qna_history->next_line();
-			f.set("qna_value", entry);
-			f.set("qna_value_pos", std::to_string(entry.length()));
-		}
-		break;
-	case OP_INT_QNA_PREVHIST:
-		if (qna_history) {
-			std::string entry = qna_history->previous_line();
-			f.set("qna_value", entry);
-			f.set("qna_value_pos", std::to_string(entry.length()));
-		}
-		break;
-	case OP_INT_END_QUESTION:
-		/*
-		 * An answer has been entered, we save the value, and ask the
-		 * next question.
-		 */
-		qna_responses.push_back(f.get("qna_value"));
-		start_next_question();
-		break;
 	case OP_VIEWDIALOGS:
 		v->view_dialogs();
 		break;
@@ -263,96 +229,163 @@ void FormAction::handle_cmdline(const std::string& cmdline)
 	 * It works the same way basically everywhere: first the command line
 	 * is tokenized, and then the tokens are looked at.
 	 */
-	std::vector<std::string> tokens =
-		utils::tokenize_quoted(cmdline, " \t=");
+	constexpr auto delimiters = " \t=";
+	const auto command = FormAction::parse_command(cmdline, delimiters);
 	assert(cfg != nullptr);
-	if (!tokens.empty()) {
-		std::string cmd = tokens[0];
-		tokens.erase(tokens.begin());
-		if (cmd == "set") {
-			if (tokens.empty()) {
-				v->show_error(
-					_("usage: set <variable>[=<value>]"));
-			} else if (tokens.size() == 1) {
-				std::string var = tokens[0];
-				if (var.length() > 0) {
-					if (var[var.length() - 1] == '!') {
-						var.erase(var.length() - 1);
-						cfg->toggle(var);
-						set_redraw(true);
-					} else if (var[var.length() - 1] ==
-						'&') {
-						var.erase(var.length() - 1);
-						cfg->reset_to_default(var);
-						set_redraw(true);
-					}
-					v->set_status(strprintf::fmt("  %s=%s",
-							var,
-							utils::quote_if_necessary(
-								cfg->get_configvalue(
-									var))));
-				}
-			} else if (tokens.size() == 2) {
-				std::string result =
-					ConfigParser::evaluate_backticks(
-						tokens[1]);
-				utils::trim_end(result);
-				cfg->set_configvalue(tokens[0], result);
-				// because some configuration value might have changed something UI-related
-				set_redraw(true);
-			} else {
-				v->show_error(
-					_("usage: set <variable>[=<value>]"));
+	handle_parsed_command(command);
+}
+
+void FormAction::handle_set(const std::vector<std::string>& args)
+{
+	if (args.size() == 1) {
+		if (handle_single_argument_set(args[0])) {
+			return;
+		}
+		v->get_statusline().show_message(strprintf::fmt("  %s=%s",
+				args[0],
+				utils::quote_if_necessary(cfg->get_configvalue(args[0]))));
+	} else if (args.size() == 2) {
+		std::string result = ConfigParser::evaluate_backticks(args[1]);
+		utils::trim_end(result);
+		cfg->set_configvalue(args[0], result);
+		// because some configuration value might have changed something UI-related
+		set_redraw(true);
+	} else {
+		v->get_statusline().show_error(
+			_("usage: set <variable>[=<value>]"));
+	}
+}
+
+void FormAction::handle_quit()
+{
+	while (v->formaction_stack_size() > 0) {
+		v->pop_current_formaction();
+	}
+}
+
+void FormAction::handle_source(const std::vector<std::string>& args)
+{
+	if (args.empty()) {
+		v->get_statusline().show_error(_("usage: source <file> [...]"));
+	} else {
+		for (const auto& param : args) {
+			try {
+				v->get_ctrl()->load_configfile(
+					utils::resolve_tilde(
+						param));
+			} catch (const ConfigException& ex) {
+				v->get_statusline().show_error(ex.what());
+				break;
 			}
-		} else if (cmd == "q" || cmd == "quit") {
-			while (v->formaction_stack_size() > 0) {
-				v->pop_current_formaction();
-			}
-		} else if (cmd == "source") {
-			if (tokens.empty()) {
-				v->show_error(_("usage: source <file> [...]"));
-			} else {
-				for (const auto& token : tokens) {
-					try {
-						v->get_ctrl()->load_configfile(
-							utils::resolve_tilde(
-								token));
-					} catch (const ConfigException& ex) {
-						v->show_error(ex.what());
-						break;
-					}
-				}
-			}
-		} else if (cmd == "dumpconfig") {
-			if (tokens.size() != 1) {
-				v->show_error(_("usage: dumpconfig <file>"));
-			} else {
-				v->get_ctrl()->dump_config(
-					utils::resolve_tilde(tokens[0]));
-				v->show_error(strprintf::fmt(
-						_("Saved configuration to %s"),
-						tokens[0]));
-			}
-		} else if (cmd == "dumpform") {
-			v->dump_current_form();
-		} else if (cmd == "exec") {
-			if (tokens.size() != 1) {
-				v->show_error(_("usage: exec <operation>"));
-			} else {
-				const auto op = v->get_keymap()->get_opcode(tokens[0]);
-				if (op != OP_NIL) {
-					process_op(op);
-				} else {
-					v->show_error(_("Operation not found"));
-				}
-			}
-		} else {
-			v->show_error(strprintf::fmt(
-					_("Not a command: %s"), cmdline));
 		}
 	}
 }
 
+void FormAction::handle_dumpconfig(const std::vector<std::string>& args)
+{
+	if (args.size() != 1) {
+		v->get_statusline().show_error(_("usage: dumpconfig <file>"));
+	} else {
+		v->get_ctrl()->dump_config(
+			utils::resolve_tilde(args[0]));
+		v->get_statusline().show_message(strprintf::fmt(
+				_("Saved configuration to %s"),
+				args[0]));
+	}
+}
+
+void FormAction::handle_exec(const std::vector<std::string>& args)
+{
+	if (args.size() != 1) {
+		v->get_statusline().show_error(_("usage: exec <operation>"));
+	} else {
+		const auto op = v->get_keymap()->get_opcode(args[0]);
+		if (op != OP_NIL) {
+			process_op(op);
+		} else {
+			v->get_statusline().show_error(_("Operation not found"));
+		}
+	}
+}
+
+void FormAction::handle_parsed_command(const Command& command)
+{
+	switch (command.type) {
+	case CommandType::SET:
+		handle_set(command.args);
+		break;
+	case CommandType::QUIT:
+		handle_quit();
+		break;
+	case CommandType::SOURCE:
+		handle_source(command.args);
+		break;
+	case CommandType::DUMPCONFIG:
+		handle_dumpconfig(command.args);
+		break;
+	case CommandType::EXEC:
+		handle_exec(command.args);
+		break;
+	case CommandType::UNKNOWN:
+		v->get_statusline().show_error(strprintf::fmt(_("Not a command: %s"), command.args[0]));
+		break;
+	case CommandType::INVALID:
+		break;
+	default:
+		break;
+	}
+}
+
+bool FormAction::handle_list_operations(ListWidget& list, Operation op)
+{
+	switch (op) {
+	case OP_SK_UP:
+		list.move_up(cfg->get_configvalue_as_bool("wrap-scroll"));
+		break;
+	case OP_SK_DOWN:
+		list.move_down(cfg->get_configvalue_as_bool("wrap-scroll"));
+		break;
+	case OP_SK_HOME:
+		list.move_to_first();
+		break;
+	case OP_SK_END:
+		list.move_to_last();
+		break;
+	case OP_SK_PGUP:
+		list.move_page_up(cfg->get_configvalue_as_bool("wrap-scroll"));
+		break;
+	case OP_SK_PGDOWN:
+		list.move_page_down(cfg->get_configvalue_as_bool("wrap-scroll"));
+		break;
+	case OP_SK_HALF_PAGE_UP:
+		list.scroll_halfpage_up(cfg->get_configvalue_as_bool("wrap-scroll"));
+		break;
+	case OP_SK_HALF_PAGE_DOWN:
+		list.scroll_halfpage_down(cfg->get_configvalue_as_bool("wrap-scroll"));
+		break;
+	default:
+		return false;
+	}
+	return false;
+}
+
+bool FormAction::handle_single_argument_set(std::string argument)
+{
+	if (argument.size() >= 1 && argument.back() == '!') {
+		argument.pop_back();
+		cfg->toggle(argument);
+		set_redraw(true);
+		return true;
+	}
+	if (argument.size() >= 1 && argument.back() == '&') {
+		argument.pop_back();
+		cfg->reset_to_default(argument);
+		set_redraw(true);
+		return true;
+	}
+	return false;
+}
+
 void FormAction::start_qna(const std::vector<QnaPair>& prompts,
 	Operation finish_op,
 	History* h)
@@ -380,6 +413,45 @@ void FormAction::start_qna(const std::vector<QnaPair>& prompts,
 	start_next_question();
 }
 
+void FormAction::finish_qna_question()
+{
+	qna_responses.push_back(get_value("qna_value"));
+	start_next_question();
+}
+
+void FormAction::cancel_qna()
+{
+	LOG(Level::DEBUG, "FormAction::cancel_qna");
+
+	f.set("show_qna_label", "0");
+	f.set("show_qna_input", "0");
+	f.set("show_msg", "1");
+	f.set("msg", "");
+
+	f.set_focus(main_widget());
+
+	v->inside_qna(false);
+	v->inside_cmdline(false);
+}
+
+void FormAction::qna_next_history()
+{
+	if (qna_history) {
+		std::string entry = qna_history->next_line();
+		set_value("qna_value", entry);
+		set_value("qna_value_pos", std::to_string(entry.length()));
+	}
+}
+
+void FormAction::qna_previous_history()
+{
+	if (qna_history) {
+		std::string entry = qna_history->previous_line();
+		set_value("qna_value", entry);
+		set_value("qna_value_pos", std::to_string(entry.length()));
+	}
+}
+
 void FormAction::finished_qna(Operation op)
 {
 	v->inside_qna(false);
@@ -398,15 +470,15 @@ void FormAction::finished_qna(Operation op)
 	case OP_INT_BM_END: {
 		assert(qna_responses.size() == 4 &&
 			qna_prompts.size() == 0); // everything must be answered
-		v->set_status(_("Saving bookmark..."));
+		v->get_statusline().show_message(_("Saving bookmark..."));
 		std::string retval = bookmark(qna_responses[0],
 				qna_responses[1],
 				qna_responses[2],
 				qna_responses[3]);
 		if (retval.length() == 0) {
-			v->set_status(_("Saved bookmark."));
+			v->get_statusline().show_message(_("Saved bookmark."));
 		} else {
-			v->set_status(
+			v->get_statusline().show_message(
 				_s("Error while saving bookmark: ") + retval);
 			LOG(Level::DEBUG,
 				"FormAction::finished_qna: error while saving "
@@ -416,7 +488,7 @@ void FormAction::finished_qna(Operation op)
 	}
 	break;
 	case OP_INT_END_CMDLINE: {
-		f.set_focus("feeds");
+		f.set_focus(main_widget());
 		std::string cmdline = qna_responses[0];
 		FormAction::cmdlinehistory.add_line(cmdline);
 		LOG(Level::DEBUG, "FormAction: commandline = `%s'", cmdline);
@@ -430,57 +502,50 @@ void FormAction::finished_qna(Operation op)
 
 void FormAction::start_bookmark_qna(const std::string& default_title,
 	const std::string& default_url,
-	const std::string& default_desc,
 	const std::string& default_feed_title)
 {
 	LOG(Level::DEBUG,
 		"FormAction::start_bookmark_qna: starting bookmark Q&A... "
-		"default_title = %s default_url = %s default_desc = %s "
+		"default_title = %s default_url = %s "
 		"default_feed_title = %s",
 		default_title,
 		default_url,
-		default_desc,
 		default_feed_title);
 	std::vector<QnaPair> prompts;
 
-	std::string new_title = "";
-	bool is_bm_autopilot =
-		cfg->get_configvalue_as_bool("bookmark-autopilot");
+	bool is_bm_autopilot = cfg->get_configvalue_as_bool("bookmark-autopilot");
 	prompts.push_back(QnaPair(_("URL: "), default_url));
 	// call the function to figure out title from url only if the default_title is no good
 	if (default_title.empty()) {
-		new_title = utils::make_title(default_url);
-		prompts.push_back(QnaPair(_("Title: "), new_title));
+		prompts.push_back(QnaPair(_("Title: "), utils::make_title(default_url)));
 	} else {
 		prompts.push_back(QnaPair(_("Title: "), utils::utf8_to_locale(default_title)));
 	}
-	prompts.push_back(QnaPair(_("Description: "), default_desc));
+	prompts.push_back(QnaPair(_("Description: "), ""));
 	prompts.push_back(QnaPair(_("Feed title: "), default_feed_title));
 
-	if (is_bm_autopilot) { // If bookmarking is set to autopilot don't
-		// prompt for url, title, desc
+	if (is_bm_autopilot) { // If bookmarking is set to autopilot don't prompt for url, title, desc
+		std::string title;
 		if (default_title.empty()) {
-			new_title = utils::make_title(
-					default_url); // try to make the title from url
+			title = utils::make_title(default_url); // try to make the title from url
 		} else {
 			// assignment just to make the call to bookmark() below easier
-			new_title = utils::utf8_to_locale(default_title);
+			title = utils::utf8_to_locale(default_title);
 		}
 
 		// if url or title is missing, abort autopilot and ask user
-		if (default_url.empty() || new_title.empty() ||
-			default_feed_title.empty()) {
+		if (default_url.empty() || title.empty()) {
 			start_qna(prompts, OP_INT_BM_END);
 		} else {
-			v->set_status(_("Saving bookmark on autopilot..."));
+			v->get_statusline().show_message(_("Saving bookmark on autopilot..."));
 			std::string retval = bookmark(default_url,
-					new_title,
-					default_desc,
+					title,
+					"",
 					default_feed_title);
 			if (retval.length() == 0) {
-				v->set_status(_("Saved bookmark."));
+				v->get_statusline().show_message(_("Saved bookmark."));
 			} else {
-				v->set_status(
+				v->get_statusline().show_message(
 					_s("Error while saving bookmark: ") +
 					retval);
 				LOG(Level::DEBUG,
@@ -494,29 +559,56 @@ void FormAction::start_bookmark_qna(const std::string& default_title,
 	}
 }
 
+Command FormAction::parse_command(const std::string& input,
+	std::string delimiters)
+{
+	auto tokens = utils::tokenize_quoted(input, delimiters);
+	if (tokens.empty()) {
+		return Command { .type = CommandType::INVALID, .args = {} };
+	} else {
+		auto cmd_name = tokens.front();
+		tokens.erase(tokens.begin());
+		if (cmd_name == "set") {
+			return Command { .type = CommandType::SET, .args = std::move(tokens) };
+		} else if (cmd_name == "q" || cmd_name == "quit") {
+			return Command { .type = CommandType::QUIT, .args = std::move(tokens) };
+		} else if (cmd_name == "source") {
+			return Command { .type = CommandType::SOURCE, .args = std::move(tokens) };
+		} else if (cmd_name == "dumpconfig") {
+			return Command { .type = CommandType::DUMPCONFIG, .args = std::move(tokens) };
+		} else if (cmd_name == "exec") {
+			return Command { .type = CommandType::EXEC, .args = std::move(tokens) };
+		} else if (cmd_name == "tag") {
+			return Command { .type = CommandType::TAG, .args = std::move(tokens) };
+		} else if (cmd_name == "goto") {
+			return Command { .type = CommandType::GOTO, .args = std::move(tokens) };
+		} else if (cmd_name == "save") {
+			return Command { .type = CommandType::SAVE, .args = std::move(tokens) };
+		} else {
+			tokens.insert(tokens.begin(), std::move(cmd_name));
+			return Command { .type = CommandType::UNKNOWN, .args = std::move(tokens) };
+		}
+	}
+}
+
 void FormAction::start_next_question()
 {
 	/*
 	 * If there is one more prompt to be presented to the user, set it up.
 	 */
 	if (qna_prompts.size() > 0) {
-		std::string replacestr(
-			"{hbox[lastline] .expand:0 {label .expand:0 text:");
-		replacestr.append(Stfl::quote(qna_prompts[0].first));
-		replacestr.append(
-			"}{input[qnainput] on_ESC:cancel-qna "
-			"on_UP:qna-prev-history on_DOWN:qna-next-history "
-			"on_ENTER:end-question modal:1 .expand:h @bind_home:** "
-			"@bind_end:** text[qna_value]:");
-		replacestr.append(Stfl::quote(qna_prompts[0].second));
-		replacestr.append(" pos[qna_value_pos]:0");
-		replacestr.append("}}");
-		f.modify("lastline", "replace", replacestr);
+		f.set("qna_prompt", qna_prompts[0].first);
+		f.set("qna_value", qna_prompts[0].second);
+
+		f.set("show_qna_label", "1");
+		f.set("show_qna_input", "1");
+		f.set("show_msg", "0");
+
 		f.set_focus("qnainput");
 
 		// Set position to 0 and back to ensure that the text is visible
-		f.run(-1);
-		f.set("qna_value_pos",
+		draw_form();
+		set_value("qna_value_pos",
 			std::to_string(qna_prompts[0].second.length()));
 
 		qna_prompts.erase(qna_prompts.begin());
@@ -526,10 +618,13 @@ void FormAction::start_next_question()
 		 * usual label, and signal the end of the "Q&A" to the
 		 * finished_qna() method.
 		 */
-		f.modify("lastline",
-			"replace",
-			"{hbox[lastline] .expand:0 {label[msglabel] .expand:h "
-			"text[msg]:\"\"}}");
+		f.set("show_qna_label", "0");
+		f.set("show_qna_input", "0");
+		f.set("show_msg", "1");
+		f.set("msg", "");
+
+		f.set_focus(main_widget());
+
 		this->finished_qna(finish_operation);
 	}
 }
@@ -570,8 +665,8 @@ std::string FormAction::bookmark(const std::string& url,
 		if (is_interactive) {
 			v->push_empty_formaction();
 			Stfl::reset();
-			utils::run_interactively(
-				cmdline, "FormAction::bookmark");
+			utils::run_interactively(cmdline, "FormAction::bookmark");
+			v->drop_queued_input();
 			v->pop_current_formaction();
 			return "";
 		} else {
