@@ -52,6 +52,12 @@ bool HelpFormAction::process_operation(Operation op,
 	case OP_SK_PGDOWN:
 		textview.scroll_page_down();
 		break;
+	case OP_SK_HALF_PAGE_UP:
+		textview.scroll_halfpage_up();
+		break;
+	case OP_SK_HALF_PAGE_DOWN:
+		textview.scroll_halfpage_down();
+		break;
 	case OP_QUIT:
 		quit = true;
 		break;
