@@ -3,6 +3,7 @@
 #include <cassert>
 #include <cinttypes>
 #include <cstdio>
+#include <cstring>
 #include <langinfo.h>
 #include <sstream>
 #include <string>
@@ -13,6 +14,7 @@
 #include "dbexception.h"
 #include "fmtstrformatter.h"
 #include "formaction.h"
+#include "itemutils.h"
 #include "logger.h"
 #include "matcherexception.h"
 #include "rssfeed.h"
@@ -30,18 +32,17 @@ ItemListFormAction::ItemListFormAction(View* vv,
 	ConfigContainer* cfg,
 	RegexManager& r)
 	: ListFormAction(vv, formstr, "items", cfg)
+	, old_itempos(-1)
+	, filter_active(false)
 	, pos(0)
-	, apply_filter(false)
-	, show_searchresult(false)
 	, set_filterpos(false)
 	, filterpos(0)
 	, rxman(r)
 	, old_width(0)
-	, old_itempos(-1)
 	, invalidation_mode(InvalidationMode::NONE)
 	, listfmt(&rxman, "articlelist")
 	, rsscache(cc)
-	, filters(f)
+	, filter_container(f)
 {
 	register_format_styles();
 }
@@ -71,8 +72,7 @@ bool ItemListFormAction::process_operation(Operation op,
 			// that
 			old_itempos = itempos;
 			v->push_itemview(feed,
-				visible_items[itempos].first->guid(),
-				show_searchresult ? search_phrase : "");
+				visible_items[itempos].first->guid());
 			invalidate(itempos);
 		} else {
 			v->get_statusline().show_error(
@@ -396,15 +396,9 @@ bool ItemListFormAction::process_operation(Operation op,
 		v->push_help();
 		break;
 	case OP_RELOAD:
-		if (!show_searchresult) {
-			LOG(Level::INFO,
-				"ItemListFormAction: reloading current feed");
-			v->get_ctrl()->get_reloader()->reload(pos);
-			invalidate_list();
-		} else {
-			v->get_statusline().show_error(
-				_("Error: you can't reload search results."));
-		}
+		LOG(Level::INFO, "ItemListFormAction: reloading current feed");
+		v->get_ctrl()->get_reloader()->reload(pos);
+		invalidate_list();
 		break;
 	case OP_QUIT:
 		LOG(Level::INFO, "ItemListFormAction: quitting");
@@ -498,7 +492,7 @@ bool ItemListFormAction::process_operation(Operation op,
 				}
 				rsscache->mark_items_read_by_guid(guids);
 
-				if (apply_filter) {
+				if (filter_active) {
 					// We're only viewing a subset of items, so mark them off one by one.
 					v->get_ctrl()->mark_all_read(guids);
 				} else {
@@ -623,36 +617,21 @@ bool ItemListFormAction::process_operation(Operation op,
 		v->get_ctrl()->edit_urls_file();
 		break;
 	case OP_SELECTFILTER:
-		if (filters.size() > 0) {
+		if (filter_container.size() > 0) {
 			std::string newfilter;
-			if (automatic) {
-				if (args->size() > 0) {
-					newfilter = (*args)[0];
+			if (automatic && args->size() > 0) {
+				const std::string filter_name = (*args)[0];
+				const auto filter = filter_container.get_filter(filter_name);
+
+				if (filter.has_value()) {
+					apply_filter(filter.value());
+				} else {
+					v->get_statusline().show_error(strprintf::fmt(_("No filter found with name `%s'."),
+							filter_name));
 				}
 			} else {
-				newfilter = v->select_filter(
-						filters.get_filters());
-				LOG(Level::DEBUG,
-					"ItemListFormAction::run: newfilters "
-					"= %s",
-					newfilter);
-			}
-			if (newfilter != "") {
-				filterhistory.add_line(newfilter);
-				if (newfilter.length() > 0) {
-					if (!matcher.parse(newfilter)) {
-						v->get_statusline().show_error(strprintf::fmt(
-								_("Error: couldn't "
-									"parse filter "
-									"expression `%s': %s"),
-								newfilter,
-								matcher.get_parse_error()));
-					} else {
-						apply_filter = true;
-						invalidate_list();
-						save_filterpos();
-					}
-				}
+				const std::string filter_text = v->select_filter(filter_container.get_filters());
+				apply_filter(filter_text);
 			}
 		} else {
 			v->get_statusline().show_error(_("No filters defined."));
@@ -674,14 +653,14 @@ bool ItemListFormAction::process_operation(Operation op,
 		}
 		break;
 	case OP_CLEARFILTER:
-		apply_filter = false;
+		filter_active = false;
 		invalidate_list();
 		save_filterpos();
 		break;
 	case OP_SORT: {
-		/// This string is related to the letters in parentheses in the
-		/// "Sort by (d)ate/..." and "Reverse Sort by (d)ate/..."
-		/// messages
+		// i18n: This string is related to the letters in parentheses in the
+		// "Sort by (d)ate/..." and "Reverse Sort by (d)ate/..."
+		// messages
 		std::string input_options = _("dtfalgr");
 		char c = v->confirm(
 				_("Sort by "
@@ -759,6 +738,29 @@ bool ItemListFormAction::process_operation(Operation op,
 		}
 	}
 	break;
+	case OP_ENQUEUE:
+		if (!visible_items.empty() && itempos < visible_items.size()) {
+			const auto item = visible_items[itempos].first;
+			return enqueue_item_enclosure(item, feed, *v, *rsscache);
+		} else {
+			v->get_statusline().show_error(_("No item selected!"));
+			return false;
+		}
+		break;
+	case OP_ARTICLEFEED: {
+		auto feeds = v->get_ctrl()->get_feedcontainer()->get_all_feeds();
+		size_t pos;
+		auto article_feed = visible_items[itempos].first->get_feedptr();
+		for (pos = 0; pos < feeds.size(); pos++) {
+			if (feeds[pos] == article_feed) {
+				break;
+			}
+		}
+		if (pos != feeds.size()) {
+			v->push_itemlist(pos);
+		}
+	}
+	break;
 	default:
 		ListFormAction::process_operation(op, automatic, args);
 		break;
@@ -784,7 +786,8 @@ bool ItemListFormAction::open_position_in_browser(
 		const auto item = visible_items[pos].first;
 		const auto link = item->link();
 		const auto feedurl = item->feedurl();
-		const auto exit_code = v->open_in_browser(link, feedurl, interactive);
+		const auto exit_code = v->open_in_browser(link, feedurl, "article", item->title(),
+				interactive);
 		if (!exit_code.has_value()) {
 			v->get_statusline().show_error(_("Failed to spawn browser"));
 			return false;
@@ -850,20 +853,7 @@ void ItemListFormAction::finished_qna(Operation op)
 void ItemListFormAction::qna_end_setfilter()
 {
 	std::string filtertext = qna_responses[0];
-	filterhistory.add_line(filtertext);
-
-	if (filtertext.length() > 0) {
-		if (!matcher.parse(filtertext)) {
-			v->get_statusline().show_error(strprintf::fmt(
-					_("Error: couldn't parse filter expression `%s': %s"),
-					filtertext, matcher.get_parse_error()));
-			return;
-		}
-
-		apply_filter = true;
-		invalidate_list();
-		save_filterpos();
-	}
+	apply_filter(filtertext);
 }
 
 void ItemListFormAction::qna_end_editflags()
@@ -915,10 +905,6 @@ void ItemListFormAction::qna_start_search()
 	std::shared_ptr<RssFeed> search_dummy_feed(new RssFeed(rsscache, ""));
 	search_dummy_feed->set_search_feed(true);
 	search_dummy_feed->add_items(items);
-
-	if (show_searchresult) {
-		v->pop_current_formaction();
-	}
 	v->push_searchresult(search_dummy_feed, searchphrase);
 }
 
@@ -945,7 +931,7 @@ void ItemListFormAction::do_update_visible_items()
 	for (const auto& item : items) {
 		item->set_index(i + 1);
 		if ((show_read || item->unread()) &&
-			(!apply_filter || matcher.matches(item.get()))) {
+			(!filter_active || matcher.matches(item.get()))) {
 			new_visible_items.push_back(ItemPtrPosPair(item, i));
 		}
 		i++;
@@ -1086,6 +1072,7 @@ std::string ItemListFormAction::item2formatted_line(const ItemPtrPosPair& item,
 	fmt.register_fmt('n', item.first->unread() ? "N" : " ");
 	fmt.register_fmt('d', item.first->deleted() ? "D" : " ");
 	fmt.register_fmt('F', item.first->flags());
+	fmt.register_fmt('e', item.first->enclosure_url());
 
 	using namespace std::chrono;
 	const auto article_time_point = system_clock::from_time_t(
@@ -1174,16 +1161,11 @@ void ItemListFormAction::init()
 	invalidate_list();
 }
 
-void ItemListFormAction::set_head(const std::string& s,
+FmtStrFormatter ItemListFormAction::setup_head_formatter(const std::string& s,
 	unsigned int unread,
 	unsigned int total,
 	const std::string& url)
 {
-	/*
-	 * Since the ItemListFormAction is also used to display search results,
-	 * we always need to set the right title
-	 */
-	std::string title;
 	FmtStrFormatter fmt;
 
 	fmt.register_fmt('N', PROGRAM_NAME);
@@ -1198,18 +1180,24 @@ void ItemListFormAction::set_head(const std::string& s,
 
 	fmt.register_fmt('U', utils::censor_url(url));
 
-	fmt.register_fmt('F', apply_filter ? matcher.get_expression() : "");
+	fmt.register_fmt('F', filter_active ? matcher.get_expression() : "");
+
+	return fmt;
+}
+
+void ItemListFormAction::set_head(const std::string& s,
+	unsigned int unread,
+	unsigned int total,
+	const std::string& url)
+{
+	std::string title;
+
+	FmtStrFormatter fmt = setup_head_formatter(s, unread, total, url);
 
 	const unsigned int width = utils::to_u(f.get("title:w"));
-	if (!show_searchresult) {
-		title = fmt.do_format(
-				cfg->get_configvalue("articlelist-title-format"),
-				width);
-	} else {
-		title = fmt.do_format(
-				cfg->get_configvalue("searchresult-title-format"),
-				width);
-	}
+	title = fmt.do_format(
+			cfg->get_configvalue("articlelist-title-format"),
+			width);
 	set_value("head", title);
 }
 
@@ -1451,7 +1439,7 @@ void ItemListFormAction::register_format_styles()
 {
 	const std::string attrstr = rxman.get_attrs_stfl_string("articlelist", true);
 	const std::string textview = strprintf::fmt(
-			"{list[items] .expand:vh style_normal[listnormal]: "
+			"{!list[items] .expand:vh style_normal[listnormal]: "
 			"style_focus[listfocus]:fg=yellow,bg=blue,attr=bold "
 			"pos[items_pos]:0 offset[items_offset]:0 %s richtext:1}",
 			attrstr);
@@ -1506,19 +1494,15 @@ void ItemListFormAction::set_feed(std::shared_ptr<RssFeed> fd)
 
 std::string ItemListFormAction::title()
 {
-	if (feed->rssurl() == "") {
-		return strprintf::fmt(_("Search Result - '%s'"), search_phrase);
+	if (feed->is_query_feed()) {
+		return strprintf::fmt(_("Query Feed - %s"),
+				feed->rssurl().substr(
+					6, feed->rssurl().length() - 6));
 	} else {
-		if (feed->is_query_feed()) {
-			return strprintf::fmt(_("Query Feed - %s"),
-					feed->rssurl().substr(
-						6, feed->rssurl().length() - 6));
-		} else {
-			auto feedtitle = feed->title();
-			utils::remove_soft_hyphens(feedtitle);
-			return strprintf::fmt(
-					_("Article List - %s"), feedtitle);
-		}
+		auto feedtitle = feed->title();
+		utils::remove_soft_hyphens(feedtitle);
+		return strprintf::fmt(
+				_("Article List - %s"), feedtitle);
 	}
 }
 
@@ -1620,4 +1604,23 @@ void ItemListFormAction::handle_op_saveall()
 	}
 }
 
+void ItemListFormAction::apply_filter(const std::string& filtertext)
+{
+	if (filtertext.empty()) {
+		return;
+	}
+
+	filterhistory.add_line(filtertext);
+	if (!matcher.parse(filtertext)) {
+		v->get_statusline().show_error(strprintf::fmt(
+				_("Error: couldn't parse filter expression `%s': %s"),
+				filtertext,
+				matcher.get_parse_error()));
+	} else {
+		save_filterpos();
+		filter_active = true;
+		invalidate_list();
+	}
+}
+
 } // namespace newsboat
