@@ -6,9 +6,12 @@
 
 #include "config.h"
 #include "confighandlerexception.h"
+#include "controller.h"
 #include "dbexception.h"
 #include "fmtstrformatter.h"
+#include "itemlistformaction.h"
 #include "itemrenderer.h"
+#include "itemutils.h"
 #include "htmlrenderer.h"
 #include "logger.h"
 #include "rssfeed.h"
@@ -28,7 +31,6 @@ ItemViewFormAction::ItemViewFormAction(View* vv,
 	RegexManager& r)
 	: FormAction(vv, formstr, cfg)
 	, show_source(false)
-	, quit(false)
 	, rxman(r)
 	, num_lines(0)
 	, itemlist(il)
@@ -45,16 +47,16 @@ ItemViewFormAction::~ItemViewFormAction() {}
 
 void ItemViewFormAction::init()
 {
-	f.set("msg", "");
+	set_value("msg", "");
 	do_redraw = true;
-	quit = false;
+	textview.set_scroll_offset(0);
 	links.clear();
 	num_lines = 0;
 	if (!cfg->get_configvalue_as_bool("display-article-progress")) {
-		f.set("percentwidth", "0");
+		set_value("percentwidth", "0");
 	} else {
 		const size_t min_field_width = 6;
-		f.set("percentwidth",
+		set_value("percentwidth",
 			std::to_string(
 		std::max({
 			min_field_width,
@@ -92,9 +94,9 @@ void ItemViewFormAction::prepare()
 	 */
 	if (do_redraw) {
 		{
-			ScopeMeasure("itemview::prepare: rendering");
+			ScopeMeasure sm("itemview::prepare: rendering");
 			// XXX HACK: render once so that we get a proper widget width
-			f.run(-3);
+			recalculate_widget_dimensions();
 		}
 
 		update_head(item);
@@ -142,7 +144,7 @@ void ItemViewFormAction::prepare()
 		}
 
 		textview.stfl_replace_lines(num_lines, formatted_text);
-		f.set("article_offset", "0");
+		update_percent();
 
 		if (in_search) {
 			rxman.remove_last_regex("article");
@@ -158,6 +160,7 @@ bool ItemViewFormAction::process_operation(Operation op,
 	std::vector<std::string>* args)
 {
 	bool hardquit = false;
+	bool quit = false;
 
 	/*
 	 * whenever we process an operation, we mark the item
@@ -173,7 +176,7 @@ bool ItemViewFormAction::process_operation(Operation op,
 			v->get_ctrl()->mark_article_read(item->guid(), true);
 		}
 	} catch (const DbException& e) {
-		v->show_error(strprintf::fmt(
+		v->get_statusline().show_error(strprintf::fmt(
 				_("Error while marking article as read: %s"),
 				e.what()));
 	}
@@ -197,24 +200,20 @@ bool ItemViewFormAction::process_operation(Operation op,
 	case OP_SK_PGDOWN:
 		textview.scroll_page_down();
 		break;
+	case OP_SK_HALF_PAGE_UP:
+		textview.scroll_halfpage_up();
+		break;
+	case OP_SK_HALF_PAGE_DOWN:
+		textview.scroll_halfpage_down();
+		break;
 	case OP_TOGGLESOURCEVIEW:
 		LOG(Level::INFO, "ItemViewFormAction::process_operation: toggling source view");
 		show_source = !show_source;
 		do_redraw = true;
+		textview.set_scroll_offset(0);
 		break;
-	case OP_ENQUEUE: {
-		if (item->enclosure_url().length() > 0 &&
-			utils::is_http_url(item->enclosure_url())) {
-			v->get_ctrl()->enqueue_url(item, feed);
-			v->set_status(
-				strprintf::fmt(_("Added %s to download queue."),
-					item->enclosure_url()));
-		} else {
-			v->set_status(strprintf::fmt(
-					_("Invalid URL: '%s'"), item->enclosure_url()));
-		}
-	}
-	break;
+	case OP_ENQUEUE:
+		return enqueue_item_enclosure(item, feed, *v, *rsscache);
 	case OP_SAVE: {
 		LOG(Level::INFO, "ItemViewFormAction::process_operation: saving article");
 		std::string filename;
@@ -227,14 +226,14 @@ bool ItemViewFormAction::process_operation(Operation op,
 							item->title())));
 		}
 		if (filename == "") {
-			v->show_error(_("Aborted saving."));
+			v->get_statusline().show_error(_("Aborted saving."));
 		} else {
 			try {
 				v->get_ctrl()->write_item(item, filename);
-				v->show_error(strprintf::fmt(
+				v->get_statusline().show_message(strprintf::fmt(
 						_("Saved article to %s."), filename));
 			} catch (...) {
-				v->show_error(strprintf::fmt(
+				v->get_statusline().show_error(strprintf::fmt(
 						_("Error: couldn't write article to "
 							"file %s"),
 						filename));
@@ -245,17 +244,14 @@ bool ItemViewFormAction::process_operation(Operation op,
 	case OP_OPENINBROWSER:
 	case OP_OPENBROWSER_AND_MARK: {
 		LOG(Level::INFO, "ItemViewFormAction::process_operation: starting browser");
-		v->set_status(_("Starting browser..."));
-		const auto exit_code = v->open_in_browser(item->link());
-		if (!exit_code.has_value()) {
-			v->show_error(_("Failed to spawn browser"));
-			return false;
-		} else if (*exit_code != 0) {
-			v->show_error(strprintf::fmt(_("Browser returned error code %i"), *exit_code));
-			return false;
-		}
-
-		v->set_status("");
+		const bool interactive = true;
+		return open_link_in_browser(item->link(), "article", item->title(), interactive);
+	}
+	break;
+	case OP_OPENINBROWSER_NONINTERACTIVE: {
+		LOG(Level::INFO, "ItemViewFormAction::process_operation: starting browser");
+		const bool interactive = false;
+		return open_link_in_browser(item->link(), "article", item->title(), interactive);
 	}
 	break;
 	case OP_BOOKMARK:
@@ -267,7 +263,9 @@ bool ItemViewFormAction::process_operation(Operation op,
 				args->size() > 0 ? (*args)[0] : "");
 			qna_responses.push_back(feed->title());
 		} else {
-			this->start_bookmark_qna(utils::utf8_to_locale(item->title()), item->link(), "",
+			this->start_bookmark_qna(
+				utils::utf8_to_locale(item->title()),
+				item->link(),
 				feed->title());
 		}
 		break;
@@ -322,7 +320,7 @@ bool ItemViewFormAction::process_operation(Operation op,
 			if (links.size() > 0) {
 				v->push_urlview(links, feed);
 			} else {
-				v->show_error(_("URL list empty."));
+				v->get_statusline().show_error(_("URL list empty."));
 			}
 		} else {
 			qna_responses.clear();
@@ -342,9 +340,10 @@ bool ItemViewFormAction::process_operation(Operation op,
 			"ItemViewFormAction::process_operation: jumping to next unread article");
 		if (v->get_next_unread(*itemlist, this)) {
 			do_redraw = true;
+			textview.set_scroll_offset(0);
 		} else {
 			v->pop_current_formaction();
-			v->show_error(_("No unread items."));
+			v->get_statusline().show_error(_("No unread items."));
 		}
 		break;
 	case OP_PREVUNREAD:
@@ -353,9 +352,10 @@ bool ItemViewFormAction::process_operation(Operation op,
 			"article");
 		if (v->get_previous_unread(*itemlist, this)) {
 			do_redraw = true;
+			textview.set_scroll_offset(0);
 		} else {
 			v->pop_current_formaction();
-			v->show_error(_("No unread items."));
+			v->get_statusline().show_error(_("No unread items."));
 		}
 		break;
 	case OP_NEXT:
@@ -363,9 +363,10 @@ bool ItemViewFormAction::process_operation(Operation op,
 			"ItemViewFormAction::process_operation: jumping to next article");
 		if (v->get_next(*itemlist, this)) {
 			do_redraw = true;
+			textview.set_scroll_offset(0);
 		} else {
 			v->pop_current_formaction();
-			v->show_error(_("Already on last item."));
+			v->get_statusline().show_error(_("Already on last item."));
 		}
 		break;
 	case OP_PREV:
@@ -373,9 +374,10 @@ bool ItemViewFormAction::process_operation(Operation op,
 			"ItemViewFormAction::process_operation: jumping to previous article");
 		if (v->get_previous(*itemlist, this)) {
 			do_redraw = true;
+			textview.set_scroll_offset(0);
 		} else {
 			v->pop_current_formaction();
-			v->show_error(_("Already on first item."));
+			v->get_statusline().show_error(_("Already on first item."));
 		}
 		break;
 	case OP_RANDOMUNREAD:
@@ -383,26 +385,38 @@ bool ItemViewFormAction::process_operation(Operation op,
 			"ItemViewFormAction::process_operation: jumping to random unread article");
 		if (v->get_random_unread(*itemlist, this)) {
 			do_redraw = true;
+			textview.set_scroll_offset(0);
 		} else {
 			v->pop_current_formaction();
-			v->show_error(_("No unread items."));
+			v->get_statusline().show_error(_("No unread items."));
 		}
 		break;
-	case OP_TOGGLEITEMREAD:
+	case OP_TOGGLEITEMREAD: {
 		LOG(Level::INFO,
 			"ItemViewFormAction::process_operation: setting unread and quitting");
-		v->set_status(_("Toggling read flag for article..."));
+		const auto message_lifetime = v->get_statusline().show_message_until_finished(
+				_("Toggling read flag for article..."));
 		try {
-			item->set_unread(true);
-			v->get_ctrl()->mark_article_read(item->guid(), false);
+			if (automatic && args->size() > 0) {
+				if ((*args)[0] == "read") {
+					item->set_unread(false);
+					v->get_ctrl()->mark_article_read(item->guid(), true);
+				} else if ((*args)[0] == "unread") {
+					item->set_unread(true);
+					v->get_ctrl()->mark_article_read(item->guid(), false);
+				}
+			} else {
+				item->set_unread(true);
+				v->get_ctrl()->mark_article_read(item->guid(), false);
+			}
 		} catch (const DbException& e) {
-			v->show_error(strprintf::fmt(
+			v->get_statusline().show_error(strprintf::fmt(
 					_("Error while marking article as unread: %s"),
 					e.what()));
 		}
-		v->set_status("");
 		quit = true;
 		break;
+	}
 	case OP_QUIT:
 		LOG(Level::INFO, "ItemViewFormAction::process_operation: quitting");
 		quit = true;
@@ -431,18 +445,9 @@ bool ItemViewFormAction::process_operation(Operation op,
 			op,
 			idx);
 		if (idx < links.size()) {
-			v->set_status(_("Starting browser..."));
-
-			const auto exit_code = v->open_in_browser(links[idx].first);
-			if (!exit_code.has_value()) {
-				v->show_error(_("Failed to spawn browser"));
-				return false;
-			} else if (*exit_code != 0) {
-				v->show_error(strprintf::fmt(_("Browser returned error code %i"), *exit_code));
-				return false;
-			}
-
-			v->set_status("");
+			const bool interactive = true;
+			return open_link_in_browser(links[idx].first, utils::link_type_str(links[idx].second),
+					item->title(), interactive);
 		}
 	}
 	break;
@@ -460,6 +465,20 @@ bool ItemViewFormAction::process_operation(Operation op,
 		}
 	}
 	break;
+	case OP_ARTICLEFEED: {
+		auto feeds = v->get_ctrl()->get_feedcontainer()->get_all_feeds();
+		size_t pos;
+		auto article_feed = item->get_feedptr();
+		for (pos = 0; pos < feeds.size(); pos++) {
+			if (feeds[pos] == article_feed) {
+				break;
+			}
+		}
+		if (pos != feeds.size()) {
+			v->push_itemlist(pos);
+		}
+	}
+	break;
 	default:
 		break;
 	}
@@ -470,6 +489,13 @@ bool ItemViewFormAction::process_operation(Operation op,
 		}
 	} else if (quit) {
 		v->pop_current_formaction();
+
+		auto parent_itemlist = std::dynamic_pointer_cast<ItemListFormAction>
+			(get_parent_formaction());
+		if (parent_itemlist != nullptr) {
+			parent_itemlist->invalidate_list();
+			parent_itemlist->restore_selected_position();
+		}
 	}
 
 	update_percent();
@@ -477,15 +503,30 @@ bool ItemViewFormAction::process_operation(Operation op,
 	return true;
 }
 
-KeyMapHintEntry* ItemViewFormAction::get_keymap_hint()
+bool ItemViewFormAction::open_link_in_browser(const std::string& link,
+	const std::string& type, const std::string& title, bool interactive) const
 {
-	static KeyMapHintEntry hints[] = {{OP_QUIT, _("Quit")},
+	const std::string feedurl = item->feedurl();
+	const auto exit_code = v->open_in_browser(link, feedurl, type, title, interactive);
+	if (!exit_code.has_value()) {
+		v->get_statusline().show_error(_("Failed to spawn browser"));
+		return false;
+	} else if (*exit_code != 0) {
+		v->get_statusline().show_error(strprintf::fmt(_("Browser returned error code %i"),
+				*exit_code));
+		return false;
+	}
+	return true;
+}
+
+const std::vector<KeyMapHintEntry>& ItemViewFormAction::get_keymap_hint() const
+{
+	static const std::vector<KeyMapHintEntry> hints = {{OP_QUIT, _("Quit")},
 		{OP_SAVE, _("Save")},
 		{OP_NEXTUNREAD, _("Next Unread")},
 		{OP_OPENINBROWSER, _("Open in Browser")},
 		{OP_ENQUEUE, _("Enqueue")},
-		{OP_HELP, _("Help")},
-		{OP_NIL, nullptr}
+		{OP_HELP, _("Help")}
 	};
 	return hints;
 }
@@ -512,37 +553,42 @@ void ItemViewFormAction::set_head(const std::string& s,
 
 	const unsigned int width = textview.get_width();
 
-	f.set("head",
+	set_value("head",
 		fmt.do_format(
 			cfg->get_configvalue("itemview-title-format"), width));
 }
 
 void ItemViewFormAction::handle_cmdline(const std::string& cmd)
 {
-	std::vector<std::string> tokens = utils::tokenize_quoted(cmd);
-	if (!tokens.empty()) {
-		if (tokens[0] == "save" && tokens.size() >= 2) {
-			std::string filename = utils::resolve_tilde(tokens[1]);
+	const auto command = FormAction::parse_command(cmd);
+	switch (command.type) {
+	case CommandType::SAVE:
+		if (!command.args.empty()) {
+			handle_save(command.args.front());
+		}
+		break;
+	default:
+		FormAction::handle_parsed_command(command);
+	}
+}
 
-			if (filename == "") {
-				v->show_error(_("Aborted saving."));
-			} else {
-				try {
-					v->get_ctrl()->write_item(
-						item, filename);
-					v->show_error(strprintf::fmt(
-							_("Saved article to %s"),
-							filename));
-				} catch (...) {
-					v->show_error(strprintf::fmt(
-							_("Error: couldn't save "
-								"article to %s"),
-							filename));
-				}
-			}
-
-		} else {
-			FormAction::handle_cmdline(cmd);
+void ItemViewFormAction::handle_save(const std::string& filename_param)
+{
+	std::string filename = utils::resolve_tilde(filename_param);
+	if (filename == "") {
+		v->get_statusline().show_error(_("Aborted saving."));
+	} else {
+		try {
+			v->get_ctrl()->write_item(
+				item, filename);
+			v->get_statusline().show_message(strprintf::fmt(
+					_("Saved article to %s"),
+					filename));
+		} catch (...) {
+			v->get_statusline().show_error(strprintf::fmt(
+					_("Error: couldn't save "
+						"article to %s"),
+					filename));
 		}
 	}
 }
@@ -555,7 +601,7 @@ void ItemViewFormAction::finished_qna(Operation op)
 	case OP_INT_EDITFLAGS_END:
 		item->set_flags(qna_responses[0]);
 		v->get_ctrl()->update_flags(item);
-		v->set_status(_("Flags updated."));
+		v->get_statusline().show_message(_("Flags updated."));
 		do_redraw = true;
 		break;
 	case OP_INT_START_SEARCH:
@@ -573,6 +619,7 @@ void ItemViewFormAction::finished_qna(Operation op)
 			fwrite(data.c_str(), data.length(), 1, f);
 			pclose(f);
 		}
+		v->drop_queued_input();
 		v->pop_current_formaction();
 	}
 	break;
@@ -580,9 +627,9 @@ void ItemViewFormAction::finished_qna(Operation op)
 		unsigned int idx = 0;
 		sscanf(qna_responses[0].c_str(), "%u", &idx);
 		if (idx && idx - 1 < links.size()) {
-			v->set_status(_("Starting browser..."));
-			v->open_in_browser(links[idx - 1].first);
-			v->set_status("");
+			const bool interactive = true;
+			open_link_in_browser(links[idx - 1].first, utils::link_type_str(links[idx - 1].second),
+				item->title(), interactive);
 		}
 	}
 	break;
@@ -625,11 +672,13 @@ void ItemViewFormAction::update_percent()
 			percent);
 
 		if (offset == 0 || percent == 0) {
-			f.set("percent", _("Top"));
+			set_value("percent", _("Top"));
 		} else if (offset == (num_lines - 1)) {
-			f.set("percent", _("Bottom"));
+			set_value("percent", _("Bottom"));
 		} else {
-			f.set("percent", strprintf::fmt("%3u %% ", percent));
+			// i18n: `%3u` gets replaced by a number between 0 and 100.
+			// The `%%` is replaced by a single percent sign.
+			set_value("percent", strprintf::fmt(_("%3u %% "), percent));
 		}
 	}
 }
@@ -688,7 +737,7 @@ void ItemViewFormAction::highlight_text(const std::string& searchphrase)
 			"ItemViewFormAction::highlight_text: handle_action "
 			"failed, error = %s",
 			e.what());
-		v->show_error(_("Error: invalid regular expression!"));
+		v->get_statusline().show_error(_("Error: invalid regular expression!"));
 	}
 }
 
