@@ -1,5 +1,6 @@
 #include "keymap.h"
 
+#include <algorithm>
 #include <iostream>
 #include <map>
 #include <string>
@@ -7,10 +8,13 @@
 
 #include "config.h"
 #include "confighandlerexception.h"
+#include "configparser.h"
 #include "logger.h"
 #include "strprintf.h"
 #include "utils.h"
 
+#include "libnewsboat-ffi/src/keymap.rs.h"
+
 namespace newsboat {
 
 struct OpDesc {
@@ -30,446 +34,441 @@ static const std::vector<OpDesc> opdescs = {
 		OP_OPEN,
 		"open",
 		"ENTER",
-		_("Open feed/article"),
+		translatable("Open feed/article"),
 		KM_FEEDLIST | KM_FILEBROWSER | KM_ARTICLELIST | KM_TAGSELECT |
-		KM_FILTERSELECT | KM_URLVIEW | KM_DIALOGS | KM_DIRBROWSER
+		KM_FILTERSELECT | KM_URLVIEW | KM_DIALOGS | KM_DIRBROWSER | KM_SEARCHRESULTSLIST
 	},
 	{
 		OP_SWITCH_FOCUS,
 		"switch-focus",
 		"TAB",
-		_("Switch focus between widgets"),
+		translatable("Switch focus between widgets"),
 		KM_FILEBROWSER | KM_DIRBROWSER
 	},
-	{OP_QUIT, "quit", "q", _("Return to previous dialog/Quit"), KM_BOTH},
+	{OP_QUIT, "quit", "q", translatable("Return to previous dialog/Quit"), KM_BOTH},
 	{
 		OP_HARDQUIT,
 		"hard-quit",
 		"Q",
-		_("Quit program, no confirmation"),
+		translatable("Quit program, no confirmation"),
 		KM_BOTH
 	},
 	{
 		OP_RELOAD,
 		"reload",
 		"r",
-		_("Reload currently selected feed"),
+		translatable("Reload currently selected feed"),
 		KM_FEEDLIST | KM_ARTICLELIST
 	},
-	{OP_RELOADALL, "reload-all", "R", _("Reload all feeds"), KM_FEEDLIST},
+	{OP_RELOADALL, "reload-all", "R", translatable("Reload all feeds"), KM_FEEDLIST},
 	{
 		OP_MARKFEEDREAD,
 		"mark-feed-read",
 		"A",
-		_("Mark feed read"),
-		KM_FEEDLIST | KM_ARTICLELIST
+		translatable("Mark feed read"),
+		KM_FEEDLIST | KM_ARTICLELIST | KM_SEARCHRESULTSLIST
 	},
 	{
 		OP_MARKALLFEEDSREAD,
 		"mark-all-feeds-read",
 		"C",
-		_("Mark all feeds read"),
+		translatable("Mark all feeds read"),
 		KM_FEEDLIST
 	},
 	{
 		OP_MARKALLABOVEASREAD,
 		"mark-all-above-as-read",
 		"",
-		_("Mark all above as read"),
-		KM_ARTICLELIST
+		translatable("Mark all above as read"),
+		KM_ARTICLELIST | KM_SEARCHRESULTSLIST
 	},
-	{OP_SAVE, "save", "s", _("Save article"), KM_ARTICLELIST | KM_ARTICLE},
-	{OP_SAVEALL, "save-all", "", _("Save articles"), KM_ARTICLELIST},
+	{OP_SAVE, "save", "s", translatable("Save article"), KM_ARTICLELIST | KM_ARTICLE | KM_SEARCHRESULTSLIST},
+	{OP_SAVEALL, "save-all", "", translatable("Save articles"), KM_ARTICLELIST | KM_SEARCHRESULTSLIST},
 	{
 		OP_NEXT,
 		"next",
 		"J",
-		_("Go to next entry"),
-		KM_FEEDLIST | KM_ARTICLELIST | KM_ARTICLE | KM_DIALOGS | KM_DIRBROWSER | KM_FILEBROWSER | KM_FILTERSELECT | KM_TAGSELECT | KM_URLVIEW | KM_PODBOAT
+		translatable("Go to next entry"),
+		KM_FEEDLIST | KM_ARTICLELIST | KM_ARTICLE | KM_DIALOGS | KM_DIRBROWSER | KM_FILEBROWSER | KM_FILTERSELECT | KM_TAGSELECT | KM_URLVIEW | KM_SEARCHRESULTSLIST | KM_PODBOAT
 	},
 	{
 		OP_PREV,
 		"prev",
 		"K",
-		_("Go to previous entry"),
-		KM_FEEDLIST | KM_ARTICLELIST | KM_ARTICLE | KM_DIALOGS | KM_DIRBROWSER | KM_FILEBROWSER | KM_FILTERSELECT | KM_TAGSELECT | KM_URLVIEW | KM_PODBOAT
+		translatable("Go to previous entry"),
+		KM_FEEDLIST | KM_ARTICLELIST | KM_ARTICLE | KM_DIALOGS | KM_DIRBROWSER | KM_FILEBROWSER | KM_FILTERSELECT | KM_TAGSELECT | KM_URLVIEW | KM_SEARCHRESULTSLIST | KM_PODBOAT
 	},
 	{
 		OP_NEXTUNREAD,
 		"next-unread",
 		"n",
-		_("Go to next unread article"),
-		KM_FEEDLIST | KM_ARTICLELIST | KM_ARTICLE
+		translatable("Go to next unread article"),
+		KM_FEEDLIST | KM_ARTICLELIST | KM_SEARCHRESULTSLIST | KM_ARTICLE
 	},
 	{
 		OP_PREVUNREAD,
 		"prev-unread",
 		"p",
-		_("Go to previous unread article"),
-		KM_FEEDLIST | KM_ARTICLELIST | KM_ARTICLE
+		translatable("Go to previous unread article"),
+		KM_FEEDLIST | KM_ARTICLELIST | KM_SEARCHRESULTSLIST | KM_ARTICLE
 	},
 	{
 		OP_RANDOMUNREAD,
 		"random-unread",
 		"^K",
-		_("Go to a random unread article"),
-		KM_FEEDLIST | KM_ARTICLELIST | KM_ARTICLE
+		translatable("Go to a random unread article"),
+		KM_FEEDLIST | KM_ARTICLELIST | KM_SEARCHRESULTSLIST | KM_ARTICLE
 	},
 	{
 		OP_OPENBROWSER_AND_MARK,
 		"open-in-browser-and-mark-read",
 		"O",
-		_("Open article in browser and mark read"),
-		KM_ARTICLELIST | KM_ARTICLE | KM_URLVIEW
+		translatable("Open URL of article, or entry in URL view. Mark read"),
+		KM_ARTICLELIST | KM_SEARCHRESULTSLIST | KM_ARTICLE | KM_URLVIEW
 	},
 	{
 		OP_OPENALLUNREADINBROWSER,
 		"open-all-unread-in-browser",
 		"",
-		_("Open all unread items of selected feed in browser"),
-		KM_FEEDLIST | KM_ARTICLELIST
+		translatable("Open all unread items of selected feed in browser"),
+		KM_FEEDLIST | KM_ARTICLELIST | KM_SEARCHRESULTSLIST
 	},
 	{
 		OP_OPENALLUNREADINBROWSER_AND_MARK,
 		"open-all-unread-in-browser-and-mark-read",
 		"",
-		_("Open all unread items of selected feed in browser and mark "
+		translatable("Open all unread items of selected feed in browser and mark "
 			"read"),
-		KM_FEEDLIST | KM_ARTICLELIST
+		KM_FEEDLIST | KM_ARTICLELIST | KM_SEARCHRESULTSLIST
 	},
 	{
 		OP_OPENINBROWSER,
 		"open-in-browser",
 		"o",
-		_("Open article in browser"),
-		KM_FEEDLIST | KM_ARTICLELIST | KM_ARTICLE | KM_URLVIEW
+		translatable("Open URL of article, feed, or entry in URL view"),
+		KM_FEEDLIST | KM_ARTICLELIST  | KM_SEARCHRESULTSLIST| KM_ARTICLE | KM_URLVIEW
+	},
+	{
+		OP_OPENINBROWSER_NONINTERACTIVE,
+		"open-in-browser-noninteractively",
+		"",
+		translatable("Open URL of article, feed, or entry in a browser, non-interactively"),
+		KM_FEEDLIST | KM_ARTICLELIST | KM_SEARCHRESULTSLIST | KM_ARTICLE | KM_URLVIEW
 	},
 	{
 		OP_HELP,
 		"help",
 		"?",
-		_("Open help dialog"),
-		KM_FEEDLIST | KM_ARTICLELIST | KM_ARTICLE | KM_PODBOAT
+		translatable("Open help dialog"),
+		KM_FEEDLIST | KM_ARTICLELIST | KM_SEARCHRESULTSLIST | KM_ARTICLE | KM_PODBOAT | KM_URLVIEW
 	},
 	{
 		OP_TOGGLESOURCEVIEW,
 		"toggle-source-view",
 		"^U",
-		_("Toggle source view"),
+		translatable("Toggle source view"),
 		KM_ARTICLE
 	},
 	{
 		OP_TOGGLEITEMREAD,
 		"toggle-article-read",
 		"N",
-		_("Toggle read status for article"),
-		KM_ARTICLELIST | KM_ARTICLE
+		translatable("Toggle read status for article"),
+		KM_ARTICLELIST | KM_SEARCHRESULTSLIST | KM_ARTICLE
 	},
 	{
 		OP_TOGGLESHOWREAD,
 		"toggle-show-read-feeds",
 		"l",
-		_("Toggle show read feeds/articles"),
-		KM_FEEDLIST | KM_ARTICLELIST
+		translatable("Toggle show read feeds/articles"),
+		KM_FEEDLIST | KM_ARTICLELIST | KM_SEARCHRESULTSLIST
 	},
 	{
 		OP_SHOWURLS,
 		"show-urls",
 		"u",
-		_("Show URLs in current article"),
-		KM_ARTICLE | KM_ARTICLELIST
+		translatable("Show URLs in current article"),
+		KM_ARTICLE | KM_SEARCHRESULTSLIST | KM_ARTICLELIST
 	},
-	{OP_CLEARTAG, "clear-tag", "^T", _("Clear current tag"), KM_FEEDLIST},
-	{OP_SETTAG, "set-tag", "t", _("Select tag"), KM_FEEDLIST},
-	{OP_SETTAG, "select-tag", "t", _("Select tag"), KM_FEEDLIST},
+	{OP_CLEARTAG, "clear-tag", "^T", translatable("Clear current tag"), KM_FEEDLIST},
+	{OP_SETTAG, "set-tag", "t", translatable("Select tag"), KM_FEEDLIST},
+	{OP_SETTAG, "select-tag", "t", translatable("Select tag"), KM_FEEDLIST},
 	{
 		OP_SEARCH,
 		"open-search",
 		"/",
-		_("Open search dialog"),
-		KM_FEEDLIST | KM_HELP | KM_ARTICLELIST | KM_ARTICLE
+		translatable("Open search dialog"),
+		KM_FEEDLIST | KM_HELP | KM_ARTICLELIST | KM_SEARCHRESULTSLIST | KM_ARTICLE
 	},
-	{OP_GOTO_URL, "goto-url", "#", _("Goto URL #"), KM_ARTICLE},
-	{OP_ENQUEUE, "enqueue", "e", _("Add download to queue"), KM_ARTICLE},
+	{OP_GOTO_URL, "goto-url", "#", translatable("Goto URL #"), KM_ARTICLE},
+	{OP_GOTO_TITLE, "goto-title", "", translatable("Goto item with title"), KM_FEEDLIST | KM_ARTICLELIST | KM_SEARCHRESULTSLIST},
+	{OP_ENQUEUE, "enqueue", "e", translatable("Add download to queue"), KM_ARTICLELIST | KM_SEARCHRESULTSLIST | KM_ARTICLE},
 	{
 		OP_RELOADURLS,
 		"reload-urls",
 		"^R",
-		_("Reload the list of URLs from the configuration"),
+		translatable("Reload the list of URLs from the configuration"),
 		KM_FEEDLIST
 	},
-	{OP_PB_DOWNLOAD, "pb-download", "d", _("Download file"), KM_PODBOAT},
-	{OP_PB_CANCEL, "pb-cancel", "c", _("Cancel download"), KM_PODBOAT},
+	{OP_PB_DOWNLOAD, "pb-download", "d", translatable("Download file"), KM_PODBOAT},
+	{OP_PB_CANCEL, "pb-cancel", "c", translatable("Cancel download"), KM_PODBOAT},
 	{
 		OP_PB_DELETE,
 		"pb-delete",
 		"D",
-		_("Mark download as deleted"),
+		translatable("Mark download as deleted"),
 		KM_PODBOAT
 	},
 	{
 		OP_PB_PURGE,
 		"pb-purge",
 		"P",
-		_("Purge finished and deleted downloads from queue"),
+		translatable("Purge finished and deleted downloads from queue"),
 		KM_PODBOAT
 	},
 	{
 		OP_PB_TOGGLE_DLALL,
 		"pb-toggle-download-all",
 		"a",
-		_("Toggle automatic download on/off"),
+		translatable("Toggle automatic download on/off"),
 		KM_PODBOAT
 	},
 	{
 		OP_PB_PLAY,
 		"pb-play",
 		"p",
-		_("Start player with currently selected download"),
+		translatable("Start player with currently selected download"),
 		KM_PODBOAT
 	},
 	{
 		OP_PB_MARK_FINISHED,
 		"pb-mark-as-finished",
 		"m",
-		_("Mark file as finished (not played)"),
+		translatable("Mark file as finished (not played)"),
 		KM_PODBOAT
 	},
 	{
 		OP_PB_MOREDL,
 		"pb-increase-max-dls",
 		"+",
-		_("Increase the number of concurrent downloads"),
+		translatable("Increase the number of concurrent downloads"),
 		KM_PODBOAT
 	},
 	{
 		OP_PB_LESSDL,
 		"pb-decreate-max-dls",
 		"-",
-		_("Decrease the number of concurrent downloads"),
+		translatable("Decrease the number of concurrent downloads"),
 		KM_PODBOAT
 	},
-	{OP_REDRAW, "redraw", "^L", _("Redraw screen"), KM_SYSKEYS},
-	{OP_CMDLINE, "cmdline", ":", _("Open the commandline"), KM_NEWSBOAT},
+	{OP_REDRAW, "redraw", "^L", translatable("Redraw screen"), KM_SYSKEYS},
+	{OP_CMDLINE, "cmdline", ":", translatable("Open the commandline"), KM_NEWSBOAT},
 	{
 		OP_SETFILTER,
 		"set-filter",
 		"F",
-		_("Set a filter"),
-		KM_FEEDLIST | KM_ARTICLELIST
+		translatable("Set a filter"),
+		KM_FEEDLIST | KM_ARTICLELIST | KM_SEARCHRESULTSLIST
 	},
 	{
 		OP_SELECTFILTER,
 		"select-filter",
 		"f",
-		_("Select a predefined filter"),
-		KM_FEEDLIST | KM_ARTICLELIST
+		translatable("Select a predefined filter"),
+		KM_FEEDLIST | KM_ARTICLELIST | KM_SEARCHRESULTSLIST
 	},
 	{
 		OP_CLEARFILTER,
 		"clear-filter",
 		"^F",
-		_("Clear currently set filter"),
-		KM_FEEDLIST | KM_HELP | KM_ARTICLELIST
+		translatable("Clear currently set filter"),
+		KM_FEEDLIST | KM_HELP | KM_ARTICLELIST | KM_SEARCHRESULTSLIST
 	},
 	{
 		OP_BOOKMARK,
 		"bookmark",
 		"^B",
-		_("Bookmark current link/article"),
-		KM_ARTICLELIST | KM_ARTICLE | KM_URLVIEW
+		translatable("Bookmark current link/article"),
+		KM_ARTICLELIST | KM_SEARCHRESULTSLIST | KM_ARTICLE | KM_URLVIEW
 	},
 	{
 		OP_EDITFLAGS,
 		"edit-flags",
 		"^E",
-		_("Edit flags"),
-		KM_ARTICLELIST | KM_ARTICLE
+		translatable("Edit flags"),
+		KM_ARTICLELIST | KM_SEARCHRESULTSLIST | KM_ARTICLE
 	},
-	{OP_NEXTFEED, "next-feed", "j", _("Go to next feed"), KM_ARTICLELIST},
+	{OP_NEXTFEED, "next-feed", "j", translatable("Go to next feed"), KM_ARTICLELIST},
 	{
 		OP_PREVFEED,
 		"prev-feed",
 		"k",
-		_("Go to previous feed"),
+		translatable("Go to previous feed"),
 		KM_ARTICLELIST
 	},
 	{
 		OP_NEXTUNREADFEED,
 		"next-unread-feed",
 		"^N",
-		_("Go to next unread feed"),
+		translatable("Go to next unread feed"),
 		KM_ARTICLELIST
 	},
 	{
 		OP_PREVUNREADFEED,
 		"prev-unread-feed",
 		"^P",
-		_("Go to previous unread feed"),
+		translatable("Go to previous unread feed"),
 		KM_ARTICLELIST
 	},
-	{OP_MACROPREFIX, "macro-prefix", ",", _("Call a macro"), KM_NEWSBOAT},
+	{OP_MACROPREFIX, "macro-prefix", ",", translatable("Call a macro"), KM_NEWSBOAT},
 	{
 		OP_DELETE,
 		"delete-article",
 		"D",
-		_("Delete article"),
-		KM_ARTICLELIST | KM_ARTICLE
+		translatable("Delete article"),
+		KM_ARTICLELIST | KM_SEARCHRESULTSLIST | KM_ARTICLE
 	},
 	{
 		OP_DELETE_ALL,
 		"delete-all-articles",
 		"^D",
-		_("Delete all articles"),
-		KM_ARTICLELIST
+		translatable("Delete all articles"),
+		KM_ARTICLELIST | KM_SEARCHRESULTSLIST
 	},
 	{
 		OP_PURGE_DELETED,
 		"purge-deleted",
 		"$",
-		_("Purge deleted articles"),
-		KM_ARTICLELIST
+		translatable("Purge deleted articles"),
+		KM_ARTICLELIST | KM_SEARCHRESULTSLIST
 	},
 	{
 		OP_EDIT_URLS,
 		"edit-urls",
 		"E",
-		_("Edit subscribed URLs"),
-		KM_FEEDLIST | KM_ARTICLELIST
+		translatable("Edit subscribed URLs"),
+		KM_FEEDLIST | KM_ARTICLELIST | KM_SEARCHRESULTSLIST
 	},
 	{
 		OP_CLOSEDIALOG,
 		"close-dialog",
 		"^X",
-		_("Close currently selected dialog"),
+		translatable("Close currently selected dialog"),
 		KM_DIALOGS
 	},
 	{
 		OP_VIEWDIALOGS,
 		"view-dialogs",
 		"v",
-		_("View list of open dialogs"),
+		translatable("View list of open dialogs"),
 		KM_NEWSBOAT
 	},
 	{
 		OP_NEXTDIALOG,
 		"next-dialog",
 		"^V",
-		_("Go to next dialog"),
+		translatable("Go to next dialog"),
 		KM_NEWSBOAT
 	},
 	{
 		OP_PREVDIALOG,
 		"prev-dialog",
 		"^G",
-		_("Go to previous dialog"),
+		translatable("Go to previous dialog"),
 		KM_NEWSBOAT
 	},
 	{
 		OP_PIPE_TO,
 		"pipe-to",
 		"|",
-		_("Pipe article to command"),
-		KM_ARTICLE | KM_ARTICLELIST
+		translatable("Pipe article to command"),
+		KM_ARTICLE | KM_ARTICLELIST | KM_SEARCHRESULTSLIST
 	},
 	{
 		OP_SORT,
 		"sort",
 		"g",
-		_("Sort current list"),
-		KM_FEEDLIST | KM_ARTICLELIST
+		translatable("Sort current list"),
+		KM_FEEDLIST | KM_ARTICLELIST | KM_SEARCHRESULTSLIST
 	},
 	{
 		OP_REVSORT,
 		"rev-sort",
 		"G",
-		_("Sort current list (reverse)"),
-		KM_FEEDLIST | KM_ARTICLELIST
+		translatable("Sort current list (reverse)"),
+		KM_FEEDLIST | KM_ARTICLELIST | KM_SEARCHRESULTSLIST
 	},
 
-	{OP_OPEN_URL_10, "zero", "0", _("Open URL 10"), KM_URLVIEW | KM_ARTICLE},
-	{OP_OPEN_URL_1, "one", "1", _("Open URL 1"), KM_URLVIEW | KM_ARTICLE},
-	{OP_OPEN_URL_2, "two", "2", _("Open URL 2"), KM_URLVIEW | KM_ARTICLE},
-	{OP_OPEN_URL_3, "three", "3", _("Open URL 3"), KM_URLVIEW | KM_ARTICLE},
-	{OP_OPEN_URL_4, "four", "4", _("Open URL 4"), KM_URLVIEW | KM_ARTICLE},
-	{OP_OPEN_URL_5, "five", "5", _("Open URL 5"), KM_URLVIEW | KM_ARTICLE},
-	{OP_OPEN_URL_6, "six", "6", _("Open URL 6"), KM_URLVIEW | KM_ARTICLE},
-	{OP_OPEN_URL_7, "seven", "7", _("Open URL 7"), KM_URLVIEW | KM_ARTICLE},
-	{OP_OPEN_URL_8, "eight", "8", _("Open URL 8"), KM_URLVIEW | KM_ARTICLE},
-	{OP_OPEN_URL_9, "nine", "9", _("Open URL 9"), KM_URLVIEW | KM_ARTICLE},
+	{
+		OP_PREVSEARCHRESULTS,
+		"prevsearchresults",
+		"z",
+		translatable("Return to previous search results (if any)"),
+		KM_SEARCHRESULTSLIST
+	},
+	{
+		OP_ARTICLEFEED,
+		"article-feed",
+		"",
+		translatable("Go to the feed of the article"),
+		KM_ARTICLE | KM_ARTICLELIST | KM_SEARCHRESULTSLIST
+	},
 
-	{OP_CMD_START_1, "cmd-one", "1", _("Start cmdline with 1"), KM_FEEDLIST | KM_ARTICLELIST | KM_TAGSELECT | KM_FILTERSELECT},
-	{OP_CMD_START_2, "cmd-two", "2", _("Start cmdline with 2"), KM_FEEDLIST | KM_ARTICLELIST | KM_TAGSELECT | KM_FILTERSELECT},
-	{OP_CMD_START_3, "cmd-three", "3", _("Start cmdline with 3"), KM_FEEDLIST | KM_ARTICLELIST | KM_TAGSELECT | KM_FILTERSELECT},
-	{OP_CMD_START_4, "cmd-four", "4", _("Start cmdline with 4"), KM_FEEDLIST | KM_ARTICLELIST | KM_TAGSELECT | KM_FILTERSELECT},
-	{OP_CMD_START_5, "cmd-five", "5", _("Start cmdline with 5"), KM_FEEDLIST | KM_ARTICLELIST | KM_TAGSELECT | KM_FILTERSELECT},
-	{OP_CMD_START_6, "cmd-six", "6", _("Start cmdline with 6"), KM_FEEDLIST | KM_ARTICLELIST | KM_TAGSELECT | KM_FILTERSELECT},
-	{OP_CMD_START_7, "cmd-seven", "7", _("Start cmdline with 7"), KM_FEEDLIST | KM_ARTICLELIST | KM_TAGSELECT | KM_FILTERSELECT},
-	{OP_CMD_START_8, "cmd-eight", "8", _("Start cmdline with 8"), KM_FEEDLIST | KM_ARTICLELIST | KM_TAGSELECT | KM_FILTERSELECT},
-	{OP_CMD_START_9, "cmd-nine", "9", _("Start cmdline with 9"), KM_FEEDLIST | KM_ARTICLELIST | KM_TAGSELECT | KM_FILTERSELECT},
+	{OP_OPEN_URL_1, "one", "1", translatable("Open URL 1"), KM_URLVIEW | KM_ARTICLE},
+	{OP_OPEN_URL_2, "two", "2", translatable("Open URL 2"), KM_URLVIEW | KM_ARTICLE},
+	{OP_OPEN_URL_3, "three", "3", translatable("Open URL 3"), KM_URLVIEW | KM_ARTICLE},
+	{OP_OPEN_URL_4, "four", "4", translatable("Open URL 4"), KM_URLVIEW | KM_ARTICLE},
+	{OP_OPEN_URL_5, "five", "5", translatable("Open URL 5"), KM_URLVIEW | KM_ARTICLE},
+	{OP_OPEN_URL_6, "six", "6", translatable("Open URL 6"), KM_URLVIEW | KM_ARTICLE},
+	{OP_OPEN_URL_7, "seven", "7", translatable("Open URL 7"), KM_URLVIEW | KM_ARTICLE},
+	{OP_OPEN_URL_8, "eight", "8", translatable("Open URL 8"), KM_URLVIEW | KM_ARTICLE},
+	{OP_OPEN_URL_9, "nine", "9", translatable("Open URL 9"), KM_URLVIEW | KM_ARTICLE},
+	{OP_OPEN_URL_10, "zero", "0", translatable("Open URL 10"), KM_URLVIEW | KM_ARTICLE},
 
-	{OP_SK_UP, "up", "UP", _("Move to the previous entry"), KM_SYSKEYS},
-	{OP_SK_DOWN, "down", "DOWN", _("Move to the next entry"), KM_SYSKEYS},
+	{OP_CMD_START_1, "cmd-one", "1", translatable("Start cmdline with 1"), KM_FEEDLIST | KM_ARTICLELIST | KM_SEARCHRESULTSLIST | KM_TAGSELECT | KM_FILTERSELECT},
+	{OP_CMD_START_2, "cmd-two", "2", translatable("Start cmdline with 2"), KM_FEEDLIST | KM_ARTICLELIST | KM_SEARCHRESULTSLIST | KM_TAGSELECT | KM_FILTERSELECT},
+	{OP_CMD_START_3, "cmd-three", "3", translatable("Start cmdline with 3"), KM_FEEDLIST | KM_ARTICLELIST | KM_SEARCHRESULTSLIST | KM_TAGSELECT | KM_FILTERSELECT},
+	{OP_CMD_START_4, "cmd-four", "4", translatable("Start cmdline with 4"), KM_FEEDLIST | KM_ARTICLELIST | KM_SEARCHRESULTSLIST | KM_TAGSELECT | KM_FILTERSELECT},
+	{OP_CMD_START_5, "cmd-five", "5", translatable("Start cmdline with 5"), KM_FEEDLIST | KM_ARTICLELIST | KM_SEARCHRESULTSLIST | KM_TAGSELECT | KM_FILTERSELECT},
+	{OP_CMD_START_6, "cmd-six", "6", translatable("Start cmdline with 6"), KM_FEEDLIST | KM_ARTICLELIST | KM_SEARCHRESULTSLIST | KM_TAGSELECT | KM_FILTERSELECT},
+	{OP_CMD_START_7, "cmd-seven", "7", translatable("Start cmdline with 7"), KM_FEEDLIST | KM_ARTICLELIST | KM_SEARCHRESULTSLIST | KM_TAGSELECT | KM_FILTERSELECT},
+	{OP_CMD_START_8, "cmd-eight", "8", translatable("Start cmdline with 8"), KM_FEEDLIST | KM_ARTICLELIST | KM_SEARCHRESULTSLIST | KM_TAGSELECT | KM_FILTERSELECT},
+	{OP_CMD_START_9, "cmd-nine", "9", translatable("Start cmdline with 9"), KM_FEEDLIST | KM_ARTICLELIST | KM_SEARCHRESULTSLIST | KM_TAGSELECT | KM_FILTERSELECT},
+
+	{OP_SK_UP, "up", "UP", translatable("Move to the previous entry"), KM_SYSKEYS},
+	{OP_SK_DOWN, "down", "DOWN", translatable("Move to the next entry"), KM_SYSKEYS},
 	{
 		OP_SK_PGUP,
 		"pageup",
 		"PPAGE",
-		_("Move to the previous page"),
+		translatable("Move to the previous page"),
 		KM_SYSKEYS
 	},
 	{
 		OP_SK_PGDOWN,
 		"pagedown",
 		"NPAGE",
-		_("Move to the next page"),
+		translatable("Move to the next page"),
 		KM_SYSKEYS
 	},
+	{OP_SK_HALF_PAGE_UP, "halfpageup", "", translatable("Move half page up"), KM_SYSKEYS},
+	{OP_SK_HALF_PAGE_DOWN, "halfpagedown", "", translatable("Move half page down"), KM_SYSKEYS},
 
 	{
 		OP_SK_HOME,
 		"home",
 		"HOME",
-		_("Move to the start of page/list"),
+		translatable("Move to the start of page/list"),
 		KM_SYSKEYS
 	},
 	{
 		OP_SK_END,
 		"end",
 		"END",
-		_("Move to the end of page/list"),
+		translatable("Move to the end of page/list"),
 		KM_SYSKEYS
 	},
 
-	{
-		OP_INT_END_QUESTION,
-		"XXXNOKEY-end-question",
-		"end-question",
-		"",
-		KM_INTERNAL
-	},
-	{
-		OP_INT_CANCEL_QNA,
-		"XXXNOKEY-cancel-qna",
-		"cancel-qna",
-		"",
-		KM_INTERNAL
-	},
-	{
-		OP_INT_QNA_NEXTHIST,
-		"XXXNOKEY-qna-next-history",
-		"qna-next-history",
-		"",
-		KM_INTERNAL
-	},
-	{
-		OP_INT_QNA_PREVHIST,
-		"XXXNOKEY-qna-prev-history",
-		"qna-prev-history",
-		"",
-		KM_INTERNAL
-	},
-
-	{OP_INT_RESIZE, "RESIZE", "internal-resize", "", KM_INTERNAL},
 	{OP_INT_SET, "set", "internal-set", "", KM_INTERNAL},
 
 	{OP_INT_GOTO_URL, "gotourl", "internal-goto-url", "", KM_INTERNAL},
@@ -487,6 +486,7 @@ static const std::map<std::string, std::uint32_t> contexts = {
 	{"podboat", KM_PODBOAT},
 	{"dialogs", KM_DIALOGS},
 	{"dirbrowser", KM_DIRBROWSER},
+	{"searchresultslist", KM_SEARCHRESULTSLIST},
 };
 
 KeyMap::KeyMap(unsigned flags)
@@ -535,7 +535,7 @@ std::vector<KeyMapDesc> KeyMap::get_keymap_descriptions(std::string context)
 			const std::string& key = keymap.first;
 			const Operation op = keymap.second;
 			if (opdesc.op == op) {
-				descs.push_back({key, opdesc.opstr, opdesc.help_text, context, opdesc.flags});
+				descs.push_back({key, opdesc.opstr, _(opdesc.help_text.c_str()), context, opdesc.flags});
 				bound_to_key = true;
 			}
 		}
@@ -544,12 +544,17 @@ std::vector<KeyMapDesc> KeyMap::get_keymap_descriptions(std::string context)
 				"KeyMap::get_keymap_descriptions: found unbound function: %s context = %s",
 				opdesc.opstr,
 				context);
-			descs.push_back({"", opdesc.opstr, opdesc.help_text, context, opdesc.flags});
+			descs.push_back({"", opdesc.opstr, _(opdesc.help_text.c_str()), context, opdesc.flags});
 		}
 	}
 	return descs;
 }
 
+const std::map<std::string, MacroBinding>& KeyMap::get_macro_descriptions()
+{
+	return macros_;
+}
+
 KeyMap::~KeyMap() {}
 
 void KeyMap::set_key(Operation op,
@@ -632,11 +637,11 @@ Operation KeyMap::get_operation(const std::string& keycode,
 	return keymap_[context][key];
 }
 
-void KeyMap::dump_config(std::vector<std::string>& config_output)
+void KeyMap::dump_config(std::vector<std::string>& config_output) const
 {
 	for (const auto& ctx : contexts) {
 		const std::string& context = ctx.first;
-		std::map<std::string, Operation>& x = keymap_[context];
+		const auto& x = keymap_.at(context);
 		for (const auto& keymap : x) {
 			if (keymap.second < OP_INT_MIN) {
 				std::string configline = "bind-key ";
@@ -653,22 +658,29 @@ void KeyMap::dump_config(std::vector<std::string>& config_output)
 		std::string configline = "macro ";
 		configline.append(macro.first);
 		configline.append(" ");
-		for (unsigned int i = 0; i < macro.second.size(); ++i) {
-			const auto& cmd = macro.second[i];
+		for (unsigned int i = 0; i < macro.second.cmds.size(); ++i) {
+			const auto& cmd = macro.second.cmds[i];
 			configline.append(getopname(cmd.op));
 			for (const auto& arg : cmd.args) {
 				configline.append(" ");
 				configline.append(utils::quote(arg));
 			}
-			if (i < (macro.second.size() - 1)) {
+			if (i < (macro.second.cmds.size() - 1)) {
 				configline.append(" ; ");
 			}
 		}
+		if (macro.second.description.size() >= 1) {
+			const auto escaped_string = utils::replace_all(macro.second.description, {
+				{R"(\)", R"(\\)"},
+				{R"(")", R"(\")"},
+			});
+			configline.append(strprintf::fmt(R"( -- "%s")", escaped_string));
+		}
 		config_output.push_back(configline);
 	}
 }
 
-std::string KeyMap::getopname(Operation op)
+std::string KeyMap::getopname(Operation op) const
 {
 	for (const auto& opdesc : opdescs) {
 		if (opdesc.op == op) {
@@ -687,16 +699,17 @@ void KeyMap::handle_action(const std::string& action, const std::string& params)
 	LOG(Level::DEBUG, "KeyMap::handle_action(%s, ...) called", action);
 	if (action == "bind-key") {
 		const auto tokens = utils::tokenize_quoted(params);
-		if (tokens.size() < 2)
-			throw ConfigHandlerException(
-				ActionHandlerStatus::TOO_FEW_PARAMS);
+		if (tokens.size() < 2) {
+			throw ConfigHandlerException(ActionHandlerStatus::TOO_FEW_PARAMS);
+		}
 		std::string context = "all";
 		if (tokens.size() >= 3) {
 			context = tokens[2];
 		}
-		if (!is_valid_context(context))
+		if (!is_valid_context(context)) {
 			throw ConfigHandlerException(strprintf::fmt(
 					_("`%s' is not a valid context"), context));
+		}
 		const Operation op = get_opcode(tokens[1]);
 		if (op == OP_NIL) {
 			throw ConfigHandlerException(
@@ -723,44 +736,61 @@ void KeyMap::handle_action(const std::string& action, const std::string& params)
 	} else if (action == "macro") {
 		std::string remaining_params = params;
 		const auto token = utils::extract_token_quoted(remaining_params);
-		const std::vector<MacroCmd> cmds = parse_operation_sequence(remaining_params);
+		const auto parsed = parse_operation_sequence(remaining_params, action);
+		const std::vector<MacroCmd> cmds = parsed.operations;
+		const std::string description = parsed.description;
 		if (!token.has_value() || cmds.empty()) {
 			throw ConfigHandlerException(ActionHandlerStatus::TOO_FEW_PARAMS);
 		}
 		const std::string macrokey = token.value();
 
-		macros_[macrokey] = cmds;
+		macros_[macrokey] = {cmds, description};
 	} else if (action == "run-on-startup") {
-		startup_operations_sequence = parse_operation_sequence(params);
+		startup_operations_sequence = parse_operation_sequence(params, action, false).operations;
 	} else {
 		throw ConfigHandlerException(ActionHandlerStatus::INVALID_PARAMS);
 	}
 }
 
 
-std::vector<MacroCmd> KeyMap::parse_operation_sequence(const std::string& line)
+ParsedOperations KeyMap::parse_operation_sequence(const std::string& line,
+	const std::string& command_name, bool allow_description)
 {
-	const auto operations = utils::tokenize_quoted(line, ";");
+	rust::String description;
+	bool parsing_failed = false;
+	const auto operations = keymap::bridged::tokenize_operation_sequence(line, description,
+			allow_description, parsing_failed);
+	if (parsing_failed) {
+		throw ConfigHandlerException(strprintf::fmt(_("failed to parse operation sequence for %s"),
+				command_name));
+	}
 
 	std::vector<MacroCmd> cmds;
-	for (auto operation : operations) {
-		const auto command_name = utils::extract_token_quoted(operation);
-		if (!command_name.has_value()) {
+	for (const auto& operation : operations) {
+		const auto& tokens = keymap::bridged::operation_tokens(operation);
+		if (tokens.empty()) {
 			continue;
 		}
-		const auto arguments = utils::tokenize_quoted(operation);
+
+		const auto command_name = std::string(tokens[0]);
+		const auto arguments = std::vector<std::string>(std::next(std::begin(tokens)),
+				std::end(tokens));
+
 		MacroCmd cmd;
-		cmd.op = get_opcode(command_name.value());
+		cmd.op = get_opcode(command_name);
 		if (cmd.op == OP_NIL) {
 			throw ConfigHandlerException(strprintf::fmt(_("`%s' is not a valid operation"),
-					command_name.value()));
+					command_name));
 		}
 		cmd.args = arguments;
 
 		cmds.push_back(cmd);
 	}
 
-	return cmds;
+	return ParsedOperations{
+		.operations = cmds,
+		.description = std::string(description)
+	};
 }
 
 std::vector<MacroCmd> KeyMap::get_startup_operation_sequence()
@@ -783,7 +813,7 @@ std::vector<std::string> KeyMap::get_keys(Operation op,
 std::vector<MacroCmd> KeyMap::get_macro(const std::string& key)
 {
 	if (macros_.count(key) >= 1) {
-		return macros_.at(key);
+		return macros_.at(key).cmds;
 	}
 	return {};
 }
@@ -821,4 +851,28 @@ unsigned short KeyMap::get_flag_from_context(const std::string& context)
 	return 0; // shouldn't happen
 }
 
+
+std::string KeyMap::prepare_keymap_hint(const std::vector<KeyMapHintEntry>& hints,
+	const std::string& context)
+{
+	std::string keymap_hint;
+	for (const auto& hint : hints) {
+		std::vector<std::string> keys = get_keys(hint.op, context);
+
+		if (keys.empty()) {
+			keys = {"<none>"};
+		}
+
+		for (auto& key : keys) {
+			key = strprintf::fmt("<key>%s</>", utils::quote_for_stfl(key));
+		}
+
+		keymap_hint.append(utils::join(keys, "<comma>,</>"));
+		keymap_hint.append("<colon>:</>");
+		keymap_hint.append(strprintf::fmt("<desc>%s</>", utils::quote_for_stfl(hint.text)));
+		keymap_hint.append(" ");
+	}
+	return keymap_hint;
+}
+
 } // namespace newsboat
