@@ -8,6 +8,7 @@
 #include <memory>
 #include <time.h>
 
+#include "curlhandle.h"
 #include "utils.h"
 
 #define OCNEWS_API "/index.php/apps/news/api/v1-2/"
@@ -15,7 +16,6 @@
 namespace newsboat {
 
 typedef std::unique_ptr<json_object, decltype(*json_object_put)> JsonUptr;
-typedef std::unique_ptr<CURL, decltype(*curl_easy_cleanup)> CurlUptr;
 
 OcNewsApi::OcNewsApi(ConfigContainer* c)
 	: RemoteApi(c)
@@ -82,7 +82,11 @@ std::vector<TaggedFeedUrl> OcNewsApi::get_subscribed_urls()
 		long folder_id = json_object_get_int(node);
 
 		json_object_object_get_ex(folder, "name", &node);
-		folders_map[folder_id] = json_object_get_string(node);
+		const auto name = json_object_get_string(node);
+
+		if (name != nullptr) {
+			folders_map[folder_id] = name;
+		}
 	}
 
 	rsspp::Feed starred;
@@ -111,17 +115,24 @@ std::vector<TaggedFeedUrl> OcNewsApi::get_subscribed_urls()
 		long feed_id = json_object_get_int(node);
 
 		json_object_object_get_ex(feed, "title", &node);
-		current_feed.title = json_object_get_string(node);
+		const auto title = json_object_get_string(node);
+		if (title != nullptr) {
+			current_feed.title = title;
+		} else {
+			LOG(Level::WARN, "Subscription has no title, so let's call it \"%i\"", i);
+			current_feed.title = std::string("~") + std::to_string(i);
+		}
 
 		json_object_object_get_ex(feed, "url", &node);
-		current_feed.link = json_object_get_string(node);
+		const auto link = json_object_get_string(node);
+		if (link != nullptr) {
+			current_feed.link = link;
+		}
 
-		while (known_feeds.find(current_feed.title) !=
-			known_feeds.end()) {
+		while (known_feeds.find(current_feed.title) != known_feeds.end()) {
 			current_feed.title += "*";
 		}
-		known_feeds[current_feed.title] =
-			std::make_pair(current_feed, feed_id);
+		known_feeds[current_feed.title] = std::make_pair(current_feed, feed_id);
 
 		json_object_object_get_ex(feed, "folderId", &node);
 		long folder_id = json_object_get_int(node);
@@ -168,26 +179,40 @@ bool OcNewsApi::mark_article_read(const std::string& guid, bool read)
 	return this->query(query, nullptr, "{}");
 }
 
+bool OcNewsApi::mark_articles_read(const std::vector<std::string>& guids)
+{
+	std::vector<std::string> ids;
+	for (const auto& guid : guids) {
+		ids.push_back(guid.substr(0, guid.find_first_of(":")));
+	}
+
+	const std::string query = "items/read/multiple";
+	const std::string id_array = strprintf::fmt("[%s]", utils::join(ids, ","));
+	const std::string parameters = strprintf::fmt(R"({"items": %s})", id_array);
+	return this->query(query, nullptr, parameters);
+}
+
 bool OcNewsApi::update_article_flags(const std::string& oldflags,
 	const std::string& newflags,
 	const std::string& guid)
 {
 	std::string star_flag = cfg->get_configvalue("ocnews-flag-star");
-	std::string query = "items/";
-	query += guid.substr(guid.find_first_of(":") + 1);
-
+	bool success = true;
 	if (star_flag.length() > 0) {
-		if (strchr(oldflags.c_str(), star_flag[0]) == nullptr &&
-			strchr(newflags.c_str(), star_flag[0]) != nullptr) {
-			query += "/star";
-		} else if (strchr(oldflags.c_str(), star_flag[0]) != nullptr &&
-			strchr(newflags.c_str(), star_flag[0]) == nullptr) {
-			query += "/unstar";
-		}
+		update_flag(oldflags, newflags, star_flag[0], [&](bool added) {
+			const auto feedIdStart = guid.find_first_of(':') + 1;
+			const auto feedIdEnd = guid.find_first_of('/');
+			const auto feedId = guid.substr(feedIdStart, feedIdEnd - feedIdStart);
+			const auto ocnewsGuid = guid.substr(feedIdEnd + 1);
+			const auto query = strprintf::fmt(
+					"items/%s/%s/%s",
+					feedId,
+					utils::md5hash(ocnewsGuid),
+					added ? "star" : "unstar");
+			success = this->query(query, nullptr, "{}");
+		});
 	}
-
-	return this->query(query, nullptr, "{}");
-	;
+	return success;
 }
 
 rsspp::Feed OcNewsApi::fetch_feed(const std::string& feed_id)
@@ -224,35 +249,47 @@ rsspp::Feed OcNewsApi::fetch_feed(const std::string& feed_id)
 		rsspp::Item item;
 
 		json_object_object_get_ex(item_j, "title", &node);
-		item.title = json_object_get_string(node);
+		const auto title = json_object_get_string(node);
+		if (title != nullptr) {
+			item.title = title;
+		}
 
 		json_object_object_get_ex(item_j, "url", &node);
-		if (node) {
-			item.link = json_object_get_string(node);
+		const auto link = json_object_get_string(node);
+		if (link != nullptr) {
+			item.link = link;
 		}
 
 		json_object_object_get_ex(item_j, "author", &node);
-		item.author = json_object_get_string(node);
+		const auto author = json_object_get_string(node);
+		if (author != nullptr) {
+			item.author = author;
+		}
 
 		json_object_object_get_ex(item_j, "body", &node);
-		item.content_encoded = json_object_get_string(node);
+		const auto content_encoded = json_object_get_string(node);
+		if (content_encoded != nullptr) {
+			item.content_encoded = content_encoded;
+		}
 
 		{
 			json_object* type_obj;
 
-			json_object_object_get_ex(
-				item_j, "enclosureMime", &type_obj);
-			json_object_object_get_ex(
-				item_j, "enclosureLink", &node);
+			json_object_object_get_ex(item_j, "enclosureMime", &type_obj);
+			json_object_object_get_ex(item_j, "enclosureLink", &node);
 
-			if (type_obj && node) {
-				const std::string type =
-					json_object_get_string(type_obj);
-				if (utils::is_valid_podcast_type(type)) {
-					item.enclosure_url =
-						json_object_get_string(node);
-					item.enclosure_type = std::move(type);
+			const auto type_ptr = json_object_get_string(type_obj);
+			const auto url_ptr = json_object_get_string(node);
+
+			if (type_ptr != nullptr && url_ptr != nullptr) {
+				const std::string type = type_ptr;
+				const std::string url = url_ptr;
+				item.enclosures.push_back(
+				rsspp::Enclosure {
+					url,
+					type,
 				}
+				);
 			}
 		}
 
@@ -263,8 +300,9 @@ rsspp::Feed OcNewsApi::fetch_feed(const std::string& feed_id)
 		long f_id = json_object_get_int(node);
 
 		json_object_object_get_ex(item_j, "guid", &node);
+		const auto guid = json_object_get_string(node);
 		item.guid = std::to_string(id) + ":" + std::to_string(f_id) +
-			"/" + json_object_get_string(node);
+			"/" + (guid ? std::string(guid) : std::to_string(i));
 
 		json_object_object_get_ex(item_j, "unread", &node);
 		bool unread = json_object_get_boolean(node);
@@ -296,11 +334,11 @@ bool OcNewsApi::query(const std::string& query,
 	json_object** result,
 	const std::string& post)
 {
-	CurlUptr curlhandle(curl_easy_init(), curl_easy_cleanup);
-	CURL* handle = curlhandle.get();
+	CurlHandle handle;
 
 	std::string url = server + OCNEWS_API + query;
-	curl_easy_setopt(handle, CURLOPT_URL, url.c_str());
+	curl_easy_setopt(handle.ptr(), CURLOPT_URL, url.c_str());
+	curl_slist* headers = NULL;
 
 	utils::set_common_curl_options(handle, cfg);
 
@@ -312,19 +350,24 @@ bool OcNewsApi::query(const std::string& query,
 		return size * nmemb;
 	};
 	std::string buff;
-	curl_easy_setopt(handle, CURLOPT_WRITEFUNCTION, *write_fn);
-	curl_easy_setopt(handle, CURLOPT_WRITEDATA, &buff);
+	curl_easy_setopt(handle.ptr(), CURLOPT_WRITEFUNCTION, *write_fn);
+	curl_easy_setopt(handle.ptr(), CURLOPT_WRITEDATA, &buff);
 
 	if (!post.empty()) {
-		curl_easy_setopt(handle, CURLOPT_POST, 1);
-		curl_easy_setopt(handle, CURLOPT_POSTFIELDS, post.c_str());
-		curl_easy_setopt(handle, CURLOPT_CUSTOMREQUEST, "PUT");
+		curl_easy_setopt(handle.ptr(), CURLOPT_POST, 1);
+		curl_easy_setopt(handle.ptr(), CURLOPT_POSTFIELDS, post.c_str());
+		curl_easy_setopt(handle.ptr(), CURLOPT_CUSTOMREQUEST, "PUT");
+
+		headers = curl_slist_append(headers, "Content-Type: application/json");
+		curl_easy_setopt(handle.ptr(), CURLOPT_HTTPHEADER, headers);
 	}
 
-	curl_easy_setopt(handle, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
-	curl_easy_setopt(handle, CURLOPT_USERPWD, auth.c_str());
+	curl_easy_setopt(handle.ptr(), CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
+	curl_easy_setopt(handle.ptr(), CURLOPT_USERPWD, auth.c_str());
 
-	CURLcode res = curl_easy_perform(handle);
+	CURLcode res = curl_easy_perform(handle.ptr());
+
+	curl_slist_free_all(headers);
 
 	if (res != CURLE_OK && res != CURLE_HTTP_RETURNED_ERROR) {
 		LOG(Level::CRITICAL,
@@ -335,15 +378,15 @@ bool OcNewsApi::query(const std::string& query,
 	}
 
 	long response_code;
-	curl_easy_getinfo(handle, CURLINFO_RESPONSE_CODE, &response_code);
+	curl_easy_getinfo(handle.ptr(), CURLINFO_RESPONSE_CODE, &response_code);
 	if (response_code != 200) {
 		if (response_code == 401)
 			LOG(Level::CRITICAL,
 				"OcNewsApi::query: authentication error");
 		else {
-			std::string msg = "OcNewsApi::query: error ";
-			msg += response_code;
-			LOG(Level::CRITICAL, msg);
+			LOG(Level::CRITICAL,
+				"OcNewsApi::query: error %" PRIi64,
+				static_cast<int64_t>(response_code));
 		}
 		return false;
 	}
