@@ -1,5 +1,6 @@
 #include "opml.h"
 
+#include <algorithm>
 #include <cassert>
 #include <cinttypes>
 #include <cstring>
@@ -8,12 +9,14 @@
 
 namespace newsboat {
 
-xmlDocPtr opml::generate(const FeedContainer& feedcontainer)
+xmlDocPtr opml::generate(const FeedContainer& feedcontainer, bool version2)
 {
 	xmlDocPtr root = xmlNewDoc((const xmlChar*)"1.0");
 	xmlNodePtr opml_node =
 		xmlNewDocNode(root, nullptr, (const xmlChar*)"opml", nullptr);
-	xmlSetProp(opml_node, (const xmlChar*)"version", (const xmlChar*)"1.0");
+	xmlSetProp(opml_node,
+		(const xmlChar*)"version",
+		(const xmlChar*)(version2 ? "2.0" : "1.0"));
 	xmlDocSetRootElement(root, opml_node);
 
 	xmlNodePtr head = xmlNewTextChild(
@@ -47,6 +50,27 @@ xmlDocPtr opml::generate(const FeedContainer& feedcontainer)
 			xmlSetProp(outline,
 				(const xmlChar*)"title",
 				(const xmlChar*)title.c_str());
+
+			if (version2) {
+				// OPML 2.0 supports including tags
+				std::vector<std::string> tags = feed->get_tags();
+				std::string opml_tags;
+
+				bool first_tag = true;
+				for (auto t : tags) {
+					utils::trim(t);
+					t = utils::replace_all(t, ",", "_");
+					if (first_tag) {
+						first_tag = false;
+					} else {
+						opml_tags.append(",");
+					}
+					opml_tags.append(t);
+				}
+				xmlSetProp(outline,
+					(const xmlChar*)"category",
+					(const xmlChar*)opml_tags.c_str());
+			}
 		}
 	}
 
@@ -62,14 +86,17 @@ void rec_find_rss_outlines(
 		std::string newtag = tag;
 
 		if (strcmp((const char*)node->name, "outline") == 0) {
-			char* url = (char*)xmlGetProp(
+			char* url_p = (char*)xmlGetProp(
 					node, (const xmlChar*)"xmlUrl");
-			if (!url) {
-				url = (char*)xmlGetProp(
+			if (!url_p) {
+				url_p = (char*)xmlGetProp(
 						node, (const xmlChar*)"url");
 			}
 
-			if (url) {
+			if (url_p) {
+				const std::string url(url_p);
+				xmlFree(url_p);
+
 				LOG(Level::DEBUG,
 					"opml::import: found RSS outline with "
 					"url = "
@@ -81,13 +108,12 @@ void rec_find_rss_outlines(
 				// Liferea uses a pipe to signal feeds read from
 				// the output of a program in its OPMLs. Convert
 				// them to our syntax.
-				if (*url == '|') {
+				if (url.length() >= 1 && url[0] == '|') {
 					nurl = strprintf::fmt(
-							"exec:%s", url + 1);
+							"exec:%s", url.substr(1));
 					LOG(Level::DEBUG,
 						"opml::import: liferea-style "
-						"url %s "
-						"converted to %s",
+						"url %s converted to %s",
 						url,
 						nurl);
 				}
@@ -107,55 +133,30 @@ void rec_find_rss_outlines(
 					xmlFree(filtercmd);
 				}
 
-				xmlFree(url);
 				// Filters and scripts may have arguments, so,
 				// quote them when needed.
-				// TODO: get rid of xmlStrdup, it's useless
-				url = (char*)xmlStrdup(
-						(const xmlChar*)
-						utils::quote_if_necessary(nurl)
-						.c_str());
-				assert(url);
-
-				bool found = false;
+				const std::string quoted_url = utils::quote_if_necessary(nurl);
 
 				LOG(Level::DEBUG,
 					"opml::import: size = %" PRIu64,
 					static_cast<uint64_t>(urlcfg.get_urls().size()));
-				// TODO: replace with algorithm::any or something
-				if (urlcfg.get_urls().size() > 0) {
-					for (const auto& u :
-						urlcfg.get_urls()) {
-						if (u == url) {
-							found = true;
-						}
-					}
-				}
 
-				if (!found) {
-					LOG(Level::DEBUG,
-						"opml::import: added url = %s",
-						url);
-					urlcfg.get_urls().push_back(
-						std::string(url));
+				auto& urls = urlcfg.get_urls();
+				if (std::find(urls.begin(), urls.end(), quoted_url) == urls.end()) {
+					LOG(Level::DEBUG, "opml::import: added url = %s", quoted_url);
+					urls.push_back(quoted_url);
 					if (tag.length() > 0) {
 						LOG(Level::DEBUG,
-							"opml::import: "
-							"appending "
-							"tag %s to url %s",
+							"opml::import: appending tag %s to url %s",
 							tag,
-							url);
-						urlcfg.get_tags(url).push_back(
-							tag);
+							quoted_url);
+						urlcfg.get_tags(quoted_url).push_back(tag);
 					}
 				} else {
 					LOG(Level::DEBUG,
-						"opml::import: url = %s is "
-						"already "
-						"in list",
-						url);
+						"opml::import: url = %s is already in list",
+						quoted_url);
 				}
-				xmlFree(url);
 			} else {
 				char* text = (char*)xmlGetProp(
 						node, (const xmlChar*)"text");
@@ -178,29 +179,34 @@ void rec_find_rss_outlines(
 	}
 }
 
-bool opml::import(
+nonstd::optional<std::string> opml::import(
 	const std::string& filename,
 	FileUrlReader& urlcfg)
 {
 	xmlDoc* doc = xmlReadFile(filename.c_str(), nullptr, 0);
 	if (doc == nullptr) {
-		return false;
+		return strprintf::fmt(_("Error: failed to parse OPML file \"%s\""), filename);
 	}
 
-	xmlNode* root = xmlDocGetRootElement(doc);
+	nonstd::optional<std::string> error_message;
 
+	xmlNode* root = xmlDocGetRootElement(doc);
 	for (xmlNode* node = root->children; node != nullptr;
 		node = node->next) {
 		if (strcmp((const char*)node->name, "body") == 0) {
 			LOG(Level::DEBUG, "opml::import: found body");
 			rec_find_rss_outlines(urlcfg, node->children, "");
-			urlcfg.write_config();
+
+			error_message = urlcfg.write_config();
+			if (error_message.has_value()) {
+				break;
+			}
 		}
 	}
 
 	xmlFreeDoc(doc);
 
-	return true;
+	return error_message;
 }
 
 } // namespace newsboat
