@@ -1,5 +1,7 @@
 #include "pbcontroller.h"
 
+#include <cinttypes>
+#include <cstdint>
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
@@ -15,7 +17,7 @@
 #include "config.h"
 #include "configcontainer.h"
 #include "configexception.h"
-#include "globals.h"
+#include "configparser.h"
 #include "logger.h"
 #include "matcherexception.h"
 #include "nullconfigactionhandler.h"
@@ -106,32 +108,22 @@ bool PbController::setup_dirs_xdg(const char* env_home)
 	}
 
 	/* in config */
-	url_file = config_dir + NEWSBEUTER_PATH_SEP + url_file;
 	config_file =
 		config_dir + NEWSBEUTER_PATH_SEP + config_file;
 
 	/* in data */
-	cache_file =
-		xdg_data_dir + NEWSBEUTER_PATH_SEP + cache_file;
-	lock_file = cache_file + LOCK_SUFFIX;
+	const std::string LOCK_SUFFIX(".lock");
+	lock_file = xdg_data_dir + NEWSBEUTER_PATH_SEP + LOCK_SUFFIX;
 	queue_file =
 		xdg_data_dir + NEWSBEUTER_PATH_SEP + queue_file;
-	searchfile = strprintf::fmt(
-			"%s%chistory.search", xdg_data_dir, NEWSBEUTER_PATH_SEP);
-	cmdlinefile = strprintf::fmt(
-			"%s%chistory.cmdline", xdg_data_dir, NEWSBEUTER_PATH_SEP);
 
 	return true;
 }
 
 PbController::PbController()
-	: v(0)
-	, config_file("config")
+	: config_file("config")
 	, queue_file("queue")
-	, cfg(0)
-	, view_update_(true)
 	, max_dls(1)
-	, ql(0)
 	, lock_file("pb-lock.pid")
 	, keys(KM_PODBOAT)
 {
@@ -180,11 +172,6 @@ PbController::PbController()
 	lock_file = config_dir + NEWSBEUTER_PATH_SEP + lock_file;
 }
 
-PbController::~PbController()
-{
-	delete cfg;
-}
-
 void PbController::initialize(int argc, char* argv[])
 {
 	int c;
@@ -220,12 +207,12 @@ void PbController::initialize(int argc, char* argv[])
 			automatic_dl = true;
 			break;
 		case 'd':
-			Logger::set_logfile(optarg);
+			logger::set_logfile(optarg);
 			break;
 		case 'l': {
 			Level l = static_cast<Level>(atoi(optarg));
-			if (l > Level::NONE && l <= Level::DEBUG) {
-				Logger::set_loglevel(l);
+			if (l >= Level::USERERROR && l <= Level::DEBUG) {
+				logger::set_loglevel(l);
 			} else {
 				std::cerr << strprintf::fmt(_("%s: %d: invalid "
 							"loglevel value"),
@@ -243,18 +230,23 @@ void PbController::initialize(int argc, char* argv[])
 	};
 
 	std::cout << strprintf::fmt(
-			_("Starting %s %s..."), "podboat", utils::program_version())
+			_("Starting %s %s..."), "Podboat", utils::program_version())
 		<< std::endl;
 
 	fslock = std::unique_ptr<FsLock>(new FsLock());
 	pid_t pid;
-	if (!fslock->try_lock(lock_file, pid)) {
-		std::cout << strprintf::fmt(
-				_("Error: an instance of %s is already "
-					"running (PID: %u)"),
-				"podboat",
-				pid)
-			<< std::endl;
+	std::string error_message;
+	if (!fslock->try_lock(lock_file, pid, error_message)) {
+		if (pid != 0) {
+			std::cout << strprintf::fmt(
+					_("Error: an instance of %s is already "
+						"running (PID: %s)"),
+					"Podboat",
+					std::to_string(pid))
+				<< std::endl;
+		} else {
+			std::cout << _("Error: ") << error_message << std::endl;
+		}
 		exit(EXIT_FAILURE);
 	}
 
@@ -262,8 +254,7 @@ void PbController::initialize(int argc, char* argv[])
 	std::cout.flush();
 
 	ConfigParser cfgparser;
-	cfg = new ConfigContainer();
-	cfg->register_commands(cfgparser);
+	cfg.register_commands(cfgparser);
 	colorman.register_commands(cfgparser);
 
 	cfgparser.register_handler("bind-key", keys);
@@ -276,7 +267,9 @@ void PbController::initialize(int argc, char* argv[])
 	cfgparser.register_handler("define-filter", null_cah);
 	cfgparser.register_handler("highlight", null_cah);
 	cfgparser.register_handler("highlight-article", null_cah);
+	cfgparser.register_handler("highlight-feed", null_cah);
 	cfgparser.register_handler("reset-unread-on-update", null_cah);
+	cfgparser.register_handler("run-on-startup", null_cah);
 
 	try {
 		cfgparser.parse_file("/etc/newsboat/config");
@@ -287,21 +280,20 @@ void PbController::initialize(int argc, char* argv[])
 	}
 }
 
-int PbController::run()
+int PbController::run(PbView& v)
 {
-	v->apply_colors_to_all_forms();
+	v.apply_colors_to_all_forms();
 
-	max_dls = cfg->get_configvalue_as_int("max-downloads");
+	max_dls = cfg.get_configvalue_as_int("max-downloads");
 
 	std::cout << _("done.") << std::endl;
 
-	ql = new QueueLoader(queue_file, *cfg,
-		std::bind(&PbController::set_view_update_necessary, this, true));
+	ql.reset(new QueueLoader(queue_file, cfg, [&]() {
+		v.set_view_update_necessary();
+	}));
 	ql->reload(downloads_);
 
-	v->set_keymap(&keys);
-
-	v->run(automatic_dl, cfg->get_configvalue_as_bool("wrap-scroll"));
+	v.run(automatic_dl, cfg.get_configvalue_as_bool("wrap-scroll"));
 
 	Stfl::reset();
 
@@ -309,18 +301,22 @@ int PbController::run()
 	std::cout.flush();
 
 	ql->reload(downloads_);
-	delete ql;
 
 	std::cout << _("done.") << std::endl;
 
 	return EXIT_SUCCESS;
 }
 
+newsboat::KeyMap& PbController::get_keymap()
+{
+	return keys;
+}
+
 void PbController::print_usage(const char* argv0)
 {
 	auto msg = strprintf::fmt(
 			_("%s %s\nusage %s [-C <file>] [-q <file>] [-h]\n"),
-			"podboat",
+			"Podboat",
 			utils::program_version(),
 			argv0);
 	std::cout << msg;
@@ -374,11 +370,13 @@ void PbController::print_usage(const char* argv0)
 		}
 		std::cout << a.desc << std::endl;
 	}
-}
 
-std::string PbController::get_formatstr()
-{
-	return cfg->get_configvalue("podlist-format");
+	std::cout << std::endl
+		<< _("Support at #newsboat at https://libera.chat or on our mailing "
+			"list https://groups.google.com/g/newsboat")
+		<< std::endl
+		<< _("For more information, check out https://newsboat.org/")
+		<< std::endl;
 }
 
 unsigned int PbController::downloads_in_progress()
@@ -399,7 +397,7 @@ unsigned int PbController::get_maxdownloads()
 
 void PbController::purge_queue()
 {
-	if (ql) {
+	if (ql != nullptr) {
 		ql->reload(downloads_, true);
 	}
 }
@@ -424,13 +422,18 @@ void PbController::start_downloads()
 		}
 
 		if (download.status() == DlStatus::QUEUED) {
-			std::thread t{PodDlThread(&download, cfg)};
+			start_download(download);
 			--dl2start;
-			t.detach();
 		}
 	}
 }
 
+void PbController::start_download(Download& item)
+{
+	std::thread t{PodDlThread(&item, &cfg)};
+	t.detach();
+}
+
 void PbController::increase_parallel_downloads()
 {
 	++max_dls;
@@ -446,7 +449,7 @@ void PbController::decrease_parallel_downloads()
 void PbController::play_file(const std::string& file)
 {
 	std::string cmdline;
-	std::string player = cfg->get_configvalue("player");
+	std::string player = cfg.get_configvalue("player");
 	if (player == "") {
 		return;
 	}
