@@ -3,7 +3,7 @@
 #include <cinttypes>
 #include <cstdio>
 #include <cstring>
-#include <curses.h>
+#include <ncurses.h>
 #include <iostream>
 #include <sstream>
 
@@ -16,7 +16,6 @@
 #include "listformatter.h"
 #include "logger.h"
 #include "pbcontroller.h"
-#include "poddlthread.h"
 #include "strprintf.h"
 #include "utils.h"
 
@@ -24,14 +23,15 @@ using namespace newsboat;
 
 namespace podboat {
 
-PbView::PbView(PbController* c)
-	: ctrl(c)
+PbView::PbView(PbController& c)
+	: update_view(true)
+	, ctrl(c)
 	, dllist_form(dllist_str)
 	, help_form(help_str)
-	, keys(0)
-	, colorman(ctrl->get_colormanager())
+	, keys(ctrl.get_keymap())
+	, colorman(ctrl.get_colormanager())
 	, downloads_list("dls", dllist_form,
-		  ctrl->get_cfgcont()->get_configvalue_as_int("scrolloff"))
+		  ctrl.get_cfgcont()->get_configvalue_as_int("scrolloff"))
 	, help_textview("helptext", help_form)
 {
 	if (getenv("ESCDELAY") == nullptr) {
@@ -56,63 +56,61 @@ void PbView::run(bool auto_download, bool wrap_scroll)
 	set_dllist_keymap_hint();
 
 	do {
-		if (ctrl->view_update_necessary()) {
-			const double total_kbps = ctrl->get_total_kbps();
+		if (update_view) {
+			const double total_kbps = ctrl.get_total_kbps();
 			const auto speed = get_speed_human_readable(total_kbps);
 
-			char parbuf[128] = "";
-			if (ctrl->get_maxdownloads() > 1) {
-				snprintf(parbuf,
-					sizeof(parbuf),
-					_(" - %u parallel downloads"),
-					ctrl->get_maxdownloads());
+			auto title = strprintf::fmt(
+					_("Queue (%u downloads in progress, %u total) - %.2f %s total"),
+					static_cast<unsigned int>(ctrl.downloads_in_progress()),
+					static_cast<unsigned int>(ctrl.downloads().size()),
+					speed.first,
+					speed.second);
+
+			if (ctrl.get_maxdownloads() > 1) {
+				title += strprintf::fmt(_(" - %u parallel downloads"), ctrl.get_maxdownloads());
 			}
 
-			char buf[1024];
-			snprintf(buf,
-				sizeof(buf),
-				_("Queue (%u downloads in progress, %u total) "
-					"- %.2f %s total%s"),
-				static_cast<unsigned int>(
-					ctrl->downloads_in_progress()),
-				static_cast<unsigned int>(
-					ctrl->downloads().size()),
-				speed.first,
-				speed.second.c_str(),
-				parbuf);
-
-			dllist_form.set("head", buf);
+			dllist_form.set("head", title);
 
 			LOG(Level::DEBUG,
 				"PbView::run: updating view... "
 				"downloads().size() "
 				"= %" PRIu64,
-				static_cast<uint64_t>(ctrl->downloads().size()));
+				static_cast<uint64_t>(ctrl.downloads().size()));
 
 			ListFormatter listfmt;
-			std::string formatstring = ctrl->get_formatstr();
+			const std::string line_format =
+				ctrl.get_cfgcont()->get_configvalue("podlist-format");
 
 			dllist_form.run(-3); // compute all widget dimensions
 			const unsigned int width = downloads_list.get_width();
 
 			unsigned int i = 0;
-			for (const auto& dl : ctrl->downloads()) {
-				auto lbuf = format_line(formatstring, dl, i, width);
+			for (const auto& dl : ctrl.downloads()) {
+				auto lbuf = format_line(line_format, dl, i, width);
 				listfmt.add_line(lbuf);
 				i++;
 			}
 
 			downloads_list.stfl_replace_lines(listfmt);
 
-			ctrl->set_view_update_necessary(false);
+			update_view = false;
+		}
+
+		// If there's no status message, we know there's no error to show
+		// Thus, it's safe to replace with the download's status
+		if (dllist_form.get("msg").empty() && ctrl.downloads().size() > 0) {
+			const auto idx = downloads_list.get_position();
+			dllist_form.set("msg", ctrl.downloads()[idx].status_msg());
 		}
 
 		const char* event = dllist_form.run(500);
 
 		if (auto_download) {
-			if (ctrl->get_maxdownloads() >
-				ctrl->downloads_in_progress()) {
-				ctrl->start_downloads();
+			if (ctrl.get_maxdownloads() >
+				ctrl.downloads_in_progress()) {
+				ctrl.start_downloads();
 			}
 		}
 
@@ -120,11 +118,16 @@ void PbView::run(bool auto_download, bool wrap_scroll)
 			continue;
 		}
 
-		Operation op = keys->get_operation(event, "podboat");
+		if (strcmp(event, "RESIZE") == 0) {
+			handle_resize();
+			continue;
+		}
+
+		Operation op = keys.get_operation(event, "podboat");
 
 		if (dllist_form.get("msg").length() > 0) {
 			dllist_form.set("msg", "");
-			ctrl->set_view_update_necessary(true);
+			update_view = true;
 		}
 
 		switch (op) {
@@ -151,50 +154,51 @@ void PbView::run(bool auto_download, bool wrap_scroll)
 		case OP_SK_PGDOWN:
 			downloads_list.move_page_down(wrap_scroll);
 			break;
+		case OP_SK_HALF_PAGE_UP:
+			downloads_list.scroll_halfpage_up(wrap_scroll);
+			break;
+		case OP_SK_HALF_PAGE_DOWN:
+			downloads_list.scroll_halfpage_down(wrap_scroll);
+			break;
 		case OP_PB_TOGGLE_DLALL:
 			auto_download = !auto_download;
 			break;
 		case OP_HARDQUIT:
 		case OP_QUIT:
-			if (ctrl->downloads_in_progress() > 0) {
-				dllist_form.set("msg",
-					_("Error: can't quit: download(s) in "
-						"progress."));
-				ctrl->set_view_update_necessary(true);
+			if (ctrl.downloads_in_progress() > 0) {
+				dllist_form.set("msg", _("Error: can't quit: download(s) in progress."));
+				update_view = true;
 			} else {
 				quit = true;
 			}
 			break;
 		case OP_PB_MOREDL:
-			ctrl->increase_parallel_downloads();
+			ctrl.increase_parallel_downloads();
 			break;
 		case OP_PB_LESSDL:
-			ctrl->decrease_parallel_downloads();
+			ctrl.decrease_parallel_downloads();
 			break;
 		case OP_PB_DOWNLOAD: {
-			if (ctrl->downloads().size() >= 1) {
+			if (ctrl.downloads().size() >= 1) {
 				const auto idx = downloads_list.get_position();
-				if (ctrl->downloads()[idx].status() !=
-					DlStatus::DOWNLOADING) {
-					std::thread t{PodDlThread(
-							&ctrl->downloads()[idx],
-							ctrl->get_cfgcont())};
-					t.detach();
+				auto& item = ctrl.downloads()[idx];
+				if (item.status() != DlStatus::DOWNLOADING) {
+					ctrl.start_download(item);
 				}
 			}
 		}
 		break;
 		case OP_PB_PLAY: {
-			if (ctrl->downloads().size() >= 1) {
+			if (ctrl.downloads().size() >= 1) {
 				const auto idx = downloads_list.get_position();
 				DlStatus status =
-					ctrl->downloads()[idx].status();
+					ctrl.downloads()[idx].status();
 				if (status == DlStatus::FINISHED ||
 					status == DlStatus::PLAYED ||
 					status == DlStatus::READY) {
-					ctrl->play_file(ctrl->downloads()[idx]
+					ctrl.play_file(ctrl.downloads()[idx]
 						.filename());
-					ctrl->downloads()[idx].set_status(
+					ctrl.downloads()[idx].set_status(
 						DlStatus::PLAYED);
 				} else {
 					dllist_form.set("msg",
@@ -206,48 +210,54 @@ void PbView::run(bool auto_download, bool wrap_scroll)
 		}
 		break;
 		case OP_PB_MARK_FINISHED: {
-			if (ctrl->downloads().size() >= 1) {
+			auto& downloads = ctrl.downloads();
+			if (downloads.size() >= 1) {
 				const auto idx = downloads_list.get_position();
 				DlStatus status =
-					ctrl->downloads()[idx].status();
+					downloads[idx].status();
 				if (status == DlStatus::PLAYED) {
-					ctrl->downloads()[idx].set_status(
-						DlStatus::FINISHED);
+					downloads[idx].set_status(DlStatus::FINISHED);
+					if (idx + 1 < downloads.size()) {
+						downloads_list.set_position(idx + 1);
+					}
 				}
 			}
 		}
 		break;
 		case OP_PB_CANCEL: {
-			if (ctrl->downloads().size() >= 1) {
+			if (ctrl.downloads().size() >= 1) {
 				const auto idx = downloads_list.get_position();
-				if (ctrl->downloads()[idx].status() ==
+				if (ctrl.downloads()[idx].status() ==
 					DlStatus::DOWNLOADING) {
-					ctrl->downloads()[idx].set_status(
+					ctrl.downloads()[idx].set_status(
 						DlStatus::CANCELLED);
 				}
 			}
 		}
 		break;
 		case OP_PB_DELETE: {
-			if (ctrl->downloads().size() >= 1) {
+			auto& downloads = ctrl.downloads();
+			if (downloads.size() >= 1) {
 				const auto idx = downloads_list.get_position();
-				if (ctrl->downloads()[idx].status() !=
+				if (downloads[idx].status() !=
 					DlStatus::DOWNLOADING) {
-					ctrl->downloads()[idx].set_status(
-						DlStatus::DELETED);
+					downloads[idx].set_status(DlStatus::DELETED);
+					if (idx + 1 < downloads.size()) {
+						downloads_list.set_position(idx + 1);
+					}
 				}
 			}
 		}
 		break;
 		case OP_PB_PURGE:
-			if (ctrl->downloads_in_progress() > 0) {
+			if (ctrl.downloads_in_progress() > 0) {
 				dllist_form.set("msg",
 					_("Error: unable to perform operation: "
 						"download(s) in progress."));
 			} else {
-				ctrl->purge_queue();
+				ctrl.purge_queue();
 			}
-			ctrl->set_view_update_necessary(true);
+			update_view = true;
 			break;
 		case OP_HELP:
 			run_help();
@@ -259,10 +269,22 @@ void PbView::run(bool auto_download, bool wrap_scroll)
 	} while (!quit);
 }
 
+void PbView::handle_resize()
+{
+	std::vector<std::reference_wrapper<newsboat::Stfl::Form>> forms = {dllist_form, help_form};
+	for (const auto& form : forms) {
+		form.get().run(-3);
+	}
+	update_view = true;
+}
+
 void PbView::apply_colors_to_all_forms()
 {
-	colorman.apply_colors(dllist_form);
-	colorman.apply_colors(help_form);
+	using namespace std::placeholders;
+	colorman.apply_colors(std::bind(&newsboat::Stfl::Form::set, &dllist_form, _1,
+			_2));
+	colorman.apply_colors(std::bind(&newsboat::Stfl::Form::set, &help_form, _1,
+			_2));
 }
 
 std::pair<double, std::string> PbView::get_speed_human_readable(double kbps)
@@ -282,17 +304,15 @@ void PbView::run_help()
 
 	help_form.set("head", _("Help"));
 
-	const auto descs = keys->get_keymap_descriptions("podboat");
+	const auto descs = keys.get_keymap_descriptions("podboat");
 
 	ListFormatter listfmt;
 
 	for (const auto& desc : descs) {
-		std::string descline;
-		descline.append(desc.key);
-		descline.append(8 - desc.key.length(), ' ');
-		descline.append(desc.cmd);
-		descline.append(24 - desc.cmd.length(), ' ');
-		descline.append(desc.desc);
+		const std::string descline = strprintf::fmt("%-7s %-23s %s",
+				desc.key,
+				desc.cmd,
+				desc.desc);
 
 		listfmt.add_line(descline);
 	}
@@ -308,7 +328,12 @@ void PbView::run_help()
 			continue;
 		}
 
-		Operation op = keys->get_operation(event, "help");
+		if (strcmp(event, "RESIZE") == 0) {
+			handle_resize();
+			continue;
+		}
+
+		Operation op = keys.get_operation(event, "help");
 
 		switch (op) {
 		case OP_SK_UP:
@@ -329,6 +354,12 @@ void PbView::run_help()
 		case OP_SK_PGDOWN:
 			help_textview.scroll_page_down();
 			break;
+		case OP_SK_HALF_PAGE_UP:
+			help_textview.scroll_halfpage_up();
+			break;
+		case OP_SK_HALF_PAGE_DOWN:
+			help_textview.scroll_halfpage_down();
+			break;
 		case OP_HARDQUIT:
 		case OP_QUIT:
 			quit = true;
@@ -339,33 +370,16 @@ void PbView::run_help()
 	} while (!quit);
 }
 
-std::string PbView::prepare_keymaphint(KeyMapHintEntry* hints)
-{
-	std::string keymap_hint;
-	for (int i = 0; hints[i].op != OP_NIL; ++i) {
-		std::string bound_keys = utils::join(keys->get_keys(hints[i].op, "podboat"),
-				",");
-		if (bound_keys.empty()) {
-			bound_keys = "<none>";
-		}
-		keymap_hint.append(bound_keys);
-		keymap_hint.append(":");
-		keymap_hint.append(hints[i].text);
-		keymap_hint.append(" ");
-	}
-	return keymap_hint;
-}
-
 void PbView::set_help_keymap_hint()
 {
-	KeyMapHintEntry hints[] = {{OP_QUIT, _("Quit")}, {OP_NIL, nullptr}};
-	std::string keymap_hint = prepare_keymaphint(hints);
+	static const std::vector<KeyMapHintEntry> hints = {{OP_QUIT, _("Quit")}};
+	const auto keymap_hint = keys.prepare_keymap_hint(hints, "podboat");
 	help_form.set("help", keymap_hint);
 }
 
 void PbView::set_dllist_keymap_hint()
 {
-	KeyMapHintEntry hints[] = {{OP_QUIT, _("Quit")},
+	static const std::vector<KeyMapHintEntry> hints = {{OP_QUIT, _("Quit")},
 		{OP_PB_DOWNLOAD, _("Download")},
 		{OP_PB_CANCEL, _("Cancel")},
 		{OP_PB_DELETE, _("Delete")},
@@ -373,11 +387,10 @@ void PbView::set_dllist_keymap_hint()
 		{OP_PB_TOGGLE_DLALL, _("Toggle Automatic Download")},
 		{OP_PB_PLAY, _("Play")},
 		{OP_PB_MARK_FINISHED, _("Mark as Finished")},
-		{OP_HELP, _("Help")},
-		{OP_NIL, nullptr}
+		{OP_HELP, _("Help")}
 	};
 
-	std::string keymap_hint = prepare_keymaphint(hints);
+	const auto keymap_hint = keys.prepare_keymap_hint(hints, "podboat");
 	dllist_form.set("help", keymap_hint);
 }
 
