@@ -1,5 +1,6 @@
 #include "poddlthread.h"
 
+#include <cstring>
 #include <cinttypes>
 #include <curl/curl.h>
 #include <iostream>
@@ -9,6 +10,7 @@
 #include <sys/utsname.h>
 #include <unistd.h>
 
+#include "curlhandle.h"
 #include "config.h"
 #include "logger.h"
 #include "utils.h"
@@ -20,10 +22,10 @@ namespace podboat {
 static size_t my_write_data(void* buffer, size_t size, size_t nmemb,
 	void* userp);
 static int progress_callback(void* clientp,
-	double dltotal,
-	double dlnow,
-	double ultotal,
-	double ulnow);
+	curl_off_t dltotal,
+	curl_off_t dlnow,
+	curl_off_t ultotal,
+	curl_off_t ulnow);
 
 PodDlThread::PodDlThread(Download* dl_, newsboat::ConfigContainer* c)
 	: dl(dl_)
@@ -48,25 +50,25 @@ void PodDlThread::run()
 	tv1 = std::chrono::steady_clock::now();
 	++bytecount;
 
-	CURL* easyhandle = curl_easy_init();
-	utils::set_common_curl_options(easyhandle, cfg);
+	CurlHandle handle;
+	utils::set_common_curl_options(handle, cfg);
 
-	curl_easy_setopt(easyhandle, CURLOPT_URL, dl->url().c_str());
-	curl_easy_setopt(easyhandle, CURLOPT_TIMEOUT, 0);
+	curl_easy_setopt(handle.ptr(), CURLOPT_URL, dl->url().c_str());
+	curl_easy_setopt(handle.ptr(), CURLOPT_TIMEOUT, 0);
 	// set up write functions:
-	curl_easy_setopt(easyhandle, CURLOPT_WRITEFUNCTION, my_write_data);
-	curl_easy_setopt(easyhandle, CURLOPT_WRITEDATA, this);
+	curl_easy_setopt(handle.ptr(), CURLOPT_WRITEFUNCTION, my_write_data);
+	curl_easy_setopt(handle.ptr(), CURLOPT_WRITEDATA, this);
 
 	// set up progress notification:
-	curl_easy_setopt(easyhandle, CURLOPT_NOPROGRESS, 0);
+	curl_easy_setopt(handle.ptr(), CURLOPT_NOPROGRESS, 0);
 	curl_easy_setopt(
-		easyhandle, CURLOPT_PROGRESSFUNCTION, progress_callback);
-	curl_easy_setopt(easyhandle, CURLOPT_PROGRESSDATA, this);
+		handle.ptr(), CURLOPT_XFERINFOFUNCTION, progress_callback);
+	curl_easy_setopt(handle.ptr(), CURLOPT_XFERINFODATA, this);
 
 	// set up max download speed
 	int max_dl_speed = cfg->get_configvalue_as_int("max-download-speed");
 	if (max_dl_speed > 0) {
-		curl_easy_setopt(easyhandle,
+		curl_easy_setopt(handle.ptr(),
 			CURLOPT_MAX_RECV_SPEED_LARGE,
 			(curl_off_t)(max_dl_speed * 1024));
 	}
@@ -97,7 +99,7 @@ void PodDlThread::run()
 			// it's 64 bits. Thus, this cast is either a no-op, or an up-cast
 			// which are always safe.
 			static_cast<int64_t>(sb.st_size));
-		curl_easy_setopt(easyhandle, CURLOPT_RESUME_FROM, sb.st_size);
+		curl_easy_setopt(handle.ptr(), CURLOPT_RESUME_FROM, sb.st_size);
 		dl->set_offset(sb.st_size);
 		f->open(filename, std::fstream::out | std::fstream::app);
 		resumed_download = true;
@@ -106,7 +108,7 @@ void PodDlThread::run()
 	if (f->is_open()) {
 		dl->set_status(DlStatus::DOWNLOADING);
 
-		CURLcode success = curl_easy_perform(easyhandle);
+		CURLcode success = curl_easy_perform(handle.ptr());
 
 		f->close();
 
@@ -119,23 +121,24 @@ void PodDlThread::run()
 			LOG(Level::DEBUG,
 				"PodDlThread::run: download complete, deleting "
 				"temporary suffix");
-			rename(filename.c_str(), dl->filename().c_str());
-			dl->set_status(DlStatus::READY);
+			if (rename(filename.c_str(), dl->filename().c_str()) == 0) {
+				dl->set_status(DlStatus::READY);
+			} else {
+				dl->set_status(DlStatus::RENAME_FAILED, strerror(errno));
+			}
 		} else if (dl->status() != DlStatus::CANCELLED) {
 			// attempt complete re-download
 			if (resumed_download) {
 				::unlink(filename.c_str());
 				this->run();
 			} else {
-				dl->set_status(DlStatus::FAILED);
+				dl->set_status(DlStatus::FAILED, curl_easy_strerror(success));
 				::unlink(filename.c_str());
 			}
 		}
 	} else {
-		dl->set_status(DlStatus::FAILED);
+		dl->set_status(DlStatus::FAILED, strerror(errno));
 	}
-
-	curl_easy_cleanup(easyhandle);
 }
 
 static size_t my_write_data(void* buffer, size_t size, size_t nmemb,
@@ -146,10 +149,10 @@ static size_t my_write_data(void* buffer, size_t size, size_t nmemb,
 }
 
 static int progress_callback(void* clientp,
-	double dltotal,
-	double dlnow,
-	double /* ultotal */,
-	double /*ulnow*/)
+	curl_off_t dltotal,
+	curl_off_t dlnow,
+	curl_off_t /* ultotal */,
+	curl_off_t /*ulnow*/)
 {
 	PodDlThread* thread = static_cast<PodDlThread*>(clientp);
 	return thread->progress(dlnow, dltotal);
