@@ -9,9 +9,8 @@
 #include "controller.h"
 #include "curlhandle.h"
 #include "dbexception.h"
-#include "downloadthread.h"
 #include "fmtstrformatter.h"
-#include "reloadrangethread.h"
+#include "matcherexception.h"
 #include "reloadthread.h"
 #include "rss/exception.h"
 #include "rssfeed.h"
@@ -38,7 +37,19 @@ void Reloader::spawn_reloadthread()
 void Reloader::start_reload_all_thread(const std::vector<int>& indexes)
 {
 	LOG(Level::INFO, "starting reload all thread");
-	std::thread t(DownloadThread(*this, indexes));
+	std::thread t([=]() {
+		LOG(Level::DEBUG,
+			"Reloader::start_reload_all_thread: inside thread, reloading all "
+			"feeds...");
+		if (trylock_reload_mutex()) {
+			if (indexes.empty()) {
+				reload_all();
+			} else {
+				reload_indexes(indexes);
+			}
+			unlock_reload_mutex();
+		}
+	});
 	t.detach();
 }
 
@@ -53,11 +64,19 @@ bool Reloader::trylock_reload_mutex()
 }
 
 void Reloader::reload(unsigned int pos,
-	unsigned int max,
-	bool unattended,
-	CurlHandle* easyhandle)
+	bool show_progress,
+	bool unattended)
 {
-	LOG(Level::DEBUG, "Reloader::reload: pos = %u max = %u", pos, max);
+	CurlHandle handle;
+	reload(pos, handle, show_progress, unattended);
+}
+
+void Reloader::reload(unsigned int pos,
+	CurlHandle& easyhandle,
+	bool show_progress,
+	bool unattended)
+{
+	LOG(Level::DEBUG, "Reloader::reload: pos = %u", pos);
 	std::shared_ptr<RssFeed> oldfeed = ctrl->get_feedcontainer()->get_feed(pos);
 	if (oldfeed) {
 		// Query feed reloading should be handled by the calling functions
@@ -68,11 +87,15 @@ void Reloader::reload(unsigned int pos,
 		}
 
 		std::string errmsg;
+		std::shared_ptr<AutoDiscardMessage> message_lifetime;
 		if (!unattended) {
-			ctrl->get_view()->set_status(
-				strprintf::fmt(_("%sLoading %s..."),
-					prepare_message(pos + 1, max),
-					utils::censor_url(oldfeed->rssurl())));
+			const std::string progress = show_progress ?
+				strprintf::fmt("(%u/%u) ", ++reload_progress, reload_progress_max) :
+				"";
+			message_lifetime = ctrl->get_view()->get_statusline().show_message_until_finished(
+					strprintf::fmt(_("%sLoading %s..."),
+						progress,
+						utils::censor_url(oldfeed->rssurl())));
 		}
 
 		const bool ignore_dl =
@@ -83,9 +106,11 @@ void Reloader::reload(unsigned int pos,
 			cfg,
 			ignore_dl ? ctrl->get_ignores() : nullptr,
 			ctrl->get_api());
-		parser.set_easyhandle(easyhandle);
+		parser.set_easyhandle(&easyhandle);
 		LOG(Level::DEBUG, "Reloader::reload: created parser");
 		try {
+			const auto inner_message_lifetime = message_lifetime;
+			message_lifetime.reset();
 			oldfeed->set_status(DlStatus::DURING_DOWNLOAD);
 			std::shared_ptr<RssFeed> newfeed = parser.parse();
 			if (newfeed != nullptr) {
@@ -97,7 +122,6 @@ void Reloader::reload(unsigned int pos,
 				}
 			}
 			oldfeed->set_status(DlStatus::SUCCESS);
-			ctrl->get_view()->set_status("");
 		} catch (const DbException& e) {
 			errmsg = strprintf::fmt(
 					_("Error while retrieving %s: %s"),
@@ -116,20 +140,51 @@ void Reloader::reload(unsigned int pos,
 		}
 		if (!errmsg.empty()) {
 			oldfeed->set_status(DlStatus::DL_ERROR);
-			ctrl->get_view()->set_status(errmsg);
+			ctrl->get_view()->get_statusline().show_error(errmsg);
 			LOG(Level::USERERROR, "%s", errmsg);
 		}
 	} else {
-		ctrl->get_view()->show_error(_("Error: invalid feed!"));
+		ctrl->get_view()->get_statusline().show_error(_("Error: invalid feed!"));
 	}
 }
 
-std::string Reloader::prepare_message(unsigned int pos, unsigned int max)
+void Reloader::partition_reload_to_threads(
+	std::function<void(unsigned int start, unsigned int end)> handle_range,
+	unsigned int num_feeds)
 {
-	if (max > 0) {
-		return strprintf::fmt("(%u/%u) ", pos, max);
+	int num_threads = cfg->get_configvalue_as_int("reload-threads");
+	// TODO: change to std::clamp in C++17
+	const int min_threads = 1;
+	const int max_threads = num_feeds;
+	num_threads = std::max(min_threads, std::min(num_threads, max_threads));
+
+	LOG(Level::DEBUG, "Reloader::partition_reload_to_threads: starting with reload...");
+	reload_progress = 0;
+	reload_progress_max = num_feeds;
+	if (num_threads == 1) {
+		handle_range(0, num_feeds - 1);
+	} else {
+		std::vector<std::pair<unsigned int, unsigned int>> partitions =
+				utils::partition_indexes(0, num_feeds - 1, num_threads);
+		std::vector<std::thread> threads;
+		LOG(Level::DEBUG,
+			"Reloader::partition_reload_to_threads: starting reload threads...");
+		for (int i = 0; i < num_threads - 1; i++) {
+			auto range = partitions[i];
+			threads.emplace_back([=]() {
+				handle_range(range.first, range.second);
+			});
+		}
+		LOG(Level::DEBUG,
+			"Reloader::partition_reload_to_threads: starting my own reload...");
+		handle_range(partitions[num_threads - 1].first,
+			partitions[num_threads - 1].second);
+		LOG(Level::DEBUG,
+			"Reloader::partition_reload_to_threads: joining other threads...");
+		for (size_t i = 0; i < threads.size(); i++) {
+			threads[i].join();
+		}
 	}
-	return "";
 }
 
 void Reloader::reload_all(bool unattended)
@@ -140,51 +195,24 @@ void Reloader::reload_all(bool unattended)
 		ctrl->get_feedcontainer()->unread_feed_count();
 	const auto unread_articles =
 		ctrl->get_feedcontainer()->unread_item_count();
-	int num_threads = cfg->get_configvalue_as_int("reload-threads");
 
 	ctrl->get_feedcontainer()->reset_feeds_status();
 	const auto num_feeds = ctrl->get_feedcontainer()->feeds_size();
 
-	// TODO: change to std::clamp in C++17
-	const int min_threads = 1;
-	const int max_threads = num_feeds;
-	num_threads = std::max(min_threads, std::min(num_threads, max_threads));
-
-	LOG(Level::DEBUG, "Reloader::reload_all: starting with reload all...");
-	if (num_threads == 1) {
-		reload_range(0, num_feeds - 1, num_feeds, unattended);
-	} else {
-		std::vector<std::pair<unsigned int, unsigned int>> partitions =
-				utils::partition_indexes(0, num_feeds - 1, num_threads);
-		std::vector<std::thread> threads;
-		LOG(Level::DEBUG,
-			"Reloader::reload_all: starting reload threads...");
-		for (int i = 0; i < num_threads - 1; i++) {
-			threads.push_back(std::thread(ReloadRangeThread(*this,
-						partitions[i].first,
-						partitions[i].second,
-						num_feeds,
-						unattended)));
-		}
-		LOG(Level::DEBUG,
-			"Reloader::reload_all: starting my own reload...");
-		reload_range(partitions[num_threads - 1].first,
-			partitions[num_threads - 1].second,
-			num_feeds,
-			unattended);
-		LOG(Level::DEBUG,
-			"Reloader::reload_all: joining other threads...");
-		for (size_t i = 0; i < threads.size(); i++) {
-			threads[i].join();
-		}
-	}
+	partition_reload_to_threads([=](unsigned int start, unsigned int end) {
+		reload_range(start, end, unattended);
+	}, num_feeds);
 
 	// refresh query feeds (update and sort)
 	LOG(Level::DEBUG, "Reloader::reload_all: refresh query feeds");
 	for (const auto& feed : ctrl->get_feedcontainer()->get_all_feeds()) {
 		if (feed->is_query_feed()) {
-			ctrl->get_view()->prepare_query_feed(feed);
-			feed->set_status(DlStatus::SUCCESS);
+			try {
+				ctrl->get_view()->prepare_query_feed(feed);
+				feed->set_status(DlStatus::SUCCESS);
+			} catch (const MatcherException& /* e */) {
+				feed->set_status(DlStatus::DL_ERROR);
+			}
 		}
 	}
 
@@ -202,22 +230,18 @@ void Reloader::reload_indexes(const std::vector<int>& indexes, bool unattended)
 		ctrl->get_feedcontainer()->unread_feed_count();
 	const auto unread_articles =
 		ctrl->get_feedcontainer()->unread_item_count();
-	const auto size = ctrl->get_feedcontainer()->feeds_size();
 
-	for (const auto& idx : indexes) {
-		reload(idx, size, unattended);
-	}
+	partition_reload_to_threads([&](unsigned int start, unsigned int end) {
+		for (auto i = start; i <= end; ++i) {
+			reload(indexes[i], true, unattended);
+		}
+	}, indexes.size());
 
 	notify_reload_finished(unread_feeds, unread_articles);
-
-	if (!unattended) {
-		ctrl->get_view()->set_status("");
-	}
 }
 
 void Reloader::reload_range(unsigned int start,
 	unsigned int end,
-	unsigned int size,
 	bool unattended)
 {
 	std::vector<unsigned int> v;
@@ -250,7 +274,7 @@ void Reloader::reload_range(unsigned int start,
 		LOG(Level::DEBUG,
 			"Reloader::reload_range: reloading feed #%u",
 			i);
-		reload(i, size, unattended, &easyhandle);
+		reload(i, easyhandle, true, unattended);
 	}
 }
 
