@@ -1,10 +1,12 @@
 #include "selectformaction.h"
 
+#include <algorithm>
 #include <cassert>
 #include <sstream>
 #include <string>
 
 #include "config.h"
+#include "controller.h"
 #include "fmtstrformatter.h"
 #include "listformatter.h"
 #include "strprintf.h"
@@ -25,7 +27,9 @@ SelectFormAction::SelectFormAction(View* vv,
 	ConfigContainer* cfg)
 	: FormAction(vv, formstr, cfg)
 	, quit(false)
+	, is_first_draw(true)
 	, type(SelectionType::TAG)
+	, value("")
 	, tags_list("taglist", FormAction::f, cfg->get_configvalue_as_int("scrolloff"))
 {
 }
@@ -54,25 +58,11 @@ bool SelectFormAction::process_operation(Operation op,
 	bool hardquit = false;
 	switch (op) {
 	case OP_PREV:
-	case OP_SK_UP:
 		tags_list.move_up(cfg->get_configvalue_as_bool("wrap-scroll"));
 		break;
 	case OP_NEXT:
-	case OP_SK_DOWN:
 		tags_list.move_down(cfg->get_configvalue_as_bool("wrap-scroll"));
 		break;
-	case OP_SK_HOME:
-		tags_list.move_to_first();
-		break;
-	case OP_SK_END:
-		tags_list.move_to_last();
-		break;
-	case OP_SK_PGUP:
-		tags_list.move_page_up(cfg->get_configvalue_as_bool("wrap-scroll"));
-		break;
-	case OP_SK_PGDOWN:
-		tags_list.move_page_down(cfg->get_configvalue_as_bool("wrap-scroll"));
-		break;
 	case OP_CMD_START_1:
 		FormAction::start_cmdline("1");
 		break;
@@ -132,6 +122,9 @@ bool SelectFormAction::process_operation(Operation op,
 	}
 	break;
 	default:
+		if (handle_list_operations(tags_list, op)) {
+			break;
+		}
 		break;
 	}
 
@@ -144,10 +137,11 @@ bool SelectFormAction::process_operation(Operation op,
 	}
 	return true;
 }
-
 void SelectFormAction::prepare()
 {
 	if (do_redraw) {
+		update_heading();
+
 		ListFormatter listfmt;
 		unsigned int i = 0;
 		const auto selecttag_format = cfg->get_configvalue("selecttag-format");
@@ -178,41 +172,27 @@ void SelectFormAction::prepare()
 		}
 		tags_list.stfl_replace_lines(listfmt);
 
+		if (is_first_draw && type == SelectionType::TAG && !value.empty()) {
+			const auto it = std::find(tags.begin(), tags.end(), value);
+			if (it != tags.end()) {
+				const auto index = std::distance(tags.begin(), it);
+				tags_list.set_position(index);
+			}
+		}
+
+		is_first_draw = false;
 		do_redraw = false;
 	}
 }
 
 void SelectFormAction::init()
 {
-	std::string title;
 	do_redraw = true;
 	quit = false;
-	value = "";
 
-	f.run(-3); // compute all widget dimensions
-
-	const unsigned int width = tags_list.get_width();
+	recalculate_widget_dimensions();
 
 	set_keymap_hints();
-
-	FmtStrFormatter fmt;
-	fmt.register_fmt('N', PROGRAM_NAME);
-	fmt.register_fmt('V', utils::program_version());
-
-	switch (type) {
-	case SelectionType::TAG:
-		title = fmt.do_format(
-				cfg->get_configvalue("selecttag-title-format"), width);
-		break;
-	case SelectionType::FILTER:
-		title = fmt.do_format(
-				cfg->get_configvalue("selectfilter-title-format"),
-				width);
-		break;
-	default:
-		assert(0); // should never happen
-	}
-	f.set("head", title);
 }
 
 std::string SelectFormAction::format_line(const std::string& selecttag_format,
@@ -241,15 +221,38 @@ std::string SelectFormAction::format_line(const std::string& selecttag_format,
 	return formattedLine;
 }
 
-KeyMapHintEntry* SelectFormAction::get_keymap_hint()
+void SelectFormAction::update_heading()
 {
-	static KeyMapHintEntry hints_tag[] = {{OP_QUIT, _("Cancel")},
-		{OP_OPEN, _("Select Tag")},
-		{OP_NIL, nullptr}
+	std::string title;
+	const unsigned int width = tags_list.get_width();
+
+	FmtStrFormatter fmt;
+	fmt.register_fmt('N', PROGRAM_NAME);
+	fmt.register_fmt('V', utils::program_version());
+
+	switch (type) {
+	case SelectionType::TAG:
+		title = fmt.do_format(
+				cfg->get_configvalue("selecttag-title-format"), width);
+		break;
+	case SelectionType::FILTER:
+		title = fmt.do_format(
+				cfg->get_configvalue("selectfilter-title-format"),
+				width);
+		break;
+	default:
+		assert(0); // should never happen
+	}
+	set_value("head", title);
+}
+
+const std::vector<KeyMapHintEntry>& SelectFormAction::get_keymap_hint() const
+{
+	static const std::vector<KeyMapHintEntry> hints_tag = {{OP_QUIT, _("Cancel")},
+		{OP_OPEN, _("Select Tag")}
 	};
-	static KeyMapHintEntry hints_filter[] = {{OP_QUIT, _("Cancel")},
-		{OP_OPEN, _("Select Filter")},
-		{OP_NIL, nullptr}
+	static const std::vector<KeyMapHintEntry> hints_filter = {{OP_QUIT, _("Cancel")},
+		{OP_OPEN, _("Select Filter")}
 	};
 	switch (type) {
 	case SelectionType::TAG:
@@ -257,7 +260,12 @@ KeyMapHintEntry* SelectFormAction::get_keymap_hint()
 	case SelectionType::FILTER:
 		return hints_filter;
 	}
-	return nullptr;
+
+	// The above `switch` handles all the cases, but GCC doesn't understand
+	// that and wants a `return` here. These lines will never actually be
+	// reached, so it's fine to return an empty vector here.
+	static const std::vector<KeyMapHintEntry> no_hints;
+	return no_hints;
 }
 
 std::string SelectFormAction::title()
