@@ -3,6 +3,8 @@
 #include <algorithm>
 #include <cinttypes>
 #include <cstdlib>
+#include <cstring>
+#include <cwchar>
 #include <iostream>
 #include <istream>
 #include <sstream>
@@ -11,7 +13,6 @@
 #include "config.h"
 #include "logger.h"
 #include "utils.h"
-#include "xmlexception.h"
 
 namespace newsboat {
 
@@ -21,21 +22,15 @@ namespace newsboat {
  * remotely looks like XML. We use this parser for the HTML renderer.
  */
 
-TagSoupPullParser::TagSoupPullParser()
-	: inputstream(0)
+TagSoupPullParser::TagSoupPullParser(std::istream& is)
+	: inputstream(is)
 	, current_event(Event::START_DOCUMENT)
 {
 }
 
 TagSoupPullParser::~TagSoupPullParser() {}
 
-void TagSoupPullParser::set_input(std::istream& is)
-{
-	inputstream = &is;
-	current_event = Event::START_DOCUMENT;
-}
-
-std::string TagSoupPullParser::get_attribute_value(
+nonstd::optional<std::string> TagSoupPullParser::get_attribute_value(
 	const std::string& name) const
 {
 	for (const auto& attr : attributes) {
@@ -43,7 +38,7 @@ std::string TagSoupPullParser::get_attribute_value(
 			return attr.second;
 		}
 	}
-	throw std::invalid_argument(_("attribute not found"));
+	return nonstd::nullopt;
 }
 
 TagSoupPullParser::Event TagSoupPullParser::get_event_type() const
@@ -66,7 +61,7 @@ TagSoupPullParser::Event TagSoupPullParser::next()
 	attributes.clear();
 	text = "";
 
-	if (inputstream->eof()) {
+	if (inputstream.eof()) {
 		current_event = Event::END_DOCUMENT;
 	}
 
@@ -75,8 +70,8 @@ TagSoupPullParser::Event TagSoupPullParser::next()
 	case Event::START_TAG:
 	case Event::END_TAG: {
 		char c = 0;
-		inputstream->read(&c, 1);
-		if (inputstream->eof()) {
+		inputstream.read(&c, 1);
+		if (inputstream.eof()) {
 			current_event = Event::END_DOCUMENT;
 		} else if (c == '<') {
 			handle_tag();
@@ -120,14 +115,12 @@ void TagSoupPullParser::add_attribute(std::string s)
 	attributes.push_back(Attribute(attribname, attribvalue));
 }
 
-std::string TagSoupPullParser::read_tag()
+nonstd::optional<std::string> TagSoupPullParser::read_tag()
 {
 	std::string s;
-	getline(*inputstream, s, '>');
-	if (inputstream->eof()) {
-		// TODO: test whether this works reliably
-		throw XmlException(
-			_("EOF found while reading XML tag"));
+	getline(inputstream, s, '>');
+	if (inputstream.eof()) {
+		return nonstd::nullopt;
 	}
 	return s;
 }
@@ -159,19 +152,29 @@ std::string TagSoupPullParser::decode_attribute(const std::string& s)
 std::string TagSoupPullParser::decode_entities(const std::string& s)
 {
 	std::string result;
-	std::istringstream sbuf(s);
-	std::string tmp;
-	getline(sbuf, tmp, '&');
-	while (!sbuf.eof()) {
-		result.append(tmp);
-		getline(sbuf, tmp, ';');
-		if (sbuf.eof()) {
+	std::string encoded_entity;
+	size_t offset = 0;
+	size_t ampersand_offset;
+	size_t semicolon_offset;
+	while ((ampersand_offset = s.find('&', offset)) != std::string::npos) {
+		semicolon_offset = s.find(';', ampersand_offset + 1);
+		if (semicolon_offset == std::string::npos) {
 			break;
 		}
-		result.append(decode_entity(tmp));
-		getline(sbuf, tmp, '&');
+		result.append(s, offset, ampersand_offset - offset);
+		encoded_entity = decode_entity(s.substr(ampersand_offset + 1,
+					semicolon_offset - ampersand_offset - 1));
+		if (!encoded_entity.empty()) {
+			result.append(encoded_entity);
+			offset = semicolon_offset + 1;
+		} else {
+			result.push_back('&');
+			offset = ampersand_offset + 1;
+		}
+	}
+	if (s.size() > offset) {
+		result.append(s, offset, std::string::npos);
 	}
-	result.append(tmp);
 	return result;
 }
 
@@ -445,10 +448,14 @@ std::string TagSoupPullParser::decode_entity(std::string s)
 	LOG(Level::DEBUG,
 		"TagSoupPullParser::decode_entity: decoding '%s'...",
 		s);
+
+	mbstate_t mb_state;
+	::memset(&mb_state, 0, sizeof(mb_state));
+
 	if (s.length() > 1 && s[0] == '#') {
 		std::string result;
 		unsigned int wc;
-		char mbc[MB_CUR_MAX];
+		char mbc[MB_LEN_MAX + 1];
 		mbc[0] = '\0';
 		if (s[1] == 'x') {
 			s.erase(0, 2);
@@ -458,41 +465,96 @@ std::string TagSoupPullParser::decode_entity(std::string s)
 			s.erase(0, 1);
 			wc = utils::to_u(s);
 		}
-		int pos;
-		// convert some common but unknown numeric entities
+		// convert some windows entities according to the spec
+		// https://html.spec.whatwg.org/multipage/parsing.html#numeric-character-reference-end-state
 		switch (wc) {
-		case 133:
-			wc = 8230;
-			break; // &hellip;
-		case 134:
-			wc = 8224;
-			break; // &dagger;
-		case 135:
-			wc = 8225;
-			break; // &Dagger; (double dagger)
-		case 150:
-			wc = 8211;
-			break; // &ndash;
-		case 151:
-			wc = 8212;
-			break; // &mdash;
-		case 152:
-			wc = 732;
-			break; // &tilde;
-		case 153:
-			wc = 8482;
-			break; // &trade;
-		case 156:
-			wc = 339;
-			break; // &oelig;
+		case 0x80:
+			wc = 0x20AC;
+			break; // EURO SIGN (€)
+		case 0x82:
+			wc = 0x201A;
+			break; // SINGLE LOW-9 QUOTATION MARK (‚)
+		case 0x83:
+			wc = 0x0192;
+			break; // LATIN SMALL LETTER F WITH HOOK (ƒ)
+		case 0x84:
+			wc = 0x201E;
+			break; // DOUBLE LOW-9 QUOTATION MARK („)
+		case 0x85:
+			wc = 0x2026;
+			break; // HORIZONTAL ELLIPSIS (…)
+		case 0x86:
+			wc = 0x2020;
+			break; // DAGGER (†)
+		case 0x87:
+			wc = 0x2021;
+			break; // DOUBLE DAGGER (‡)
+		case 0x88:
+			wc = 0x02C6;
+			break; // MODIFIER LETTER CIRCUMFLEX ACCENT (ˆ)
+		case 0x89:
+			wc = 0x2030;
+			break; // PER MILLE SIGN (‰)
+		case 0x8A:
+			wc = 0x0160;
+			break; // LATIN CAPITAL LETTER S WITH CARON (Š)
+		case 0x8B:
+			wc = 0x2039;
+			break; // SINGLE LEFT-POINTING ANGLE QUOTATION MARK (‹)
+		case 0x8C:
+			wc = 0x0152;
+			break; // LATIN CAPITAL LIGATURE OE (Œ)
+		case 0x8E:
+			wc = 0x017D;
+			break; // LATIN CAPITAL LETTER Z WITH CARON (Ž)
+		case 0x91:
+			wc = 0x2018;
+			break; // LEFT SINGLE QUOTATION MARK (‘)
+		case 0x92:
+			wc = 0x2019;
+			break; // RIGHT SINGLE QUOTATION MARK (’)
+		case 0x93:
+			wc = 0x201C;
+			break; // LEFT DOUBLE QUOTATION MARK (“)
+		case 0x94:
+			wc = 0x201D;
+			break; // RIGHT DOUBLE QUOTATION MARK (”)
+		case 0x95:
+			wc = 0x2022;
+			break; // BULLET (•)
+		case 0x96:
+			wc = 0x2013;
+			break; // EN DASH (–)
+		case 0x97:
+			wc = 0x2014;
+			break; // EM DASH (—)
+		case 0x98:
+			wc = 0x02DC;
+			break; // SMALL TILDE (˜)
+		case 0x99:
+			wc = 0x2122;
+			break; // TRADE MARK SIGN (™)
+		case 0x9A:
+			wc = 0x0161;
+			break; // LATIN SMALL LETTER S WITH CARON (š)
+		case 0x9B:
+			wc = 0x203A;
+			break; // SINGLE RIGHT-POINTING ANGLE QUOTATION MARK (›)
+		case 0x9C:
+			wc = 0x0153;
+			break; // LATIN SMALL LIGATURE OE (œ)
+		case 0x9E:
+			wc = 0x017E;
+			break; // LATIN SMALL LETTER Z WITH CARON (ž)
+		case 0x9F:
+			wc = 0x0178;
+			break; // LATIN CAPITAL LETTER Y WITH DIAERESIS (Ÿ)
 		}
 
-		// std::cerr << "value: " << wc << " " <<
-		// static_cast<wchar_t>(wc) << " pos: " << pos << std::endl;
-		pos = wctomb(mbc, static_cast<wchar_t>(wc));
+		const int pos = wcrtomb(mbc, static_cast<wchar_t>(wc), &mb_state);
 		if (pos > 0) {
+			result.append(mbc, pos);
 			mbc[pos] = '\0';
-			result.append(mbc);
 		}
 		LOG(Level::DEBUG,
 			"TagSoupPullParser::decode_entity: wc = %u pos = %d "
@@ -505,8 +567,8 @@ std::string TagSoupPullParser::decode_entity(std::string s)
 	} else {
 		for (unsigned int i = 0; entity_table[i].entity; ++i) {
 			if (s == entity_table[i].entity) {
-				char mbc[MB_CUR_MAX];
-				int pos = wctomb(mbc, entity_table[i].value);
+				char mbc[MB_LEN_MAX];
+				const int pos = wcrtomb(mbc, entity_table[i].value, &mb_state);
 				if (pos == -1) {
 					return std::string();
 				} else {
@@ -594,22 +656,20 @@ void TagSoupPullParser::parse_tag(const std::string& tagstr)
 
 void TagSoupPullParser::handle_tag()
 {
-	std::string s;
-	try {
-		s = read_tag();
-	} catch (const XmlException&) {
+	auto s = read_tag();
+	if (s.has_value()) {
+		parse_tag(s.value());
+		current_event = determine_tag_type();
+	} else {
 		current_event = Event::END_DOCUMENT;
-		return;
 	}
-	parse_tag(s);
-	current_event = determine_tag_type();
 }
 
 void TagSoupPullParser::handle_text(char c)
 {
 	text.push_back(c);
 	std::string tmp;
-	getline(*inputstream, tmp, '<');
+	getline(inputstream, tmp, '<');
 	text.append(tmp);
 	text = decode_entities(text);
 	utils::remove_soft_hyphens(text);
