@@ -1,4 +1,5 @@
 #include "utils.h"
+#include <iostream>
 
 #include <algorithm>
 #include <cinttypes>
@@ -27,9 +28,9 @@
 #include <unordered_set>
 
 #include "config.h"
+#include "curlhandle.h"
 #include "htmlrenderer.h"
 #include "logger.h"
-#include "ruststring.h"
 #include "strprintf.h"
 
 #if HAVE_GCRYPT
@@ -44,131 +45,38 @@ GCRY_THREAD_OPTION_PTHREAD_IMPL;
 #include <openssl/crypto.h>
 #endif
 
-#include "rs_utils.h"
-
 using HTTPMethod = newsboat::utils::HTTPMethod;
 
 namespace newsboat {
 
-namespace utils {
-void append_escapes(std::string& str, char c)
-{
-	switch (c) {
-	case 'n':
-		str.append("\n");
-		break;
-	case 'r':
-		str.append("\r");
-		break;
-	case 't':
-		str.append("\t");
-		break;
-	case '"':
-		str.append("\"");
-		break;
-	// escaped backticks are passed through, still escaped. We un-escape
-	// them in ConfigParser::evaluate_backticks
-	case '`':
-		str.append("\\`");
-		break;
-	case '\\':
-		str.append("\\");
-		break;
-	default:
-		str.push_back(c);
-		break;
-	}
-}
-}
-
 std::string utils::strip_comments(const std::string& line)
 {
-	return RustString(rs_strip_comments(line.c_str()));
+	return std::string(utils::bridged::strip_comments(line));
 }
 
 std::vector<std::string> utils::tokenize_quoted(const std::string& str,
 	std::string delimiters)
 {
-	/*
-	 * This function tokenizes strings, obeying quotes and throwing away
-	 * comments that start with a '#'.
-	 *
-	 * e.g. line: foo bar "foo bar" "a test"
-	 * is parsed to 4 elements:
-	 * 	[0]: foo
-	 * 	[1]: bar
-	 * 	[2]: foo bar
-	 * 	[3]: a test
-	 *
-	 * e.g. line: yes great "x\ny" # comment
-	 * is parsed to 3 elements:
-	 * 	[0]: yes
-	 * 	[1]: great
-	 * 	[2]: x
-	 * 	y
-	 *
-	 * 	\", \r, \n, \t and \v are replaced with the literals that you
-	 * know from C/C++ strings.
-	 *
-	 */
-	std::vector<std::string> tokens;
-	std::string remaining = str;
-	while (!remaining.empty()) {
-		auto token = extract_token_quoted(remaining, delimiters);
-		if (token.has_value()) {
-			tokens.push_back(token.value());
-		}
-	}
+	const auto tokens = utils::bridged::tokenize_quoted(str, delimiters);
 
-	return tokens;
+	std::vector<std::string> result;
+	for (const auto& token : tokens) {
+		result.push_back(std::string(token));
+	}
+	return result;
 }
 
 nonstd::optional<std::string> utils::extract_token_quoted(std::string& str,
 	std::string delimiters)
 {
-	auto first_non_delimiter = str.find_first_not_of(delimiters, 0);
-	if (first_non_delimiter == std::string::npos) {
-		str = "";
-		return {};
+	rust::String remaining = str;
+	rust::String token;
+	if (utils::bridged::extract_token_quoted(remaining, delimiters, token)) {
+		str = std::string(remaining);
+		return std::string(token);
 	}
-	str = str.substr(first_non_delimiter);
-
-	if (str[0] == '#') { // stop as soon as we find a comment
-		str = "";
-		return {};
-	}
-
-	std::string token;
-	if (str[0] == '"') {
-		std::string::size_type pos = 1;
-		while (pos < str.length()) {
-			if (str[pos] == '"') {
-				// We've reached the end of this quoted token.
-				++pos;
-				break;
-			} else if (str[pos] == '\\') {
-				pos += 1;
-				if (pos < str.length()) {
-					append_escapes(token, str[pos]);
-					pos += 1;
-				}
-			} else {
-				token.push_back(str[pos]);
-				++pos;
-			}
-		}
-		str = str.substr(pos);
-	} else {
-		auto end_of_token = str.find_first_of(delimiters);
-		token = str.substr(0, end_of_token);
-		if (end_of_token == std::string::npos) {
-			str = "";
-		} else {
-			str = str.substr(end_of_token);
-		}
-	}
-
-	return token;
+	str = std::string(remaining);
+	return {};
 }
 
 std::vector<std::string> utils::tokenize(const std::string& str,
@@ -214,8 +122,7 @@ std::vector<std::string> utils::tokenize_spaced(const std::string& str,
 
 std::string utils::consolidate_whitespace(const std::string& str)
 {
-
-	return RustString(rs_consolidate_whitespace(str.c_str()));
+	return std::string(utils::bridged::consolidate_whitespace(str));
 }
 
 std::vector<std::string> utils::tokenize_nl(const std::string& str,
@@ -259,155 +166,29 @@ std::vector<std::string> utils::tokenize_nl(const std::string& str,
 	return tokens;
 }
 
-std::string utils::translit(const std::string& tocode,
-	const std::string& fromcode)
+std::string utils::translit(const std::string& tocode, const std::string& fromcode)
 {
-	std::string tlit = "//TRANSLIT";
-
-	enum class TranslitState { UNKNOWN, SUPPORTED, UNSUPPORTED };
-
-	static TranslitState state = TranslitState::UNKNOWN;
-
-	// TRANSLIT is not needed when converting to unicode encodings
-	if (tocode == "utf-8" || tocode == "WCHAR_T") {
-		return tocode;
-	}
-
-	if (state == TranslitState::UNKNOWN) {
-		iconv_t cd = ::iconv_open(
-				(tocode + "//TRANSLIT").c_str(), fromcode.c_str());
-
-		if (cd == reinterpret_cast<iconv_t>(-1)) {
-			if (errno == EINVAL) {
-				iconv_t cd = ::iconv_open(
-						tocode.c_str(), fromcode.c_str());
-				if (cd != reinterpret_cast<iconv_t>(-1)) {
-					state = TranslitState::UNSUPPORTED;
-				} else {
-					fprintf(stderr,
-						"iconv_open('%s', '%s') "
-						"failed: %s",
-						tocode.c_str(),
-						fromcode.c_str(),
-						strerror(errno));
-					abort();
-				}
-			} else {
-				fprintf(stderr,
-					"iconv_open('%s//TRANSLIT', '%s') "
-					"failed: %s",
-					tocode.c_str(),
-					fromcode.c_str(),
-					strerror(errno));
-				abort();
-			}
-		} else {
-			state = TranslitState::SUPPORTED;
-		}
-
-		iconv_close(cd);
-	}
-
-	return ((state == TranslitState::SUPPORTED) ? (tocode + tlit)
-			: (tocode));
-}
-
-std::string utils::convert_text(const std::string& text,
-	const std::string& tocode,
-	const std::string& fromcode)
-{
-	std::string result;
-
-	if (strcasecmp(tocode.c_str(), fromcode.c_str()) == 0) {
-		return text;
-	}
-
-	iconv_t cd = ::iconv_open(
-			translit(tocode, fromcode).c_str(), fromcode.c_str());
-
-	if (cd == reinterpret_cast<iconv_t>(-1)) {
-		return result;
-	}
-
-	size_t inbytesleft;
-	size_t outbytesleft;
-
-	/*
-	 * of all the Unix-like systems around there, only Linux/glibc seems to
-	 * come with a SuSv3-conforming iconv implementation.
-	 */
-#if !defined(__linux__) && !defined(__GLIBC__) && !defined(__APPLE__) && \
-	!defined(__OpenBSD__) && !defined(__FreeBSD__) &&                \
-	!defined(__DragonFly__)
-	const char* inbufp;
-#else
-	char* inbufp;
-#endif
-	char outbuf[16];
-	char* outbufp = outbuf;
-
-	outbytesleft = sizeof(outbuf);
-	inbufp = const_cast<char*>(
-			text.c_str()); // evil, but spares us some trouble
-	inbytesleft = strlen(inbufp);
-
-	do {
-		char* old_outbufp = outbufp;
-		int rc = ::iconv(
-				cd, &inbufp, &inbytesleft, &outbufp, &outbytesleft);
-		if (-1 == rc) {
-			switch (errno) {
-			case E2BIG:
-				result.append(
-					old_outbufp, outbufp - old_outbufp);
-				outbufp = outbuf;
-				outbytesleft = sizeof(outbuf);
-				inbufp += strlen(inbufp) - inbytesleft;
-				inbytesleft = strlen(inbufp);
-				break;
-			case EILSEQ:
-			case EINVAL:
-				result.append(
-					old_outbufp, outbufp - old_outbufp);
-				result.append("?");
-				inbufp += strlen(inbufp) - inbytesleft + 1;
-				inbytesleft = strlen(inbufp);
-				break;
-			default:
-				break;
-			}
-		} else {
-			result.append(old_outbufp, outbufp - old_outbufp);
-		}
-	} while (inbytesleft > 0);
-
-	iconv_close(cd);
-
-	return result;
+	return std::string(utils::bridged::translit(tocode, fromcode));
 }
 
 std::string utils::utf8_to_locale(const std::string& text)
 {
-	if (text.empty()) {
-		return {};
-	}
+	const auto result = utils::bridged::utf8_to_locale(text);
+	return std::string(reinterpret_cast<const char*>(result.data()), result.size());
+}
 
-	return utils::convert_text(text, nl_langinfo(CODESET), "utf-8");
+std::string utils::locale_to_utf8(const std::string& text)
+{
+	const auto text_slice =
+		rust::Slice<const unsigned char>(
+			reinterpret_cast<const unsigned char*>(text.c_str()),
+			text.length());
+	return std::string(utils::bridged::locale_to_utf8(text_slice));
 }
 
 std::string utils::get_command_output(const std::string& cmd)
 {
-	return RustString(rs_get_command_output(cmd.c_str()));
-}
-
-void utils::extract_filter(const std::string& line,
-	std::string& filter,
-	std::string& url)
-{
-	FilterUrl filterUrl = rs_extract_filter(line.c_str());
-
-	filter = RustString(filterUrl.filter);
-	url = RustString(filterUrl.url);
+	return std::string(utils::bridged::get_command_output(cmd));
 }
 
 static size_t my_write_data(void* buffer, size_t size, size_t nmemb,
@@ -440,97 +221,185 @@ std::string utils::http_method_str(const HTTPMethod method)
 	return str;
 }
 
+std::string utils::link_type_str(LinkType type)
+{
+	// Different from HtmlRenderer::type2str because the string returned
+	// isn't localized
+	switch (type) {
+	case LinkType::HREF:
+		return "link";
+	case LinkType::IMG:
+		return "image";
+	case LinkType::EMBED:
+		return "embed_flash";
+	case LinkType::IFRAME:
+		return "iframe";
+	case LinkType::VIDEO:
+		return "video";
+	case LinkType::AUDIO:
+		return "audio";
+	default:
+		return "unknown (bug)";
+	}
+}
+
 std::string utils::retrieve_url(const std::string& url,
 	ConfigContainer* cfgcont,
 	const std::string& authinfo,
 	const std::string* body,
-	const HTTPMethod method, /* = GET */
-	CURL* cached_handle)
+	const HTTPMethod method /* = GET */)
+{
+	CurlHandle handle;
+	return retrieve_url(url, handle, cfgcont, authinfo, body, method);
+}
+
+std::string utils::retrieve_url(const std::string& url,
+	CurlHandle& easyhandle,
+	ConfigContainer* cfgcont,
+	const std::string& authinfo,
+	const std::string* body,
+	const HTTPMethod method /* = GET */)
 {
 	std::string buf;
 
-	CURL* easyhandle;
-	if (cached_handle) {
-		easyhandle = cached_handle;
-	} else {
-		easyhandle = curl_easy_init();
-	}
 	set_common_curl_options(easyhandle, cfgcont);
-	curl_easy_setopt(easyhandle, CURLOPT_URL, url.c_str());
-	curl_easy_setopt(easyhandle, CURLOPT_WRITEFUNCTION, my_write_data);
-	curl_easy_setopt(easyhandle, CURLOPT_WRITEDATA, &buf);
+	curl_easy_setopt(easyhandle.ptr(), CURLOPT_URL, url.c_str());
+	curl_easy_setopt(easyhandle.ptr(), CURLOPT_WRITEFUNCTION, my_write_data);
+	curl_easy_setopt(easyhandle.ptr(), CURLOPT_WRITEDATA, &buf);
 
 	switch (method) {
 	case HTTPMethod::GET:
 		break;
 	case HTTPMethod::POST:
-		curl_easy_setopt(easyhandle, CURLOPT_POST, 1);
+		curl_easy_setopt(easyhandle.ptr(), CURLOPT_POST, 1);
 		break;
 	case HTTPMethod::PUT:
 	case HTTPMethod::DELETE:
 		const std::string method_str = http_method_str(method);
-		curl_easy_setopt(easyhandle, CURLOPT_CUSTOMREQUEST, method_str.c_str());
+		curl_easy_setopt(easyhandle.ptr(), CURLOPT_CUSTOMREQUEST, method_str.c_str());
 		break;
 	}
 
 	if (body != nullptr) {
 		curl_easy_setopt(
-			easyhandle, CURLOPT_POSTFIELDS, body->c_str());
+			easyhandle.ptr(), CURLOPT_POSTFIELDS, body->c_str());
 	}
 
 	if (!authinfo.empty()) {
-		curl_easy_setopt(easyhandle,
-			CURLOPT_HTTPAUTH,
-			get_auth_method(
-				cfgcont->get_configvalue("http-auth-method")));
-		curl_easy_setopt(easyhandle, CURLOPT_USERPWD, authinfo.c_str());
+		const auto auth_method = cfgcont->get_configvalue("http-auth-method");
+		curl_easy_setopt(easyhandle.ptr(), CURLOPT_HTTPAUTH, get_auth_method(auth_method));
+		curl_easy_setopt(easyhandle.ptr(), CURLOPT_USERPWD, authinfo.c_str());
 	}
 
-	curl_easy_perform(easyhandle);
-	if (!cached_handle) {
-		curl_easy_cleanup(easyhandle);
-	}
+	// Error handling as per https://curl.se/libcurl/c/CURLOPT_ERRORBUFFER.html
+	char errbuf[CURL_ERROR_SIZE];
+	// Please note that we clobber CURLOPT_ERRORBUFFER here in case of cached handles
+	// Currently, this is the only place we do this, so this should be fine.
+	// There does not seem to be a way to query curlopts, so we cannot save the old errorbuf value here...
+	curl_easy_setopt(easyhandle.ptr(), CURLOPT_ERRORBUFFER, errbuf);
+	errbuf[0] = '\0';
 
-	if (body != nullptr) {
-		LOG(Level::DEBUG,
-			"utils::retrieve_url(%s %s)[%s]: %s",
-			http_method_str(method),
-			url,
-			body,
-			buf);
+	CURLcode res = curl_easy_perform(easyhandle.ptr());
+
+	std::stringstream logprefix;
+	logprefix << "utils::retrieve_url(" << http_method_str(method) << " " << url << ")"
+		<< "[" << (body != nullptr ? body->c_str() : "-") << "]";
+
+	if (res != CURLE_OK) {
+		std::string errmsg(errbuf);
+		if (errmsg.empty()) {
+			errmsg = curl_easy_strerror(res);
+		}
+		if (!errmsg.empty() && errmsg.back() == '\n') {
+			// Prettify: drop superflous newlines introduced by libcurl
+			errmsg.pop_back();
+		}
+
+		LOG(Level::ERROR, "%s: LibCURL error (%d): %s", logprefix.str(), res, errmsg);
+		buf = "";
 	} else {
-		LOG(Level::DEBUG, "utils::retrieve_url(%s)[-]: %s", url, buf);
+		LOG(Level::DEBUG, "%s: %s", logprefix.str(), buf);
 	}
 
+	// Reset ERRORBUFFER: has to be valid for the whole lifetime of the handle
+	// NULL is the default value of this property according to man (3) CURLOPT_ERRORBUFFER
+	// See the clobbering note above.
+	curl_easy_setopt(easyhandle.ptr(), CURLOPT_ERRORBUFFER, NULL);
+
 	return buf;
 }
 
-void utils::run_command(const std::string& cmd, const std::string& input)
-{
-	rs_run_command(cmd.c_str(), input.c_str());
-}
-
 std::string utils::run_program(const char* argv[], const std::string& input)
 {
-	return RustString(rs_run_program(argv, input.c_str()));
+	std::vector<rust::Str> slices;
+	for (; *argv; ++argv) {
+		slices.emplace_back(*argv);
+	}
+
+	const auto rs_argv = rust::Slice<const rust::Str>(slices.data(), slices.size());
+
+	return std::string(utils::bridged::run_program(rs_argv, input));
 }
 
 std::string utils::resolve_tilde(const std::string& str)
 {
-	return RustString(rs_resolve_tilde(str.c_str()));
+	return std::string(utils::bridged::resolve_tilde(str));
 }
 
 std::string utils::resolve_relative(const std::string& reference,
 	const std::string& fname)
 {
-	return RustString(rs_resolve_relative(reference.c_str(), fname.c_str()));
+	return std::string(utils::bridged::resolve_relative(reference, fname));
 }
 
 std::string utils::replace_all(std::string str,
 	const std::string& from,
 	const std::string& to)
 {
-	return RustString( rs_replace_all(str.c_str(), from.c_str(), to.c_str()) );
+	return std::string(utils::bridged::replace_all(str, from, to));
+}
+
+std::string utils::replace_all(const std::string& str,
+	const std::vector<std::pair<std::string, std::string>> from_to_pairs)
+{
+	std::size_t cur_index = 0;
+	std::string output;
+	while (cur_index != str.size()) {
+		std::size_t first_match = std::string::npos;
+		std::pair<std::string, std::string> first_match_pair;
+		for (const auto& p : from_to_pairs) {
+			auto match = str.find(p.first, cur_index);
+			if (match != std::string::npos && match < first_match) {
+				first_match = match;
+				first_match_pair = p;
+			}
+		}
+		if (first_match == std::string::npos) {
+			output += str.substr(cur_index);
+			break;
+		} else {
+			output += str.substr(cur_index, first_match - cur_index);
+			cur_index = first_match + first_match_pair.first.size();
+			output += first_match_pair.second;
+		}
+	}
+	return output;
+}
+
+std::string utils::preserve_quotes(const std::string& str)
+{
+	std::string escaped_string = "";
+	std::vector<std::string> string_tokenized = tokenize_spaced(str, "'");
+	for (std::string string_chunk : string_tokenized) {
+		if (string_chunk[0] == '\'') {
+			for (size_t i = 0; i < string_chunk.length(); i++) {
+				escaped_string += "\\\'";
+			}
+		} else {
+			escaped_string += "'" + string_chunk + "'";
+		}
+	}
+	return escaped_string;
 }
 
 std::wstring utils::str2wstr(const std::string& str)
@@ -554,7 +423,7 @@ std::string utils::wstr2str(const std::wstring& wstr)
 
 std::string utils::absolute_url(const std::string& url, const std::string& link)
 {
-	return RustString(rs_absolute_url(url.c_str(), link.c_str()));
+	return std::string(utils::bridged::absolute_url(url, link));
 }
 
 std::string utils::get_useragent(ConfigContainer* cfgcont)
@@ -588,17 +457,7 @@ std::string utils::get_useragent(ConfigContainer* cfgcont)
 unsigned int utils::to_u(const std::string& str,
 	const unsigned int default_value)
 {
-	return rs_to_u(str.c_str(), default_value);
-}
-
-bool utils::is_valid_color(const std::string& color)
-{
-	return rs_is_valid_color(color.c_str());
-}
-
-bool utils::is_valid_attribute(const std::string& attrib)
-{
-	return rs_is_valid_attribute(attrib.c_str());
+	return bridged::to_u(str, default_value);
 }
 
 std::vector<std::pair<unsigned int, unsigned int>> utils::partition_indexes(
@@ -620,26 +479,16 @@ std::vector<std::pair<unsigned int, unsigned int>> utils::partition_indexes(
 	return partitions;
 }
 
-size_t utils::strwidth(const std::string& str)
-{
-	return rs_strwidth(str.c_str());
-}
-
-size_t utils::strwidth_stfl(const std::string& str)
-{
-	return rs_strwidth_stfl(str.c_str());
-}
-
 std::string utils::substr_with_width(const std::string& str,
 	const size_t max_width)
 {
-	return RustString(rs_substr_with_width(str.c_str(), max_width));
+	return std::string(utils::bridged::substr_with_width(str, max_width));
 }
 
 std::string utils::substr_with_width_stfl(const std::string& str,
 	const size_t max_width)
 {
-	return RustString(rs_substr_with_width_stfl(str.c_str(), max_width));
+	return std::string(utils::bridged::substr_with_width_stfl(str, max_width));
 }
 
 std::string utils::join(const std::vector<std::string>& strings,
@@ -659,84 +508,54 @@ std::string utils::join(const std::vector<std::string>& strings,
 	return result;
 }
 
-bool utils::is_special_url(const std::string& url)
-{
-	return rs_is_special_url(url.c_str());
-}
-
-bool utils::is_http_url(const std::string& url)
-{
-	return rs_is_http_url(url.c_str());
-}
-
-bool utils::is_query_url(const std::string& url)
-{
-	return rs_is_query_url(url.c_str());
-}
-
-bool utils::is_filter_url(const std::string& url)
-{
-	return rs_is_filter_url(url.c_str());
-}
-
-bool utils::is_exec_url(const std::string& url)
-{
-	return rs_is_exec_url(url.c_str());
-}
-
 std::string utils::censor_url(const std::string& url)
 {
-	return RustString(rs_censor_url(url.c_str()));
+	return std::string(utils::bridged::censor_url(url));
 }
 
 std::string utils::quote_for_stfl(std::string str)
 {
-	return RustString(rs_quote_for_stfl(str.c_str()));
+	return std::string(utils::bridged::quote_for_stfl(str));
 }
 
 void utils::trim(std::string& str)
 {
-	str = RustString(rs_trim(str.c_str()));
+	str = std::string(utils::bridged::trim(str));
 }
 
 void utils::trim_end(std::string& str)
 {
-	str = RustString(rs_trim_end(str.c_str()));
+	str = std::string(utils::bridged::trim_end(str));
 }
 
 std::string utils::quote(const std::string& str)
 {
-	return RustString(rs_quote(str.c_str()));
-}
-
-unsigned int utils::get_random_value(unsigned int max)
-{
-	return rs_get_random_value(max);
+	return std::string(utils::bridged::quote(str));
 }
 
 std::string utils::quote_if_necessary(const std::string& str)
 {
-	return RustString(rs_quote_if_necessary(str.c_str()));
+	return std::string(utils::bridged::quote_if_necessary(str));
 }
 
-void utils::set_common_curl_options(CURL* handle, ConfigContainer* cfg)
+void utils::set_common_curl_options(CurlHandle& handle, ConfigContainer* cfg)
 {
 	if (cfg) {
 		if (cfg->get_configvalue_as_bool("use-proxy")) {
 			const std::string proxy = cfg->get_configvalue("proxy");
 			if (proxy != "")
 				curl_easy_setopt(
-					handle, CURLOPT_PROXY, proxy.c_str());
+					handle.ptr(), CURLOPT_PROXY, proxy.c_str());
 
 			const std::string proxyauth =
 				cfg->get_configvalue("proxy-auth");
 			const std::string proxyauthmethod =
 				cfg->get_configvalue("proxy-auth-method");
 			if (proxyauth != "") {
-				curl_easy_setopt(handle,
+				curl_easy_setopt(handle.ptr(),
 					CURLOPT_PROXYAUTH,
 					get_auth_method(proxyauthmethod));
-				curl_easy_setopt(handle,
+				curl_easy_setopt(handle.ptr(),
 					CURLOPT_PROXYUSERPWD,
 					proxyauth.c_str());
 			}
@@ -749,48 +568,49 @@ void utils::set_common_curl_options(CURL* handle, ConfigContainer* cfg)
 					"proxytype "
 					"= %s",
 					proxytype);
-				curl_easy_setopt(handle,
+				curl_easy_setopt(handle.ptr(),
 					CURLOPT_PROXYTYPE,
 					get_proxy_type(proxytype));
 			}
 		}
 
 		const std::string useragent = utils::get_useragent(cfg);
-		curl_easy_setopt(handle, CURLOPT_USERAGENT, useragent.c_str());
+		curl_easy_setopt(handle.ptr(), CURLOPT_USERAGENT, useragent.c_str());
 
 		const unsigned int dl_timeout =
 			cfg->get_configvalue_as_int("download-timeout");
-		curl_easy_setopt(handle, CURLOPT_TIMEOUT, dl_timeout);
+		curl_easy_setopt(handle.ptr(), CURLOPT_TIMEOUT, dl_timeout);
 
 		const std::string cookie_cache =
 			cfg->get_configvalue("cookie-cache");
 		if (cookie_cache != "") {
-			curl_easy_setopt(handle,
+			curl_easy_setopt(handle.ptr(),
 				CURLOPT_COOKIEFILE,
 				cookie_cache.c_str());
-			curl_easy_setopt(handle,
+			curl_easy_setopt(handle.ptr(),
 				CURLOPT_COOKIEJAR,
 				cookie_cache.c_str());
 		}
 
-		curl_easy_setopt(handle,
+		curl_easy_setopt(handle.ptr(),
 			CURLOPT_SSL_VERIFYHOST,
 			cfg->get_configvalue_as_bool("ssl-verifyhost") ? 2 : 0);
-		curl_easy_setopt(handle,
+		curl_easy_setopt(handle.ptr(),
 			CURLOPT_SSL_VERIFYPEER,
 			cfg->get_configvalue_as_bool("ssl-verifypeer"));
 	}
 
-	curl_easy_setopt(handle, CURLOPT_NOSIGNAL, 1);
-	curl_easy_setopt(handle, CURLOPT_ENCODING, "gzip, deflate");
+	curl_easy_setopt(handle.ptr(), CURLOPT_NOSIGNAL, 1);
+	// Accept all of curl's built-in encodings
+	curl_easy_setopt(handle.ptr(), CURLOPT_ACCEPT_ENCODING, "");
 
-	curl_easy_setopt(handle, CURLOPT_FOLLOWLOCATION, 1);
-	curl_easy_setopt(handle, CURLOPT_MAXREDIRS, 10);
-	curl_easy_setopt(handle, CURLOPT_FAILONERROR, 1);
+	curl_easy_setopt(handle.ptr(), CURLOPT_FOLLOWLOCATION, 1);
+	curl_easy_setopt(handle.ptr(), CURLOPT_MAXREDIRS, 10);
+	curl_easy_setopt(handle.ptr(), CURLOPT_FAILONERROR, 1);
 
 	const char* curl_ca_bundle = ::getenv("CURL_CA_BUNDLE");
 	if (curl_ca_bundle != nullptr) {
-		curl_easy_setopt(handle, CURLOPT_CAINFO, curl_ca_bundle);
+		curl_easy_setopt(handle.ptr(), CURLOPT_CAINFO, curl_ca_bundle);
 	}
 }
 
@@ -809,12 +629,7 @@ std::string utils::get_content(xmlNode* node)
 
 std::string utils::get_basename(const std::string& url)
 {
-	return RustString(rs_get_basename(url.c_str()));
-}
-
-unsigned long utils::get_auth_method(const std::string& type)
-{
-	return rs_get_auth_method(type.c_str());
+	return std::string(utils::bridged::get_basename(url));
 }
 
 curl_proxytype utils::get_proxy_type(const std::string& type)
@@ -847,12 +662,13 @@ curl_proxytype utils::get_proxy_type(const std::string& type)
 
 std::string utils::unescape_url(const std::string& url)
 {
-	char* ptr = rs_unescape_url(url.c_str());
-	if (ptr == nullptr) {
+	bool success = false;
+	const auto result = utils::bridged::unescape_url(url, success);
+	if (!success) {
 		LOG(Level::DEBUG, "Rust failed to unescape url: %s", url );
 		throw std::runtime_error("unescaping url failed");
 	} else {
-		return RustString(ptr);
+		return std::string(result);
 	}
 }
 
@@ -866,31 +682,36 @@ std::wstring utils::clean_nonprintable_characters(std::wstring text)
 	return text;
 }
 
-unsigned int utils::gentabs(const std::string& str)
-{
-	return rs_gentabs(str.c_str());
-}
-
 /* Like mkdir(), but creates ancestors (parent directories) if they don't
  * exist. */
 int utils::mkdir_parents(const std::string& p, mode_t mode)
 {
-	return rs_mkdir_parents(p.c_str(), static_cast<std::uint32_t>(mode));
+	return utils::bridged::mkdir_parents(p, static_cast<std::uint32_t>(mode));
 }
 
 std::string utils::make_title(const std::string& const_url)
 {
-	return RustString(rs_make_title(const_url.c_str()));
+	return std::string(utils::bridged::make_title(const_url));
 }
 
 nonstd::optional<std::uint8_t> utils::run_interactively(
 	const std::string& command,
 	const std::string& caller)
 {
-	bool success = false;
-	const auto exit_code = rs_run_interactively(command.c_str(), caller.c_str(),
-			&success);
-	if (success) {
+	std::uint8_t exit_code = 0;
+	if (bridged::run_interactively(command, caller, exit_code)) {
+		return exit_code;
+	}
+
+	return nonstd::nullopt;
+}
+
+nonstd::optional<std::uint8_t> utils::run_non_interactively(
+	const std::string& command,
+	const std::string& caller)
+{
+	std::uint8_t exit_code = 0;
+	if (bridged::run_non_interactively(command, caller, exit_code)) {
 		return exit_code;
 	}
 
@@ -899,33 +720,58 @@ nonstd::optional<std::uint8_t> utils::run_interactively(
 
 std::string utils::getcwd()
 {
-	return RustString(rs_getcwd());
+	return std::string(utils::bridged::getcwd());
 }
 
-int utils::strnaturalcmp(const std::string& a, const std::string& b)
+nonstd::expected<std::vector<std::string>, utils::ReadTextFileError> utils::read_text_file(
+	const std::string& filename)
 {
-	return rs_strnaturalcmp(a.c_str(), b.c_str());
+	rust::Vec<rust::String> c;
+	std::uint64_t error_line_number{};
+	rust::String error_reason;
+	const bool result = bridged::read_text_file(filename, c, error_line_number,
+			error_reason);
+
+	if (result) {
+		std::vector<std::string> contents;
+		for (const auto& line : c) {
+			contents.push_back(std::string(line));
+		}
+		return contents;
+	} else {
+		ReadTextFileError error;
+
+		if (error_line_number == 0) {
+			error.kind = ReadTextFileErrorKind::CantOpen;
+			error.message = strprintf::fmt(_("Failed to open file: %s"),
+					std::string(error_reason));
+		} else {
+			error.kind = ReadTextFileErrorKind::LineError;
+			error.message = strprintf::fmt(_("Failed to read line %u: %s"),
+					error_line_number, std::string(error_reason));
+		}
+
+		return nonstd::make_unexpected(error);
+	}
 }
 
 void utils::remove_soft_hyphens(std::string& text)
 {
-	/* Remove all soft-hyphens as they can behave unpredictably (see
-	 * https://github.com/akrennmair/newsbeuter/issues/259#issuecomment-259609490)
-	 * and inadvertently render as hyphens */
-	text = RustString(rs_remove_soft_hyphens(text.c_str()));
+	rust::String tmp(text);
+	utils::bridged::remove_soft_hyphens(tmp);
+	text = std::string(tmp);
 }
 
 bool utils::is_valid_podcast_type(const std::string& mimetype)
 {
-	return rs_is_valid_podcast_type(mimetype.c_str());
+	return utils::bridged::is_valid_podcast_type(mimetype);
 }
 
 nonstd::optional<LinkType> utils::podcast_mime_to_link_type(
 	const std::string& mimetype)
 {
-	bool ok = false;
-	const auto result = rs_podcast_mime_to_link_type(mimetype.c_str(), &ok);
-	if (ok) {
+	std::int64_t result = 0;
+	if (utils::bridged::podcast_mime_to_link_type(mimetype, result)) {
 		return static_cast<LinkType>(result);
 	}
 
@@ -988,17 +834,17 @@ void utils::initialize_ssl_implementation(void)
 
 std::string utils::get_default_browser()
 {
-	return RustString(rs_get_default_browser());
+	return std::string(utils::bridged::get_default_browser());
+}
+
+std::string utils::md5hash(const std::string& input)
+{
+	return std::string(utils::bridged::md5hash(input));
 }
 
 std::string utils::program_version()
 {
-	return RustString(rs_program_version());
-}
-
-unsigned int utils::newsboat_version_major()
-{
-	return rs_newsboat_version_major();
+	return std::string(utils::bridged::program_version());
 }
 
 std::string utils::mt_strf_localtime(const std::string& format, time_t t)
