@@ -99,6 +99,20 @@ TEST_CASE("Throws if command argument has invalid type", "[ConfigContainer]")
 	}
 }
 
+TEST_CASE("Throws if there are too few arguments", "[ConfigContainer]")
+{
+	ConfigContainer cfg;
+
+	CHECK_THROWS_AS(cfg.handle_action("browser", {}), ConfigHandlerException);
+}
+
+TEST_CASE("Throws if there are too many arguments", "[ConfigContainer]")
+{
+	ConfigContainer cfg;
+
+	CHECK_THROWS_AS(cfg.handle_action("browser", {"xdg-open", "%u"}), ConfigHandlerException);
+}
+
 TEST_CASE("reset_to_default changes setting to its default value",
 	"[ConfigContainer]")
 {
@@ -188,13 +202,13 @@ TEST_CASE("toggle() inverts the value of a boolean setting",
 	SECTION("\"true\" becomes \"false\"") {
 		cfg.set_configvalue(key, "true");
 		REQUIRE_NOTHROW(cfg.toggle(key));
-		REQUIRE(cfg.get_configvalue_as_bool(key) == false);
+		REQUIRE_FALSE(cfg.get_configvalue_as_bool(key));
 	}
 
 	SECTION("\"false\" becomes \"true\"") {
 		cfg.set_configvalue(key, "false");
 		REQUIRE_NOTHROW(cfg.toggle(key));
-		REQUIRE(cfg.get_configvalue_as_bool(key) == true);
+		REQUIRE(cfg.get_configvalue_as_bool(key));
 	}
 }
 
@@ -242,19 +256,17 @@ TEST_CASE(
 	std::vector<std::string> result;
 
 	SECTION("By default, simply enumerates all settings") {
-		std::unordered_set<std::string> expected{
-			"always-display-description false",
-			"download-timeout 30",
-			"ignore-mode \"download\"",
-			"newsblur-min-items 20",
-			"oldreader-password \"\"",
-			"proxy-type \"http\"",
-			"ttrss-mode \"multi\""};
-
 		REQUIRE_NOTHROW(cfg.dump_config(result));
 		{
-			INFO("Checking that all the expected values were "
-				"found");
+			INFO("Checking that all the expected values were found");
+			std::unordered_set<std::string> expected{
+				"always-display-description false",
+				"download-timeout 30",
+				"ignore-mode \"download\"",
+				"newsblur-min-items 20",
+				"oldreader-password \"\"",
+				"proxy-type \"http\"",
+				"ttrss-mode \"multi\""};
 			REQUIRE(all_values_found(expected, result));
 		}
 	}
@@ -264,15 +276,13 @@ TEST_CASE(
 		cfg.set_configvalue("download-timeout", "100");
 		cfg.set_configvalue("http-auth-method", "digest");
 
-		std::unordered_set<std::string> expected{
-			"download-timeout 100 # default: 30",
-			"http-auth-method \"digest\" # default: any",
-		};
-
 		REQUIRE_NOTHROW(cfg.dump_config(result));
 		{
-			INFO("Checking that all the expected values were "
-				"found");
+			INFO("Checking that all the expected values were found");
+			std::unordered_set<std::string> expected{
+				"download-timeout 100 # default: 30",
+				"http-auth-method \"digest\" # default: any",
+			};
 			REQUIRE(all_values_found(expected, result));
 		}
 	}
