@@ -5,9 +5,11 @@
 #include <vector>
 
 #include "3rd-party/catch.hpp"
+
+#include "configexception.h"
 #include "keymap.h"
-#include "test-helpers/envvar.h"
-#include "test-helpers/tempfile.h"
+#include "test_helpers/envvar.h"
+#include "test_helpers/tempfile.h"
 
 using namespace newsboat;
 
@@ -21,14 +23,14 @@ public:
 		history.emplace_back(action, params);
 	};
 
-	void dump_config(std::vector<std::string>&) override {};
+	void dump_config(std::vector<std::string>&) const override {};
 
 	std::vector<std::pair<std::string, std::vector<std::string>>> history;
 };
 
 } // Anonymous namespace
 
-TEST_CASE("parse_line() Handles both lines with and without quoting",
+TEST_CASE("parse_line() handles both lines with and without quoting",
 	"[ConfigParser]")
 {
 	ConfigParser cfgParser;
@@ -39,8 +41,9 @@ TEST_CASE("parse_line() Handles both lines with and without quoting",
 	const std::string location = "dummy-location";
 
 	SECTION("unknown command results in exception") {
-		REQUIRE_THROWS(cfgParser.parse_line("foo", location));
-		REQUIRE_THROWS(cfgParser.parse_line("foo arg1 arg2", location));
+		REQUIRE_THROWS_AS(cfgParser.parse_line("foo", location), ConfigException);
+		REQUIRE_THROWS_AS(cfgParser.parse_line("foo arg1 arg2", location),
+			ConfigException);
 	}
 
 	SECTION("different combinations of (un)quoted commands/arguments have the same result") {
@@ -245,15 +248,42 @@ TEST_CASE("\"unbind-key -a\" removes all key bindings", "[ConfigParser]")
 	}
 }
 
-TEST_CASE("include directive includes other config files", "[ConfigParser]")
+TEST_CASE("Concatenates lines that end with a backslash", "[ConfigParser]")
+{
+	ConfigParser cfgparser;
+	KeyMap k(KM_NEWSBOAT);
+	cfgparser.register_handler("macro", k);
+	REQUIRE_NOTHROW(cfgparser.parse_file("data/config-multi-line"));
+	auto p_macro = k.get_macro("p");
+	REQUIRE(!p_macro.empty());
+	REQUIRE(p_macro[0].op == newsboat::OP_OPEN);
+	REQUIRE(p_macro[1].op == newsboat::OP_RELOAD);
+	REQUIRE(p_macro[2].op == newsboat::OP_QUIT);
+	REQUIRE(p_macro[3].op == newsboat::OP_QUIT);
+
+	auto contrived = k.get_macro("j");
+	REQUIRE(!contrived.empty());
+	REQUIRE(contrived[0].op == newsboat::OP_QUIT);
+	REQUIRE(contrived[1].op == newsboat::OP_QUIT);
+	REQUIRE(contrived[2].op == newsboat::OP_QUIT);
+	REQUIRE(contrived[3].op == newsboat::OP_QUIT);
+}
+
+TEST_CASE("`include` directive includes other config files", "[ConfigParser]")
 {
 	// TODO: error messages should be more descriptive than "file couldn't be opened"
 	ConfigParser cfgparser;
-	SECTION("Errors on not found file") {
-		REQUIRE_THROWS(cfgparser.parse_file("data/config-missing-include"));
+	SECTION("Errors if file is not found") {
+		REQUIRE_THROWS_AS(cfgparser.parse_file("data/config-missing-include"),
+			ConfigException);
+	}
+	SECTION("Errors on invalid UTF-8 in file") {
+		REQUIRE_THROWS_AS(cfgparser.parse_file("data/config-invalid-utf-8"),
+			ConfigException);
 	}
 	SECTION("Terminates on recursive include") {
-		REQUIRE_THROWS(cfgparser.parse_file("data/config-recursive-include"));
+		REQUIRE_THROWS_AS(cfgparser.parse_file("data/config-recursive-include"),
+			ConfigException);
 	}
 	SECTION("Successfully includes existing file") {
 		REQUIRE_NOTHROW(cfgparser.parse_file("data/config-absolute-include"));
@@ -265,8 +295,8 @@ TEST_CASE("include directive includes other config files", "[ConfigParser]")
 		REQUIRE_NOTHROW(cfgparser.parse_file("data/diamond-of-death/A"));
 	}
 	SECTION("File including itself only gets evaluated once") {
-		TestHelpers::TempFile testfile;
-		TestHelpers::EnvVar tmpfile("TMPFILE"); // $TMPFILE used in conf file
+		test_helpers::TempFile testfile;
+		test_helpers::EnvVar tmpfile("TMPFILE"); // $TMPFILE used in conf file
 		tmpfile.set(testfile.get_path());
 
 		// recursive includes don't fail
