@@ -1,6 +1,9 @@
 #include "utils.h"
 
+#include <algorithm>
 #include <chrono>
+#include <ctype.h>
+#include <fstream>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <tuple>
@@ -8,12 +11,11 @@
 
 #include "3rd-party/catch.hpp"
 #include "htmlrenderer.h"
-#include "rs_utils.h"
-#include "test-helpers/chdir.h"
-#include "test-helpers/envvar.h"
-#include "test-helpers/stringmaker/optional.h"
-#include "test-helpers/tempdir.h"
-#include "test-helpers/tempfile.h"
+#include "test_helpers/chdir.h"
+#include "test_helpers/envvar.h"
+#include "test_helpers/stringmaker/optional.h"
+#include "test_helpers/tempdir.h"
+#include "test_helpers/tempfile.h"
 
 using namespace newsboat;
 
@@ -343,6 +345,15 @@ TEST_CASE("extract_token_quoted() processes escape sequences within quoted strin
 	REQUIRE(str == " remainder");
 }
 
+TEST_CASE("extract_token_quoted() works with Unicode strings too", "[utils]")
+{
+	std::string str = R"("привет мир" Юникода)";
+	const auto token = utils::extract_token_quoted(str);
+	REQUIRE(token.has_value());
+	REQUIRE(token.value() == "привет мир");
+	REQUIRE(str == " Юникода");
+}
+
 TEST_CASE("tokenize_nl() split a string into delimiters and fields", "[utils]")
 {
 	std::vector<std::string> tokens;
@@ -373,7 +384,7 @@ TEST_CASE("tokenize_nl() split a string into delimiters and fields", "[utils]")
 	}
 
 	SECTION("custom delimiter") {
-		tokens = utils::tokenize_nl("first\nsecond\nthird","i");
+		tokens = utils::tokenize_nl("first\nsecond\nthird", "i");
 
 		REQUIRE(tokens.size() == 5);
 		REQUIRE(tokens[0] == "f");
@@ -530,34 +541,36 @@ TEST_CASE("get_command_output()", "[utils]")
 
 TEST_CASE("extract_filter()", "[utils]")
 {
-	std::string filter;
-	std::string url;
+	{
+		const auto parts =
+			utils::extract_filter("filter:~/bin/script.sh:https://newsboat.org");
+		REQUIRE(std::string(parts.script_name) == "~/bin/script.sh");
+		REQUIRE(std::string(parts.url) == "https://newsboat.org");
+	}
 
-	utils::extract_filter("filter:~/bin/script.sh:https://newsboat.org", filter,
-		url);
+	{
+		const auto parts = utils::extract_filter("filter::https://newsboat.org");
+		REQUIRE(std::string(parts.script_name) == "");
+		REQUIRE(std::string(parts.url) == "https://newsboat.org");
+	}
 
-	REQUIRE(filter == "~/bin/script.sh");
-	REQUIRE(url == "https://newsboat.org");
+	{
+		const auto parts = utils::extract_filter("filter:https://newsboat.org");
+		REQUIRE(std::string(parts.script_name) == "https");
+		REQUIRE(std::string(parts.url) == "//newsboat.org");
+	}
 
-	utils::extract_filter("filter::https://newsboat.org", filter, url);
+	{
+		const auto parts = utils::extract_filter("filter:foo:");
+		REQUIRE(std::string(parts.script_name) == "foo");
+		REQUIRE(std::string(parts.url) == "");
+	}
 
-	REQUIRE(filter == "");
-	REQUIRE(url == "https://newsboat.org");
-
-	utils::extract_filter("filter:https://newsboat.org", filter, url);
-
-	REQUIRE(filter == "https");
-	REQUIRE(url == "//newsboat.org");
-
-	utils::extract_filter("filter:foo:", filter, url);
-
-	REQUIRE(filter == "foo");
-	REQUIRE(url == "");
-
-	utils::extract_filter("filter:", filter, url);
-
-	REQUIRE(filter == "");
-	REQUIRE(url == "");
+	{
+		const auto parts = utils::extract_filter("filter:");
+		REQUIRE(std::string(parts.script_name) == "");
+		REQUIRE(std::string(parts.url) == "");
+	}
 }
 
 TEST_CASE("run_program()", "[utils]")
@@ -579,7 +592,7 @@ TEST_CASE("run_program()", "[utils]")
 TEST_CASE("run_command() executes the given command with a given argument",
 	"[utils]")
 {
-	TestHelpers::TempFile sentry;
+	test_helpers::TempFile sentry;
 	const auto argument = sentry.get_path();
 
 	{
@@ -634,10 +647,10 @@ TEST_CASE("run_command() doesn't wait for the command to finish",
 	REQUIRE(runtime.count() < 60000);
 }
 
-TEST_CASE("resolve_tilde() replaces ~ with the path to the $HOME directory",
+TEST_CASE("resolve_tilde() replaces ~ with the path to the home directory",
 	"[utils]")
 {
-	TestHelpers::EnvVar envVar("HOME");
+	test_helpers::EnvVar envVar("HOME");
 	envVar.set("test");
 	REQUIRE(utils::resolve_tilde("~") == "test/");
 	REQUIRE(utils::resolve_tilde("~/") == "test/");
@@ -675,6 +688,76 @@ TEST_CASE("replace_all()", "[utils]")
 	REQUIRE(utils::replace_all("o o o", "o", "<o>") == "<o> <o> <o>");
 }
 
+TEST_CASE("replace_all() with from/to pairs", "[utils]")
+{
+	SECTION("output == input if no from/to pairs are specified") {
+		const std::string input = "lorem ipsum";
+		REQUIRE(utils::replace_all("", {}) == "");
+		REQUIRE(utils::replace_all(input, {}) == input);
+	}
+
+	SECTION("supports multiple from/to pairs") {
+		const std::vector<std::pair<std::string, std::string>> rules = {
+			{"a", "x"},
+			{"bb", "y"},
+			{"c", "zz"}
+		};
+
+		SECTION("empty input results in empty output") {
+			REQUIRE(utils::replace_all("", rules) == "");
+		}
+
+		SECTION("a from/to pair can be matched multiple times") {
+			REQUIRE(utils::replace_all("aaa", rules) == "xxx");
+			REQUIRE(utils::replace_all("bbb", rules) == "yb");
+			REQUIRE(utils::replace_all("ccc", rules) == "zzzzzz");
+		}
+
+		SECTION("multiple from/to pairs can be matched in the same input") {
+			REQUIRE(utils::replace_all("abc cccbbbaaa", rules) == "xbzz zzzzzzybxxx");
+			REQUIRE(utils::replace_all("begin abc cccbbbaaa end",
+					rules) == "begin xbzz zzzzzzybxxx end");
+		}
+	}
+
+	SECTION("multiple from/to pairs do not create loop") {
+		const std::vector<std::pair<std::string, std::string>> rules = {
+			{"a", "b"},
+			{"b", "cc"},
+			{"cc", "z"}
+		};
+
+		REQUIRE(utils::replace_all("a", rules) == "b");
+		REQUIRE(utils::replace_all("aaa a", rules) == "bbb b");
+		REQUIRE(utils::replace_all("abc", rules) == "bccc");
+		REQUIRE(utils::replace_all("ccab", rules) == "zbcc");
+	}
+
+	SECTION("from/to pairs can start with common substring") {
+		const std::vector<std::pair<std::string, std::string>> rules = {
+			{"%u", "lorem"},
+			{"%F", "ipsum"},
+			{"%%", "%"}
+		};
+
+		REQUIRE(utils::replace_all("%u", rules) == "lorem");
+		REQUIRE(utils::replace_all("%F", rules) == "ipsum");
+		REQUIRE(utils::replace_all("%%", rules) == "%");
+		REQUIRE(utils::replace_all("%% %u %F", rules) == "% lorem ipsum");
+	}
+
+	SECTION("if multiple pairs match, first one is used") {
+		const std::vector<std::pair<std::string, std::string>> rules = {
+			{"test", "lorem"},
+			{"t", "ipsum"},
+			{"tester", "dolar"}
+		};
+
+		REQUIRE(utils::replace_all("tester", rules) == "loremer");
+		REQUIRE(utils::replace_all("ts", rules) == "ipsums");
+	}
+}
+
 TEST_CASE("to_string()", "[utils]")
 {
 	REQUIRE(std::to_string(0) == "0");
@@ -784,6 +867,27 @@ TEST_CASE("absolute_url()", "[utils]")
 			"bla2.html") == "http://test:test@foobar:33/bla2.html");
 }
 
+TEST_CASE("absolute_url() strips ASCII whitespace", "[utils]")
+{
+	// ASCII whitespace characters are: tab, line feed, form feed, carriage
+	// return, and space
+
+	REQUIRE(utils::absolute_url(" \t https://example.com/page1",
+			"page2") == "https://example.com/page2");
+	REQUIRE(utils::absolute_url("https://example.com/page3  \n ",
+			"page4") == "https://example.com/page4");
+	REQUIRE(utils::absolute_url("  \rhttps://example.com/page5  \f ",
+			"page6") == "https://example.com/page6");
+	REQUIRE(utils::absolute_url("https://example.com/base",
+			" \n replacement") == "https://example.com/replacement");
+	REQUIRE(utils::absolute_url("https://example.com/different",
+			"   another\t") == "https://example.com/another");
+	REQUIRE(utils::absolute_url("https://example.com/~joe/",
+			" \rhello\f") == "https://example.com/~joe/hello");
+	REQUIRE(utils::absolute_url( "\f\n\rhttps://example.com/misc/\t    ",
+			"   \t\feverything_at_once\n\r") == "https://example.com/misc/everything_at_once");
+}
+
 TEST_CASE("quote_for_stfl() adds a \'>\' after every \'<\'", "[utils]")
 {
 	REQUIRE(utils::quote_for_stfl("<<><><><") == "<><>><>><>><>");
@@ -810,7 +914,8 @@ TEST_CASE("strwidth()", "[utils]")
 
 	REQUIRE(utils::strwidth("xx") == 2);
 
-	REQUIRE(utils::strwidth(utils::wstr2str(L"\uF91F")) == 2);
+	const auto input1 = utils::wstr2str(L"\uF91F");
+	REQUIRE(utils::strwidth(input1) == 2);
 	REQUIRE(utils::strwidth("\07") == 0);
 }
 
@@ -828,12 +933,20 @@ TEST_CASE("strwidth_stfl()", "[utils]")
 
 	REQUIRE(utils::strwidth_stfl("x<>y<>z") == 5);
 
-	REQUIRE(utils::strwidth_stfl(utils::wstr2str(L"\uF91F")) == 2);
+	const auto input1 = utils::wstr2str(L"\uF91F");
+	REQUIRE(utils::strwidth_stfl(input1) == 2);
 	REQUIRE(utils::strwidth_stfl("\07") == 0);
+
+	REQUIRE(utils::strwidth_stfl("<a") == 0); // #415
+	REQUIRE(utils::strwidth_stfl("a") == 1);
+	REQUIRE(utils::strwidth_stfl("abc<tag>def") == 6);
+	REQUIRE(utils::strwidth_stfl("less-than: <>") ==  12);
+	REQUIRE(utils::strwidth_stfl("ＡＢＣＤＥＦ") == 12);
 }
 
 TEST_CASE("is_http_url()", "[utils]")
 {
+	REQUIRE(utils::is_http_url("http://example.com"));
 	REQUIRE(utils::is_http_url("http://foo.bar"));
 	REQUIRE(utils::is_http_url("https://foo.bar"));
 	REQUIRE(utils::is_http_url("http://"));
@@ -1008,6 +1121,48 @@ TEST_CASE("run_interactively runs a command with inherited I/O", "[utils]")
 	// `nonstd::nullopt`, nor to test that it returns just the lower 8 bits.
 }
 
+TEST_CASE("run_non_interactively runs a command without I/O", "[utils]")
+{
+	SECTION("echo hello should return 0") {
+		const auto result = utils::run_non_interactively("echo hello", "test");
+		REQUIRE(result == 0);
+	}
+	SECTION("exit 1 should return 1") {
+		const auto result = utils::run_non_interactively("exit 1", "test");
+		REQUIRE(result == 1);
+	}
+
+	// Unfortunately, there is no easy way to provoke this function to return
+	// `nonstd::nullopt`, nor to test that it returns just the lower 8 bits.
+}
+
+TEST_CASE("run_non_interactively waits for program to finish", "[utils]")
+{
+	using namespace std::chrono;
+
+	SECTION("when sleeping, we use at least as much time as specified") {
+		const auto start = steady_clock::now();
+
+		const auto result = utils::run_non_interactively("sleep 1", "test");
+		REQUIRE(result == 0);
+
+		const auto finish = steady_clock::now();
+		const auto runtime = duration_cast<milliseconds>(finish - start);
+		REQUIRE(runtime.count() >= 1000);
+	}
+
+	SECTION("if program is put in background, using &, we return immediately") {
+		const auto start = steady_clock::now();
+
+		const auto result = utils::run_non_interactively("sleep 5 &", "test");
+		REQUIRE(result == 0);
+
+		const auto finish = steady_clock::now();
+		const auto runtime = duration_cast<milliseconds>(finish - start);
+		REQUIRE(runtime.count() < 5000);
+	}
+}
+
 TEST_CASE("remove_soft_hyphens remove all U+00AD characters from a string",
 	"[utils]")
 {
@@ -1185,12 +1340,12 @@ TEST_CASE("getcwd() returns current directory of the process", "[utils]")
 	SECTION("Returns empty string if current directory doesn't exist") {
 		// Create a temporary directory, change to it and delete it. This leads
 		// getcwd to fail.
-		TestHelpers::TempDir tempdir;
+		test_helpers::TempDir tempdir;
 
 		const std::string tempdir_path = tempdir.get_path();
 		INFO("tempdir = " << tempdir_path);
 
-		TestHelpers::Chdir chdir(tempdir_path);
+		test_helpers::Chdir chdir(tempdir_path);
 
 		REQUIRE(0 == ::rmdir(tempdir_path.c_str()));
 
@@ -1198,33 +1353,66 @@ TEST_CASE("getcwd() returns current directory of the process", "[utils]")
 	}
 }
 
+TEST_CASE("read_text_file() returns file contents line by line", "[utils]")
+{
+	test_helpers::TempFile tempfile;
+
+	SECTION("succesful if test file contains only valid UTF-8") {
+		{
+			std::ofstream f(tempfile.get_path());
+			f << "lorem ipsum\ntest1\ntest2";
+		}
+
+		const auto result = utils::read_text_file(tempfile.get_path());
+		REQUIRE(result);
+		const auto content = result.value();
+		REQUIRE(content.size() == 3);
+		REQUIRE(content[0] == "lorem ipsum");
+		REQUIRE(content[1] == "test1");
+		REQUIRE(content[2] == "test2");
+	}
+
+	SECTION("fails with error message if file contains invalid UTF-8") {
+		{
+			std::ofstream f(tempfile.get_path());
+			f << "test1\nt\xffst2"; // \xff is an invalid UTF-8 codepoint
+		}
+		std::vector<std::string> content;
+		std::string error_message;
+
+		const auto result = utils::read_text_file(tempfile.get_path());
+		REQUIRE_FALSE(result);
+		REQUIRE(result.error().message.size() > 0);
+	}
+}
+
 TEST_CASE("strnaturalcmp() compares strings using natural numeric ordering",
 	"[utils]")
 {
 	// Tests copied over from 3rd-party/alphanum.hpp
-	REQUIRE(utils::strnaturalcmp("","") == 0);
-	REQUIRE(utils::strnaturalcmp("","a") < 0);
-	REQUIRE(utils::strnaturalcmp("a","") > 0);
-	REQUIRE(utils::strnaturalcmp("a","a") == 0);
-	REQUIRE(utils::strnaturalcmp("","9") < 0);
-	REQUIRE(utils::strnaturalcmp("9","") > 0);
-	REQUIRE(utils::strnaturalcmp("1","1") == 0);
-	REQUIRE(utils::strnaturalcmp("1","2") < 0);
-	REQUIRE(utils::strnaturalcmp("3","2") > 0);
-	REQUIRE(utils::strnaturalcmp("a1","a1") == 0);
-	REQUIRE(utils::strnaturalcmp("a1","a2") < 0);
-	REQUIRE(utils::strnaturalcmp("a2","a1") > 0);
-	REQUIRE(utils::strnaturalcmp("a1a2","a1a3") < 0);
-	REQUIRE(utils::strnaturalcmp("a1a2","a1a0") > 0);
-	REQUIRE(utils::strnaturalcmp("134","122") > 0);
-	REQUIRE(utils::strnaturalcmp("12a3","12a3") == 0);
-	REQUIRE(utils::strnaturalcmp("12a1","12a0") > 0);
-	REQUIRE(utils::strnaturalcmp("12a1","12a2") < 0);
-	REQUIRE(utils::strnaturalcmp("a","aa") < 0);
-	REQUIRE(utils::strnaturalcmp("aaa","aa") > 0);
-	REQUIRE(utils::strnaturalcmp("Alpha 2","Alpha 2") == 0);
-	REQUIRE(utils::strnaturalcmp("Alpha 2","Alpha 2A") < 0);
-	REQUIRE(utils::strnaturalcmp("Alpha 2 B","Alpha 2") > 0);
+	REQUIRE(utils::strnaturalcmp("", "") == 0);
+	REQUIRE(utils::strnaturalcmp("", "a") < 0);
+	REQUIRE(utils::strnaturalcmp("a", "") > 0);
+	REQUIRE(utils::strnaturalcmp("a", "a") == 0);
+	REQUIRE(utils::strnaturalcmp("", "9") < 0);
+	REQUIRE(utils::strnaturalcmp("9", "") > 0);
+	REQUIRE(utils::strnaturalcmp("1", "1") == 0);
+	REQUIRE(utils::strnaturalcmp("1", "2") < 0);
+	REQUIRE(utils::strnaturalcmp("3", "2") > 0);
+	REQUIRE(utils::strnaturalcmp("a1", "a1") == 0);
+	REQUIRE(utils::strnaturalcmp("a1", "a2") < 0);
+	REQUIRE(utils::strnaturalcmp("a2", "a1") > 0);
+	REQUIRE(utils::strnaturalcmp("a1a2", "a1a3") < 0);
+	REQUIRE(utils::strnaturalcmp("a1a2", "a1a0") > 0);
+	REQUIRE(utils::strnaturalcmp("134", "122") > 0);
+	REQUIRE(utils::strnaturalcmp("12a3", "12a3") == 0);
+	REQUIRE(utils::strnaturalcmp("12a1", "12a0") > 0);
+	REQUIRE(utils::strnaturalcmp("12a1", "12a2") < 0);
+	REQUIRE(utils::strnaturalcmp("a", "aa") < 0);
+	REQUIRE(utils::strnaturalcmp("aaa", "aa") > 0);
+	REQUIRE(utils::strnaturalcmp("Alpha 2", "Alpha 2") == 0);
+	REQUIRE(utils::strnaturalcmp("Alpha 2", "Alpha 2A") < 0);
+	REQUIRE(utils::strnaturalcmp("Alpha 2 B", "Alpha 2") > 0);
 
 	REQUIRE(utils::strnaturalcmp("aa10", "aa2") > 0);
 }
@@ -1350,7 +1538,7 @@ TEST_CASE(
 	"\"lynx\" if the variable is not set",
 	"[utils]")
 {
-	TestHelpers::EnvVar browserEnv("BROWSER");
+	test_helpers::EnvVar browserEnv("BROWSER");
 
 	// If BROWSER is not set, default browser is lynx(1)
 	browserEnv.unset();
@@ -1454,6 +1642,37 @@ TEST_CASE("unescape_url() takes a percent-encoded string and returns the string
 
 }
 
+TEST_CASE("preserve_quotes() preserves single quotes and double quotes, if any",
+	"[utils]")
+{
+	REQUIRE(utils::preserve_quotes("") == "''");
+	REQUIRE(utils::preserve_quotes("sss") == "'sss'");
+
+	SECTION("Preserves double quotes") {
+		REQUIRE(utils::preserve_quotes("\"sss\"") == "'\"sss\"'");
+		REQUIRE(utils::preserve_quotes("sss\"") == "'sss\"'");
+		REQUIRE(utils::preserve_quotes("\"sss") == "'\"sss'");
+		REQUIRE(utils::preserve_quotes("ss\"s") == "'ss\"s'");
+		REQUIRE(utils::preserve_quotes("ss\"\"s") == "'ss\"\"s'");
+	}
+
+	SECTION("Preserves single quotes") {
+		REQUIRE(utils::preserve_quotes("'sss'") == "\\\''sss'\\\'");
+		REQUIRE(utils::preserve_quotes("'sss") == "\\\''sss'");
+		REQUIRE(utils::preserve_quotes("sss'") == "'sss'\\\'");
+		REQUIRE(utils::preserve_quotes("ss's") == "'ss'\\\''s'");
+		REQUIRE(utils::preserve_quotes("ss''s") == "'ss'\\\'\\\''s'");
+	}
+
+	SECTION("Preserves a combination of single and double quotes") {
+		REQUIRE(utils::preserve_quotes("'\"sss\"'") == "\\\''\"sss\"'\\\'");
+		REQUIRE(utils::preserve_quotes("\"'sss'\"") == "'\"'\\\''sss'\\\''\"'");
+		REQUIRE(utils::preserve_quotes("\"sss'") == "'\"sss'\\\'");
+		REQUIRE(utils::preserve_quotes("'sss\"") == "\\\''sss\"'");
+		REQUIRE(utils::preserve_quotes("ss\"'s") == "'ss\"'\\\''s'");
+	}
+}
+
 TEST_CASE("gentabs() calculates padding tabs based on stringwidth", "[utils]")
 {
 	REQUIRE(utils::gentabs("") == 4);
@@ -1466,7 +1685,7 @@ TEST_CASE("mkdir_parents() creates all paths components and returns 0 if "
 	"the path now exists",
 	"[utils]")
 {
-	TestHelpers::TempDir tmp;
+	test_helpers::TempDir tmp;
 
 	const auto require_return_zero = [](const std::string& path) {
 		REQUIRE(utils::mkdir_parents(path, 0700) == 0);
@@ -1553,7 +1772,7 @@ TEST_CASE("mkdir_parents() creates all paths components and returns 0 if "
 TEST_CASE("mkdir_parents() doesn't care if the path ends in a slash or not",
 	"[utils]")
 {
-	TestHelpers::TempDir tmp;
+	test_helpers::TempDir tmp;
 
 	const auto path = tmp.get_path() + std::to_string(rand());
 
@@ -1570,3 +1789,226 @@ TEST_CASE("mkdir_parents() doesn't care if the path ends in a slash or not",
 		check(path + "/");
 	}
 }
+
+TEST_CASE("utf8_to_locale() converts text from UTF-8 to the encoding specified "
+	"by locale in LC_CTYPE class",
+	"[utils]")
+{
+	test_helpers::LcCtypeEnvVar lc_ctype;
+	const auto set_locale = [&lc_ctype](std::string new_locale) -> bool {
+		if (::setlocale(LC_CTYPE, new_locale.c_str()) == nullptr)
+		{
+			WARN("Couldn't set locale " + new_locale + "; test skipped.");
+			return false;
+		}
+		lc_ctype.set(new_locale);
+		return true;
+	};
+
+	SECTION("UTF-8") {
+		if (!set_locale("en_US.UTF-8")) {
+			return;
+		}
+
+		REQUIRE(utils::utf8_to_locale("") == "");
+
+		// "Просто проверяю", "Just testing" in Russian.
+		const std::string text("\xd0\x9f\xd1\x80\xd0\xbe\xd1\x81\xd1\x82\xd0"
+			"\xbe\x20\xd0\xbf\xd1\x80\xd0\xbe\xd0\xb2\xd0\xb5\xd1\x80\xd1\x8f\xd1\x8e");
+		REQUIRE(utils::utf8_to_locale(text) == text);
+	}
+
+	SECTION("KOI8-R") {
+		if (!set_locale("ru_RU.KOI8-R")) {
+			return;
+		}
+
+		REQUIRE(utils::utf8_to_locale("") == "");
+
+		// "ещё один тест", "another test" in Russian.
+		const std::string input("\xd0\xb5\xd1\x89\xd1\x91\x20\xd0\xbe\xd0\xb4"
+			"\xd0\xb8\xd0\xbd\x20\xd1\x82\xd0\xb5\xd1\x81\xd1\x82");
+		const std::string expected("\xc5\xdd\xa3\x20\xcf\xc4\xc9\xce\x20\xd4"
+			"\xc5\xd3\xd4");
+		REQUIRE(utils::utf8_to_locale(input) == expected);
+	}
+
+	SECTION("CP1251") {
+		if (!set_locale("ru_RU.CP1251")) {
+			return;
+		}
+
+		REQUIRE(utils::utf8_to_locale("") == "");
+
+		// "Приветствую!", "Greetings!" in Russian.
+		const std::string input("\xd0\x9f\xd1\x80\xd0\xb8\xd0\xb2\xd0\xb5\xd1"
+			"\x82\xd1\x81\xd1\x82\xd0\xb2\xd1\x83\xd1\x8e\x21");
+		const std::string expected("\xcf\xf0\xe8\xe2\xe5\xf2\xf1\xf2\xe2\xf3"
+			"\xfe\x21");
+		REQUIRE(utils::utf8_to_locale(input) == expected);
+	}
+}
+
+TEST_CASE("utf8_to_locale() transliterates characters unsupported by the locale's encoding",
+	"[utils]")
+{
+	test_helpers::LcCtypeEnvVar lc_ctype;
+	const auto set_locale = [&lc_ctype](std::string new_locale) -> bool {
+		if (::setlocale(LC_CTYPE, new_locale.c_str()) == nullptr)
+		{
+			WARN("Couldn't set locale " + new_locale + "; test skipped.");
+			return false;
+		}
+		lc_ctype.set(new_locale);
+		return true;
+	};
+
+	// Tests below don't spell out the expected result because different
+	// platforms might follow different transliteration rules. The best we can
+	// do is check that 1) the output is non-empty, i.e. there was no error in
+	// conversion; 2) the output is not the same as the input, i.e. the
+	// conversion did something.
+
+	SECTION("KOI8-R doesn't support Ukrainian characters") {
+		if (!set_locale("ru_RU.KOI8-R")) {
+			return;
+		}
+
+		// "Пісня", "Song" in Ukrainian.
+		const std::string input("\xd0\x9f\xd1\x96\xd1\x81\xd0\xbd\xd1\x8f");
+
+		const auto result = utils::utf8_to_locale(input);
+		REQUIRE(result != "");
+		REQUIRE(result != input);
+	}
+
+	SECTION("CP1251") {
+		if (!set_locale("ru_RU.CP1251")) {
+			return;
+		}
+
+		// "日本", "Japan" in Japanese
+		const std::string input("\xe6\x97\xa5\xe6\x9c\xac");
+
+		const auto result = utils::utf8_to_locale(input);
+		REQUIRE(result != "");
+		REQUIRE(result != input);
+	}
+}
+
+TEST_CASE("locale_to_utf8() converts text from the encoding specified by locale "
+	"in LC_CTYPE class to UTF-8",
+	"[utils]")
+{
+	test_helpers::LcCtypeEnvVar lc_ctype;
+	const auto set_locale = [&lc_ctype](std::string new_locale) -> bool {
+		if (::setlocale(LC_CTYPE, new_locale.c_str()) == nullptr)
+		{
+			WARN("Couldn't set locale " + new_locale + "; test skipped.");
+			return false;
+		}
+		lc_ctype.set(new_locale);
+		return true;
+	};
+
+	SECTION("UTF-8") {
+		if (!set_locale("en_US.UTF-8")) {
+			return;
+		}
+
+		REQUIRE(utils::locale_to_utf8("") == "");
+
+		// "Newsboat мне нравится", "I like Newsboat" in Russian.
+		const std::string text("\x4e\x65\x77\x73\x62\x6f\x61\x74\x20\xd0\xbc"
+			"\xd0\xbd\xd0\xb5\x20\xd0\xbd\xd1\x80\xd0\xb0\xd0\xb2\xd0\xb8"
+			"\xd1\x82\xd1\x81\xd1\x8f");
+		REQUIRE(utils::locale_to_utf8(text) == text);
+	}
+
+	SECTION("KOI8-R") {
+		if (!set_locale("ru_RU.KOI8-R")) {
+			return;
+		}
+
+		REQUIRE(utils::locale_to_utf8("") == "");
+
+		// "великолепная проверка", "excellent check" in Russian.
+		const std::string input("\xd7\xc5\xcc\xc9\xcb\xcf\xcc\xc5\xd0\xce\xc1"
+			"\xd1\x20\xd0\xd2\xcf\xd7\xc5\xd2\xcb\xc1");
+		const std::string expected("\xd0\xb2\xd0\xb5\xd0\xbb\xd0\xb8\xd0\xba\xd0"
+			"\xbe\xd0\xbb\xd0\xb5\xd0\xbf\xd0\xbd\xd0\xb0\xd1\x8f\x20\xd0"
+			"\xbf\xd1\x80\xd0\xbe\xd0\xb2\xd0\xb5\xd1\x80\xd0\xba\xd0\xb0");
+		REQUIRE(utils::locale_to_utf8(input) == expected);
+	}
+
+	SECTION("CP1251") {
+		if (!set_locale("ru_RU.CP1251")) {
+			return;
+		}
+
+		REQUIRE(utils::locale_to_utf8("") == "");
+
+		// "Все тесты зелёные!", "All tests green!" in Russian.
+		const std::string input("\xc2\xf1\xe5\x20\xf2\xe5\xf1\xf2\xfb\x20\xe7"
+			"\xe5\xeb\xb8\xed\xfb\xe5\x21");
+		const std::string expected("\xd0\x92\xd1\x81\xd0\xb5\x20\xd1\x82\xd0\xb5"
+			"\xd1\x81\xd1\x82\xd1\x8b\x20\xd0\xb7\xd0\xb5\xd0\xbb\xd1\x91\xd0"
+			"\xbd\xd1\x8b\xd0\xb5\x21");
+		REQUIRE(utils::locale_to_utf8(input) == expected);
+	}
+}
+
+TEST_CASE("translit() returns the value of `tocode`, maybe with \"//TRANSLIT\" appended",
+	"[utils]")
+{
+	// The behaviour of translit() is inherently platform-dependent: some
+	// platforms support transliteration for some encodings, while others
+	// support it for different encodings, or don't support it at all. This
+	// tests checks just the bare minimum: the function doesn't crash, doesn't
+	// throw exceptions, and returns something resembling a correct value.
+
+	const auto check = [](std::string fromcode, std::string tocode) {
+		DYNAMIC_SECTION(fromcode << " → " << tocode) {
+			const auto expected1(tocode);
+			const auto expected2(tocode + "//TRANSLIT");
+
+			const auto actual = utils::translit(tocode, fromcode);
+
+			INFO("Expected " << expected1);
+			INFO("      or " << expected2);
+			INFO("But got: " << actual);
+
+			// Extra parentheses required by Catch2 -- otherwise it doesn't let us use operator||.
+			REQUIRE((actual == expected1 || actual == expected2));
+		}
+	};
+
+	check("ISO-8859-1", "UTF-8");
+	check("KOI8-R", "UTF-8");
+	check("UTF-16", "UTF-8");
+	check("UTF-8", "UTF-8");
+	check("UTF-8", "UTF-16");
+	check("UTF-8", "KOI8-R");
+	check("UTF-8", "ISO-8859-1");
+}
+
+TEST_CASE("translit() always returns the same value for the same inputs",
+	"[utils]")
+{
+	const std::vector<std::string> encodings {"UTF-8", "UTF-16", "KOI8-R", "ISO-8859-1"};
+	// (fromcode, tocode) -> result
+	std::map<std::pair<std::string, std::string>, std::string> results;
+
+	for (const auto& fromcode : encodings) {
+		for (const auto& tocode : encodings) {
+			results[std::make_pair(fromcode, tocode)] = utils::translit(tocode, fromcode);
+		}
+	}
+
+	for (const auto& fromcode : encodings) {
+		for (const auto& tocode : encodings) {
+			INFO(fromcode << " → " << tocode);
+			REQUIRE(utils::translit(tocode, fromcode) == results.at(std::make_pair(fromcode, tocode)));
+		}
+	}
+}
